c*****PWIA
c
c     This program returns the necessary helicity matrix elements as 
c     calculated in the plane wave impluse approximation.
c
c
c     INPUT
c          ql............................3-momentum transfer in the 
c                                        lab frame
c          omegal........................energy transfer in lab frame
c          w.............................invariant mass of the final state
c          thetac........................polar angle of ejected nucleon
c                                        in the frame of the calculation
c          phic..........................azimuthal angle (usually set to 0)
c          iff...........................code to choose form factor type
c          iexch.........................1 for exchange term, 0 without
c
c     OUTPUT
c          curme(lnuc1,lnuc2,lgam,ldeu)..array containing current matrix elements
c
c          lnuc1=0.........positive nucleon helicity for particle 1
c               =1.........negative nucleon helicity for particle 1
c          lnuc2=0.........positive nucleon helicity for particle 2
c               =1.........negative nucleon helicity for particle 2
c          lgam=-1,0,1........photon helicity
c          ldeu=-1,0,1.....deuteron helicity
c
c          
      subroutine pwia(ql,omegal,w,thetac,phic,iff,iexch,curme)
c
      implicit real*8(a-h,o-z)
c
      complex*16 curme(0:1,0:1,0:1,-1:1)
c
      dimension sja(0:1,0:1,0:1,0:1,0:1,0:1),
     &          sjb(0:1,0:1,0:1,0:1,0:1,0:1),
     &          aba(0:1,0:1,0:1,-1:1,0:1),abb(0:1,0:1,0:1,-1:1,0:1),
     &          f(0:1,2)
c
      real*8 mn,mt
      common/dpara/mn,mt
c
      parameter(pi=3.141592653589793)
      parameter(root2=1.414213562373095,root2i=1./root2)
c
      fac=sqrt(2.*mt)  ! This factor is due to the normalization of the wave
                       ! function to 1 rather than 2.*M_T
c
c                       calculate boost parameters
c
      shztaz=-ql/w
      chztaz=sqrt(shztaz**2+1.)
      ch2ztaz=chztaz**2+shztaz**2
      sh2ztaz=2.*shztaz*chztaz
c
c                       calculate cm momenta and angles
c
      pc=sqrt(0.25*w**2-mn**2)
      epc=0.5*w
      sh=pc/mn
      ch=epc/mn
c      write(6,*)pc
      qc=chztaz*ql+shztaz*omegal
      omegac=chztaz*omegal+shztaz*ql
c
c               calculate square of four-momentum transfer
c
      q2=qc**2-omegac**2
      q2gev=q2*1.e-6
c
c                     call form factor routine
c
      call dircff(q2gev,f,iff)
c
c            calculate momentum and angles for struck nucleon
c
      csthc=cos(thetac)
c
      ch1=ch2ztaz*ch+sh2ztaz*sh*csthc
      sh1=sqrt(ch1**2-1.)
      csth1=(ch2ztaz*sh*csthc+sh2ztaz*ch)/sh1
      theta1=acos(polar_range(csth1))
      p1=mn*sh1
c      
c
      ch2=ch2ztaz*ch-sh2ztaz*sh*csthc
      sh2=sqrt(ch2**2-1.)
      csth2=(ch2ztaz*sh*csthc-sh2ztaz*ch)/sh2
      theta2=acos(polar_range(csth2))
      p2=mn*sh2
c
c              obtain current operators and boosted wave functions
c
      call currnt(qc,omegac,pc,thetac,p1,theta1,0,1,sja)
      call currnt(qc,omegac,pc,thetac,p2,theta2,1,1,sjb)
      call boost(pc,thetac,shztaz,chztaz,0,1,aba)
      call boost(pc,thetac,shztaz,chztaz,1,1,abb)
c
c                         construct matrix elements
c
c                 Note: The factor root2i is an isospin factor
c
      do ldeu=-1,1
        do lgam=0,1
          do lnuc2=0,1
            do lnuc1=0,1
              curme(lnuc1,lnuc2,lgam,ldeu)=(0.,0.)
              do nucp=0,1
                do ienrgy=0,1
                  curme(lnuc1,lnuc2,lgam,ldeu)=
     &                 curme(lnuc1,lnuc2,lgam,ldeu)
     &                 +fac*(                                   !root2i*(
     &                  (f(0,1)*sja(lnuc1,nucp,0,ienrgy,lgam,0)
     &                 + f(0,2)*sja(lnuc1,nucp,0,ienrgy,lgam,1))
     &                 *aba(nucp,lnuc2,ienrgy,-ldeu,0)
     &                 +dfloat(iexch)
     &                  *(f(1,1)*sjb(lnuc2,nucp,0,ienrgy,lgam,0)
     &                 + f(1,2)*sjb(lnuc2,nucp,0,ienrgy,lgam,1))
     &                 *abb(lnuc1,nucp,ienrgy,-ldeu,0) !there is a problem here
     &                  )
                end do
              end do
            end do
          end do
        end do
      end do
c
      return
      end
