(************** Content-type: application/mathematica **************
                     CreatedBy='Mathematica 5.0'

                    Mathematica-Compatible Notebook

This notebook can be used with any Mathematica-compatible
application, such as Mathematica, MathReader or Publicon. The data
for the notebook starts with the line containing stars above.

To get the notebook into a Mathematica-compatible application, do
one of the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the
  application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing
the word CacheID, otherwise Mathematica-compatible applications may
try to use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
*******************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[    259066,       6323]*)
(*NotebookOutlinePosition[    260021,       6352]*)
(*  CellTagsIndexPosition[    259977,       6348]*)
(*WindowFrame->Normal*)



Notebook[{
Cell["\<\
Notebook for evaluating the average value of the RC for comparison \
with non-exclusive calculations as a test of EXCLURAD.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[{
    \(<< Graphics`Graphics3D`\), "\n", 
    \(<< NumericalMath`NIntegrateInterpolatingFunct`\), "\n", 
    \($DefaultFont = {"\<Times\>", 16}\)}], "Input"],

Cell[BoxData[
    \({"Times", 16}\)], "Output"]
}, Open  ]],

Cell["\<\
Read in the big RC from exclurad, check the length for later use, \
and set the number of points used in the phi_pq direction.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[{
    \(\(data1\  = \ 
        ReadList["\</home/gilfoyle/tmp/plots/radcorW255Q2300c.dat\>", \
\[IndentingNewLine]{Number, Number, Number, Number, Number, Number, Number, 
            Number}];\)\), "\n", 
    \(\(nphis\  = \ 13;\)\), "\n", 
    \(length\  = \ Length[data1]\)}], "Input"],

Cell[BoxData[
    \(208\)], "Output"]
}, Open  ]],

Cell["Construct the table and plot it as a check.", "Text"],

Cell[BoxData[
    \(\(RC1\  = \ 
        Table[\ Table[{data1[\([i + j, 4]\)], data1[\([i + j, 5]\)], 
              data1[\([i + j, 
                  6]\)]}, \[IndentingNewLine]\ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \
\ \ \ \ \ {i, 1, 13}], \[IndentingNewLine]\ \ \ \ \ \ \ \ \ \ \ \ \ \ {j, 0, 
            length - nphis, nphis}];\)\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(p1\  = \ 
      ListSurfacePlot3D[
        RC1, \[IndentingNewLine]BoxRatios \[Rule] {1, 1, 
            1.0}, \[IndentingNewLine]Axes \[Rule] 
          True, \[IndentingNewLine]AxesLabel \[Rule] {"\<cos\[Theta]\>", "\<\
\[Phi](deg)\>", \*"\"\<\!\(RC\_\(\(\\\ \\\ \)\(p\)\)\)\>\""}, \
\[IndentingNewLine]PlotRange \[Rule] {{0.85, 1}, {0, 180}, {0.7, 
              1.75}}, \[IndentingNewLine]FaceGrids \[Rule] {{\(-1\), 0, 
              0}, {0, 1, 0}, {0, 0, \(-1\)}, {1, 0, 0}, {0, \(-1\), 
              0}}, \[IndentingNewLine]PlotLabel \[Rule] \*"\"\<E = 4.232GeV, \
W = 2.55GeV, \!\(Q\^\(\(\\\ \)\(2\)\)\)=3.0\!\(GeV\^2\), vcut=0.310GeV\>\"", \
\[IndentingNewLine]ImageSize \[Rule] {72*7.0, 
            72*7.0}, \[IndentingNewLine]AxesEdge \[Rule] {Automatic, {1, \(-1\
\)}, Automatic}]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: 1.0855 
%%ImageSize: 504 504 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics3D
%%IncludeResource: font Times
%%IncludeFont: Times
/Times findfont 16  scalefont  setfont
% Scaling calculations
-0.0567234 1.16608 2.03993e-16 1.16608 [
[.5 1.098 -250.719 0 ]
[.5 1.098 250.719 17.6875 ]
[.07825 .23514 -41.8414 -14.5625 ]
[.07825 .23514 .40861 0 ]
[.25892 .15883 -30.3723 -14.5625 ]
[.25892 .15883 2.31522 0 ]
[.45386 .07654 -36.6736 -14.5625 ]
[.45386 .07654 5.57644 0 ]
[.66483 -0.01242 -10.943 -14.5625 ]
[.66483 -0.01242 2.61952 0 ]
[.30907 .06148 -37.9655 -14.5625 ]
[.30907 .06148 4.28448 0 ]
[.68589 -0.00565 0 -10.3428 ]
[.68589 -0.00565 13.5625 4.21973 ]
[.76766 .12771 0 -10.1183 ]
[.76766 .12771 23.125 4.4442 ]
[.84029 .24617 0 -9.92449 ]
[.84029 .24617 32.6875 4.63801 ]
[.90522 .3521 0 -9.75547 ]
[.90522 .3521 32.6875 4.80703 ]
[.89481 .19838 0 -10.2604 ]
[.89481 .19838 61.375 4.73963 ]
[.07147 .27344 -42.25 -4.25157 ]
[.07147 .27344 0 10.3109 ]
[.0528 .39025 -13.5625 -4.41957 ]
[.0528 .39025 0 10.1429 ]
[.03251 .51731 -42.25 -4.60381 ]
[.03251 .51731 0 9.95869 ]
[.01035 .65601 -32.6875 -4.80677 ]
[.01035 .65601 0 9.75573 ]
[-0.01393 .80804 -42.25 -5.03145 ]
[-0.01393 .80804 0 9.53105 ]
[-0.03373 .52941 -37.1875 -5.09617 ]
[-0.03373 .52941 0 11.0913 ]
[ 0 0 0 0 ]
[ 1 1.0855 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
gsave
.5 1.098 -311.719 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 25.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding ISOLatin1Encoding def
        currentdict
    end

    newfontname exch definefont pop
} def
63.000 18.188 moveto
%%IncludeResource: font Times
%%IncludeFont: Times
%%BeginResource: font Times-MISO
%%BeginFont: Times-MISO
/Times /Times-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Times-MISO
%%IncludeFont: Times-MISO
/Times-MISO findfont 16.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 18.188 moveto
%%IncludeResource: font Times-MISO
%%IncludeFont: Times-MISO
/Times-MISO findfont 16.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(E) show
82.125 18.188 moveto
%%IncludeResource: font Mathematica1Mono
%%IncludeFont: Mathematica1Mono
/Mathematica1Mono findfont 16.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(=) show
101.250 18.188 moveto
%%IncludeResource: font Times-MISO
%%IncludeFont: Times-MISO
/Times-MISO findfont 16.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(4.232) show
149.062 18.188 moveto
(GeV) show
177.750 18.188 moveto
%%IncludeResource: font Times-MISO
%%IncludeFont: Times-MISO
/Times-MISO findfont 16.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(,) show
196.875 18.188 moveto
%%IncludeResource: font Times-MISO
%%IncludeFont: Times-MISO
/Times-MISO findfont 16.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(W) show
216.000 18.188 moveto
%%IncludeResource: font Mathematica1Mono
%%IncludeFont: Mathematica1Mono
/Mathematica1Mono findfont 16.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(=) show
235.125 18.188 moveto
%%IncludeResource: font Times-MISO
%%IncludeFont: Times-MISO
/Times-MISO findfont 16.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(2.55) show
273.375 18.188 moveto
(GeV) show
302.062 18.188 moveto
%%IncludeResource: font Times-MISO
%%IncludeFont: Times-MISO
/Times-MISO findfont 16.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(,) show
321.188 18.188 moveto
%%IncludeResource: font Times-MISO
%%IncludeFont: Times-MISO
/Times-MISO findfont 16.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(Q) show
333.688 12.062 moveto
%%IncludeResource: font Times-MISO
%%IncludeFont: Times-MISO
/Times-MISO findfont 11.375 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(2) show
341.875 18.188 moveto
%%IncludeResource: font Mathematica1Mono
%%IncludeFont: Mathematica1Mono
/Mathematica1Mono findfont 16.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(=) show
351.438 18.188 moveto
%%IncludeResource: font Times-MISO
%%IncludeFont: Times-MISO
/Times-MISO findfont 16.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(3.0) show
380.125 18.188 moveto
(GeV) show
408.812 12.062 moveto
%%IncludeResource: font Times-MISO
%%IncludeFont: Times-MISO
/Times-MISO findfont 11.375 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(2) show
417.000 18.188 moveto
%%IncludeResource: font Times-MISO
%%IncludeFont: Times-MISO
/Times-MISO findfont 16.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(,) show
436.125 18.188 moveto
%%IncludeResource: font Times-MISO
%%IncludeFont: Times-MISO
/Times-MISO findfont 16.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(vcut) show
474.375 18.188 moveto
%%IncludeResource: font Mathematica1Mono
%%IncludeFont: Mathematica1Mono
/Mathematica1Mono findfont 16.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(=) show
483.938 18.188 moveto
%%IncludeResource: font Times-MISO
%%IncludeFont: Times-MISO
/Times-MISO findfont 16.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(0.310) show
531.750 18.188 moveto
(GeV) show
560.438 18.188 moveto
%%IncludeResource: font Times-MISO
%%IncludeFont: Times-MISO
/Times-MISO findfont 16.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
.25 Mabswid
[ ] 0 setdash
.08846 .24555 m
.67245 0 L
s
.08846 .24555 m
.09356 .25076 L
s
gsave
.07825 .23514 -102.841 -18.5625 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 22.5625 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.062 moveto
%%IncludeResource: font Times
%%IncludeFont: Times
/Times findfont 16.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(0.85) show
1.000 setlinewidth
grestore
.26841 .16989 m
.27316 .17542 L
s
gsave
.25892 .15883 -91.3723 -18.5625 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 22.5625 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.062 moveto
%%IncludeResource: font Times
%%IncludeFont: Times
/Times findfont 16.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(0.9) show
1.000 setlinewidth
grestore
.4625 .08828 m
.46682 .09415 L
s
gsave
.45386 .07654 -97.6736 -18.5625 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 22.5625 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.062 moveto
%%IncludeResource: font Times
%%IncludeFont: Times
/Times findfont 16.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(0.95) show
1.000 setlinewidth
grestore
.67245 0 m
.67627 .00621 L
s
gsave
.66483 -0.01242 -71.943 -18.5625 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 22.5625 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.062 moveto
%%IncludeResource: font Times
%%IncludeFont: Times
/Times findfont 16.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(1) show
1.000 setlinewidth
grestore
.125 Mabswid
.12339 .23086 m
.12641 .23402 L
s
.15884 .21596 m
.16182 .21916 L
s
.19482 .20083 m
.19776 .20407 L
s
.23134 .18547 m
.23423 .18875 L
s
.30604 .15406 m
.30885 .15742 L
s
.34426 .138 m
.34701 .14139 L
s
.38306 .12168 m
.38576 .12512 L
s
.42247 .10511 m
.42512 .10859 L
s
.50316 .07118 m
.50569 .07475 L
s
.54447 .05381 m
.54695 .05742 L
s
.58645 .03616 m
.58886 .03981 L
s
.6291 .01823 m
.63145 .02191 L
s
gsave
.30907 .06148 -98.9655 -18.5625 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 22.5625 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding ISOLatin1Encoding def
        currentdict
    end

    newfontname exch definefont pop
} def
63.000 15.062 moveto
%%IncludeResource: font Times
%%IncludeFont: Times
%%BeginResource: font Times-MISO
%%BeginFont: Times-MISO
/Times /Times-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Times-MISO
%%IncludeFont: Times-MISO
/Times-MISO findfont 16.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 15.062 moveto
%%IncludeResource: font Times-MISO
%%IncludeFont: Times-MISO
/Times-MISO findfont 16.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(cos) show
%%IncludeResource: font Mathematica1Mono
%%IncludeFont: Mathematica1Mono
/Mathematica1Mono findfont 16.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(q) show
101.250 15.062 moveto
%%IncludeResource: font Times-MISO
%%IncludeFont: Times-MISO
/Times-MISO findfont 16.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
.25 Mabswid
.67245 0 m
.92713 .41497 L
s
.67245 0 m
.66574 .00282 L
s
gsave
.68589 -0.00565 -61 -14.3428 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 22.5625 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.062 moveto
%%IncludeResource: font Times
%%IncludeFont: Times
/Times findfont 16.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(0) show
1.000 setlinewidth
grestore
.75408 .133 m
.74729 .13565 L
s
gsave
.76766 .12771 -61 -14.1183 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 22.5625 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.062 moveto
%%IncludeResource: font Times
%%IncludeFont: Times
/Times findfont 16.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(50) show
1.000 setlinewidth
grestore
.82659 .25114 m
.81973 .25363 L
s
gsave
.84029 .24617 -61 -13.9245 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 22.5625 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.062 moveto
%%IncludeResource: font Times
%%IncludeFont: Times
/Times findfont 16.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(100) show
1.000 setlinewidth
grestore
.89142 .35678 m
.88452 .35913 L
s
gsave
.90522 .3521 -61 -13.7555 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 22.5625 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.062 moveto
%%IncludeResource: font Times
%%IncludeFont: Times
/Times findfont 16.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(150) show
1.000 setlinewidth
grestore
.125 Mabswid
.68959 .02792 m
.68555 .02959 L
s
.70631 .05516 m
.70226 .05681 L
s
.72262 .08174 m
.71856 .08337 L
s
.73854 .10768 m
.73447 .10928 L
s
.76926 .15774 m
.76518 .1593 L
s
.78409 .1819 m
.78 .18345 L
s
.79858 .20551 m
.79448 .20704 L
s
.81274 .22858 m
.80864 .23009 L
s
.84012 .27321 m
.83601 .27468 L
s
.85337 .29479 m
.84925 .29624 L
s
.86633 .3159 m
.8622 .31734 L
s
.87901 .33656 m
.87487 .33798 L
s
.90357 .37659 m
.89942 .37798 L
s
.91547 .39598 m
.91132 .39735 L
s
gsave
.89481 .19838 -61 -14.2604 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding ISOLatin1Encoding def
        currentdict
    end

    newfontname exch definefont pop
} def
63.000 15.062 moveto
%%IncludeResource: font Times
%%IncludeFont: Times
%%BeginResource: font Times-MISO
%%BeginFont: Times-MISO
/Times /Times-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Times-MISO
%%IncludeFont: Times-MISO
/Times-MISO findfont 16.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 15.062 moveto
%%IncludeResource: font Mathematica1Mono
%%IncludeFont: Mathematica1Mono
/Mathematica1Mono findfont 16.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(f) show
%%IncludeResource: font Mathematica2Mono
%%IncludeFont: Mathematica2Mono
/Mathematica2Mono findfont 16.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
72.562 15.062 moveto
(H) show
82.125 15.062 moveto
%%IncludeResource: font Times-MISO
%%IncludeFont: Times-MISO
/Times-MISO findfont 16.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(deg) show
%%IncludeResource: font Mathematica2Mono
%%IncludeFont: Mathematica2Mono
/Mathematica2Mono findfont 16.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
110.812 15.062 moveto
(L) show
120.375 15.062 moveto
%%IncludeResource: font Times-MISO
%%IncludeFont: Times-MISO
/Times-MISO findfont 16.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
.25 Mabswid
.08846 .24555 m
0 .80374 L
s
.08492 .26784 m
.09165 .26504 L
s
gsave
.07147 .27344 -103.25 -8.25157 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 22.5625 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.062 moveto
%%IncludeResource: font Times
%%IncludeFont: Times
/Times findfont 16.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(0.75) show
1.000 setlinewidth
grestore
.06637 .38492 m
.07315 .38226 L
s
gsave
.0528 .39025 -74.5625 -8.41957 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 22.5625 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.062 moveto
%%IncludeResource: font Times
%%IncludeFont: Times
/Times findfont 16.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(1) show
1.000 setlinewidth
grestore
.04619 .51228 m
.05303 .50976 L
s
gsave
.03251 .51731 -103.25 -8.60381 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 22.5625 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.062 moveto
%%IncludeResource: font Times
%%IncludeFont: Times
/Times findfont 16.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(1.25) show
1.000 setlinewidth
grestore
.02415 .65132 m
.03105 .64897 L
s
gsave
.01035 .65601 -93.6875 -8.80677 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 22.5625 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.062 moveto
%%IncludeResource: font Times
%%IncludeFont: Times
/Times findfont 16.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(1.5) show
1.000 setlinewidth
grestore
0 .80374 m
.00696 .80159 L
s
gsave
-0.01393 .80804 -103.25 -9.03145 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 22.5625 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.062 moveto
%%IncludeResource: font Times
%%IncludeFont: Times
/Times findfont 16.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(1.75) show
1.000 setlinewidth
grestore
.125 Mabswid
.08133 .2905 m
.08538 .28883 L
s
.07768 .31352 m
.08174 .31187 L
s
.07398 .33693 m
.07803 .3353 L
s
.0702 .36073 m
.07427 .35911 L
s
.06247 .40953 m
.06655 .40795 L
s
.0585 .43456 m
.06259 .43299 L
s
.05447 .46002 m
.05856 .45847 L
s
.05037 .48592 m
.05446 .48439 L
s
.04194 .5391 m
.04605 .53761 L
s
.03761 .5664 m
.04173 .56493 L
s
.03321 .5942 m
.03733 .59275 L
s
.02872 .6225 m
.03285 .62107 L
s
.0195 .68067 m
.02365 .67929 L
s
.01476 .71058 m
.01892 .70921 L
s
.00994 .74104 m
.0141 .73971 L
s
.00501 .77209 m
.00919 .77078 L
s
.08846 .24555 m
.09249 .24386 L
s
gsave
-0.03373 .52941 -98.1875 -9.09617 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 24.1875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding ISOLatin1Encoding def
        currentdict
    end

    newfontname exch definefont pop
} def
63.000 15.062 moveto
%%IncludeResource: font Times
%%IncludeFont: Times
%%BeginResource: font Times-MISO
%%BeginFont: Times-MISO
/Times /Times-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Times-MISO
%%IncludeFont: Times-MISO
/Times-MISO findfont 16.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 15.062 moveto
%%IncludeResource: font Times-MISO
%%IncludeFont: Times-MISO
/Times-MISO findfont 16.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(RC) show
88.000 17.438 moveto
%%IncludeResource: font Times-MISO
%%IncludeFont: Times-MISO
/Times-MISO findfont 11.375 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(p) show
96.188 15.062 moveto
%%IncludeResource: font Times-MISO
%%IncludeFont: Times-MISO
/Times-MISO findfont 16.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
.4 g
.25 Mabswid
.08492 .26784 m
.41726 .60236 L
s
.06637 .38492 m
.41348 .71058 L
s
.04619 .51228 m
.40943 .82657 L
s
.02415 .65132 m
.40507 .95121 L
s
0 .80374 m
.40037 1.0855 L
s
.08846 .24555 m
0 .80374 L
s
.19262 .35177 m
.12946 .89484 L
s
.28629 .44728 m
.24355 .97513 L
s
.37097 .53364 m
.34486 1.04643 L
s
0 g
.08846 .24555 m
0 .80374 L
s
0 .80374 m
.40037 1.0855 L
s
.40037 1.0855 m
.41799 .58158 L
s
.41799 .58158 m
.08846 .24555 L
s
.4 g
.67245 0 m
.70644 .58546 L
s
.75408 .133 m
.80324 .70505 L
s
.82659 .25114 m
.88704 .80858 L
s
.89142 .35678 m
.9603 .8991 L
s
.67378 .02282 m
.93008 .43661 L
s
.68078 .14345 m
.94555 .54979 L
s
.68848 .27609 m
.96226 .67199 L
s
.69699 .42266 m
.98035 .80434 L
s
.70644 .58546 m
1 .94814 L
s
0 g
.67245 0 m
.92713 .41497 L
s
.92713 .41497 m
1 .94814 L
s
1 .94814 m
.70644 .58546 L
s
.70644 .58546 m
.67245 0 L
s
.4 g
.08492 .26784 m
.67378 .02282 L
s
.06637 .38492 m
.68078 .14345 L
s
.04619 .51228 m
.68848 .27609 L
s
.02415 .65132 m
.69699 .42266 L
s
0 .80374 m
.70644 .58546 L
s
.08846 .24555 m
0 .80374 L
s
.26841 .16989 m
.21392 .73764 L
s
.4625 .08828 m
.44837 .6652 L
s
.67245 0 m
.70644 .58546 L
s
0 g
.08846 .24555 m
0 .80374 L
s
0 .80374 m
.70644 .58546 L
s
.70644 .58546 m
.67245 0 L
s
.67245 0 m
.08846 .24555 L
s
.4 g
.41799 .58158 m
.40037 1.0855 L
s
.57709 .52952 m
.58565 1.04305 L
s
.74646 .47409 m
.78497 .9974 L
s
.92713 .41497 m
1 .94814 L
s
.41726 .60236 m
.93008 .43661 L
s
.41348 .71058 m
.94555 .54979 L
s
.40943 .82657 m
.96226 .67199 L
s
.40507 .95121 m
.98035 .80434 L
s
.40037 1.0855 m
1 .94814 L
s
0 g
.41799 .58158 m
.92713 .41497 L
s
.92713 .41497 m
1 .94814 L
s
1 .94814 m
.40037 1.0855 L
s
.40037 1.0855 m
.41799 .58158 L
s
.4 g
.08846 .24555 m
.41799 .58158 L
s
.26841 .16989 m
.57709 .52952 L
s
.4625 .08828 m
.74646 .47409 L
s
.67245 0 m
.92713 .41497 L
s
.08846 .24555 m
.67245 0 L
s
.19262 .35177 m
.75408 .133 L
s
.28629 .44728 m
.82659 .25114 L
s
.37097 .53364 m
.89142 .35678 L
s
0 0 m
1 0 L
1 1.0855 L
0 1.0855 L
closepath
clip
newpath
.5 Mabswid
.582 .662 .893 r
.38894 .69532 m .41298 .72475 L .44526 .72225 L p F P
0 g
s
.549 .641 .893 r
.42145 .69148 m .44526 .72225 L .47806 .72233 L p F P
0 g
s
.582 .662 .893 r
.44526 .72225 m .42145 .69148 L .38894 .69532 L p F P
0 g
s
.602 .619 .841 r
.36469 .65687 m .38894 .69532 L .42145 .69148 L p F P
0 g
s
.508 .612 .891 r
.45447 .68984 m .47806 .72233 L .51147 .72557 L p F P
0 g
s
.549 .641 .893 r
.47806 .72233 m .45447 .68984 L .42145 .69148 L p F P
0 g
s
.581 .599 .835 r
.39743 .65048 m .42145 .69148 L .45447 .68984 L p F P
0 g
s
.602 .619 .841 r
.42145 .69148 m .39743 .65048 L .36469 .65687 L p F P
0 g
s
.623 .624 .831 r
.33999 .61695 m .36469 .65687 L .39743 .65048 L p F P
0 g
s
.464 .578 .884 r
.48806 .69091 m .51147 .72557 L .54562 .73232 L p F P
0 g
s
.508 .612 .891 r
.51147 .72557 m .48806 .69091 L .45447 .68984 L p F P
0 g
s
.557 .573 .826 r
.43063 .64555 m .45447 .68984 L .48806 .69091 L p F P
0 g
s
.581 .599 .835 r
.45447 .68984 m .43063 .64555 L .39743 .65048 L p F P
0 g
s
.609 .609 .826 r
.37295 .60832 m .39743 .65048 L .43063 .64555 L p F P
0 g
s
.623 .624 .831 r
.39743 .65048 m .37295 .60832 L .33999 .61695 L p F P
0 g
s
.64 .644 .839 r
.3146 .57878 m .33999 .61695 L .37295 .60832 L p F P
0 g
s
.557 .573 .826 r
.48806 .69091 m .46435 .64253 L .43063 .64555 L p F P
0 g
s
.592 .591 .82 r
.40636 .60057 m .43063 .64555 L .46435 .64253 L p F P
0 g
s
.609 .609 .826 r
.43063 .64555 m .40636 .60057 L .37295 .60832 L p F P
0 g
s
.631 .634 .835 r
.34782 .56856 m .37295 .60832 L .40636 .60057 L p F P
0 g
s
.64 .644 .839 r
.37295 .60832 m .34782 .56856 L .3146 .57878 L p F P
0 g
s
.464 .578 .884 r
.54562 .73232 m .52234 .69511 L .48806 .69091 L p F P
0 g
s
.532 .545 .814 r
.46435 .64253 m .48806 .69091 L .52234 .69511 L p F P
0 g
s
.416 .538 .872 r
.52234 .69511 m .54562 .73232 L .58065 .74311 L p F P
0 g
s
.652 .663 .848 r
.28844 .54242 m .3146 .57878 L .34782 .56856 L p F P
0 g
s
.592 .591 .82 r
.46435 .64253 m .44025 .59385 L .40636 .60057 L p F P
0 g
s
.62 .622 .831 r
.38147 .55878 m .40636 .60057 L .44025 .59385 L p F P
0 g
s
.631 .634 .835 r
.40636 .60057 m .38147 .55878 L .34782 .56856 L p F P
0 g
s
.532 .545 .814 r
.52234 .69511 m .49867 .64154 L .46435 .64253 L p F P
0 g
s
.575 .569 .809 r
.44025 .59385 m .46435 .64253 L .49867 .64154 L p F P
0 g
s
.647 .658 .847 r
.32191 .53122 m .34782 .56856 L .38147 .55878 L p F P
0 g
s
.652 .663 .848 r
.34782 .56856 m .32191 .53122 L .28844 .54242 L p F P
0 g
s
.416 .538 .872 r
.58065 .74311 m .55743 .7027 L .52234 .69511 L p F P
0 g
s
.509 .514 .797 r
.49867 .64154 m .52234 .69511 L .55743 .7027 L p F P
0 g
s
.379 .5 .856 r
.55743 .7027 m .58065 .74311 L .61671 .7578 L p F P
0 g
s
.575 .569 .809 r
.49867 .64154 m .47467 .58831 L .44025 .59385 L p F P
0 g
s
.609 .607 .824 r
.41559 .5495 m .44025 .59385 L .47467 .58831 L p F P
0 g
s
.62 .622 .831 r
.44025 .59385 m .41559 .5495 L .38147 .55878 L p F P
0 g
s
.659 .678 .857 r
.26145 .50738 m .28844 .54242 L .32191 .53122 L p F P
0 g
s
.509 .514 .797 r
.55743 .7027 m .53367 .64284 L .49867 .64154 L p F P
0 g
s
.558 .545 .796 r
.47467 .58831 m .49867 .64154 L .53367 .64284 L p F P
0 g
s
.641 .65 .844 r
.35584 .5201 m .38147 .55878 L .41559 .5495 L p F P
0 g
s
.647 .658 .847 r
.38147 .55878 m .35584 .5201 L .32191 .53122 L p F P
0 g
s
.96 .691 .48 r
.93066 .44085 m .91299 .41236 L .91089 .72691 L p F P
0 g
s
.96 .691 .48 r
.91089 .72691 m .93662 .80421 L .93066 .44085 L p F P
0 g
s
.657 .675 .856 r
.29519 .49548 m .32191 .53122 L .35584 .5201 L p F P
0 g
s
.659 .678 .857 r
.32191 .53122 m .29519 .49548 L .26145 .50738 L p F P
0 g
s
.379 .5 .856 r
.61671 .7578 m .59346 .71354 L .55743 .7027 L p F P
0 g
s
.492 .487 .779 r
.53367 .64284 m .55743 .7027 L .59346 .71354 L p F P
0 g
s
.558 .545 .796 r
.53367 .64284 m .50968 .58397 L .47467 .58831 L p F P
0 g
s
.598 .59 .814 r
.4502 .54076 m .47467 .58831 L .50968 .58397 L p F P
0 g
s
.609 .607 .824 r
.47467 .58831 m .4502 .54076 L .41559 .5495 L p F P
0 g
s
.634 .641 .839 r
.39022 .5091 m .41559 .5495 L .4502 .54076 L p F P
0 g
s
.641 .65 .844 r
.41559 .5495 m .39022 .5091 L .35584 .5201 L p F P
0 g
s
.361 .472 .838 r
.59346 .71354 m .61671 .7578 L .65394 .7757 L p F P
0 g
s
.492 .487 .779 r
.59346 .71354 m .56945 .64608 L .53367 .64284 L p F P
0 g
s
.547 .522 .78 r
.50968 .58397 m .53367 .64284 L .56945 .64608 L p F P
0 g
s
.664 .691 .865 r
.23356 .47332 m .26145 .50738 L .29519 .49548 L p F P
0 g
s
.654 .671 .854 r
.32941 .48341 m .35584 .5201 L .39022 .5091 L p F P
0 g
s
.657 .675 .856 r
.35584 .5201 m .32941 .48341 L .29519 .49548 L p F P
0 g
s
.598 .59 .814 r
.50968 .58397 m .48534 .53258 L .4502 .54076 L p F P
0 g
s
.628 .629 .833 r
.42509 .49821 m .4502 .54076 L .48534 .53258 L p F P
0 g
s
.634 .641 .839 r
.4502 .54076 m .42509 .49821 L .39022 .5091 L p F P
0 g
s
.547 .522 .78 r
.56945 .64608 m .54534 .58063 L .50968 .58397 L p F P
0 g
s
.59 .573 .802 r
.48534 .53258 m .50968 .58397 L .54534 .58063 L p F P
0 g
s
.921 .658 .523 r
.91299 .41236 m .89474 .38296 L .88009 .60525 L p F P
0 g
s
.921 .658 .523 r
.88009 .60525 m .91089 .72691 L .91299 .41236 L p F P
0 g
s
.664 .689 .864 r
.2676 .46079 m .29519 .49548 L .32941 .48341 L p F P
0 g
s
.664 .691 .865 r
.29519 .49548 m .2676 .46079 L .23356 .47332 L p F P
0 g
s
.651 .666 .851 r
.36409 .47127 m .39022 .5091 L .42509 .49821 L p F P
0 g
s
.654 .671 .854 r
.39022 .5091 m .36409 .47127 L .32941 .48341 L p F P
0 g
s
.361 .472 .838 r
.65394 .7757 m .63055 .72706 L .59346 .71354 L p F P
0 g
s
.486 .465 .759 r
.56945 .64608 m .59346 .71354 L .63055 .72706 L p F P
0 g
s
.486 .465 .759 r
.63055 .72706 m .6061 .65085 L .56945 .64608 L p F P
0 g
s
.541 .503 .763 r
.54534 .58063 m .56945 .64608 L .6061 .65085 L p F P
0 g
s
.59 .573 .802 r
.54534 .58063 m .52106 .52473 L .48534 .53258 L p F P
0 g
s
.623 .617 .824 r
.46047 .48744 m .48534 .53258 L .52106 .52473 L p F P
0 g
s
.628 .629 .833 r
.48534 .53258 m .46047 .48744 L .42509 .49821 L p F P
0 g
s
.663 .687 .863 r
.30211 .44804 m .32941 .48341 L .36409 .47127 L p F P
0 g
s
.664 .689 .864 r
.32941 .48341 m .30211 .44804 L .2676 .46079 L p F P
0 g
s
.667 .701 .872 r
.20474 .43976 m .23356 .47332 L .2676 .46079 L p F P
0 g
s
.541 .503 .763 r
.6061 .65085 m .5817 .57794 L .54534 .58063 L p F P
0 g
s
.585 .556 .789 r
.52106 .52473 m .54534 .58063 L .5817 .57794 L p F P
0 g
s
.367 .457 .82 r
.63055 .72706 m .65394 .7757 L .69241 .79565 L p F P
0 g
s
.648 .659 .847 r
.39925 .45891 m .42509 .49821 L .46047 .48744 L p F P
0 g
s
.651 .666 .851 r
.42509 .49821 m .39925 .45891 L .36409 .47127 L p F P
0 g
s
.668 .7 .871 r
.23908 .42672 m .2676 .46079 L .30211 .44804 L p F P
0 g
s
.667 .701 .872 r
.2676 .46079 m .23908 .42672 L .20474 .43976 L p F P
0 g
s
.663 .684 .86 r
.33712 .43498 m .36409 .47127 L .39925 .45891 L p F P
0 g
s
.663 .687 .863 r
.36409 .47127 m .33712 .43498 L .30211 .44804 L p F P
0 g
s
.585 .556 .789 r
.5817 .57794 m .55737 .51703 L .52106 .52473 L p F P
0 g
s
.62 .605 .814 r
.49637 .47664 m .52106 .52473 L .55737 .51703 L p F P
0 g
s
.623 .617 .824 r
.52106 .52473 m .49637 .47664 L .46047 .48744 L p F P
0 g
s
.906 .658 .553 r
.89474 .38296 m .87591 .35261 L .85163 .5013 L p F P
0 g
s
.906 .658 .553 r
.85163 .5013 m .88009 .60525 L .89474 .38296 L p F P
0 g
s
.646 .652 .841 r
.43492 .44643 m .46047 .48744 L .49637 .47664 L p F P
0 g
s
.648 .659 .847 r
.46047 .48744 m .43492 .44643 L .39925 .45891 L p F P
0 g
s
.367 .457 .82 r
.69241 .79565 m .66878 .74203 L .63055 .72706 L p F P
0 g
s
.491 .451 .74 r
.6061 .65085 m .63055 .72706 L .66878 .74203 L p F P
0 g
s
.491 .451 .74 r
.66878 .74203 m .64365 .65627 L .6061 .65085 L p F P
0 g
s
.542 .488 .745 r
.5817 .57794 m .6061 .65085 L .64365 .65627 L p F P
0 g
s
.542 .488 .745 r
.64365 .65627 m .61878 .57534 L .5817 .57794 L p F P
0 g
s
.584 .543 .774 r
.55737 .51703 m .5817 .57794 L .61878 .57534 L p F P
0 g
s
.669 .7 .87 r
.27391 .41341 m .30211 .44804 L .33712 .43498 L p F P
0 g
s
.668 .7 .871 r
.30211 .44804 m .27391 .41341 L .23908 .42672 L p F P
0 g
s
.662 .681 .857 r
.37261 .4216 m .39925 .45891 L .43492 .44643 L p F P
0 g
s
.663 .684 .86 r
.39925 .45891 m .37261 .4216 L .33712 .43498 L p F P
0 g
s
.669 .709 .878 r
.17491 .40653 m .20474 .43976 L .23908 .42672 L p F P
0 g
s
.584 .543 .774 r
.61878 .57534 m .59429 .50902 L .55737 .51703 L p F P
0 g
s
.62 .595 .803 r
.53281 .46568 m .55737 .51703 L .59429 .50902 L p F P
0 g
s
.62 .605 .814 r
.55737 .51703 m .53281 .46568 L .49637 .47664 L p F P
0 g
s
.645 .643 .834 r
.47109 .43363 m .49637 .47664 L .53281 .46568 L p F P
0 g
s
.646 .652 .841 r
.49637 .47664 m .47109 .43363 L .43492 .44643 L p F P
0 g
s
.399 .458 .805 r
.66878 .74203 m .69241 .79565 L .7321 .81557 L p F P
0 g
s
.551 .48 .729 r
.6821 .66098 m .65658 .57196 L .61878 .57534 L p F P
0 g
s
.551 .48 .729 r
.61878 .57534 m .64365 .65627 L .6821 .66098 L p F P
0 g
s
.589 .532 .759 r
.59429 .50902 m .61878 .57534 L .65658 .57196 L p F P
0 g
s
.67 .698 .868 r
.30925 .39964 m .33712 .43498 L .37261 .4216 L p F P
0 g
s
.669 .7 .87 r
.33712 .43498 m .30925 .39964 L .27391 .41341 L p F P
0 g
s
.662 .676 .853 r
.4086 .40785 m .43492 .44643 L .47109 .43363 L p F P
0 g
s
.662 .681 .857 r
.43492 .44643 m .4086 .40785 L .37261 .4216 L p F P
0 g
s
.67 .709 .877 r
.20957 .39302 m .23908 .42672 L .27391 .41341 L p F P
0 g
s
.669 .709 .878 r
.23908 .42672 m .20957 .39302 L .17491 .40653 L p F P
0 g
s
.507 .446 .722 r
.70812 .75658 m .6821 .66098 L .64365 .65627 L p F P
0 g
s
.507 .446 .722 r
.64365 .65627 m .66878 .74203 L .70812 .75658 L p F P
0 g
s
.589 .532 .759 r
.65658 .57196 m .63183 .50044 L .59429 .50902 L p F P
0 g
s
.621 .585 .792 r
.5698 .45428 m .59429 .50902 L .63183 .50044 L p F P
0 g
s
.62 .595 .803 r
.59429 .50902 m .5698 .45428 L .53281 .46568 L p F P
0 g
s
.399 .458 .805 r
.7321 .81557 m .70812 .75658 L .66878 .74203 L p F P
0 g
s
.646 .636 .826 r
.50778 .42046 m .53281 .46568 L .5698 .45428 L p F P
0 g
s
.645 .643 .834 r
.53281 .46568 m .50778 .42046 L .47109 .43363 L p F P
0 g
s
.896 .672 .591 r
.87591 .35261 m .85645 .32125 L .82592 .4217 L p F P
0 g
s
.896 .672 .591 r
.82592 .4217 m .85163 .5013 L .87591 .35261 L p F P
0 g
s
.671 .696 .865 r
.34509 .38545 m .37261 .4216 L .4086 .40785 L p F P
0 g
s
.67 .698 .868 r
.37261 .4216 m .34509 .38545 L .30925 .39964 L p F P
0 g
s
.662 .671 .848 r
.4451 .39359 m .47109 .43363 L .50778 .42046 L p F P
0 g
s
.662 .676 .853 r
.47109 .43363 m .4451 .39359 L .4086 .40785 L p F P
0 g
s
.672 .709 .875 r
.24474 .37913 m .27391 .41341 L .30925 .39964 L p F P
0 g
s
.67 .709 .877 r
.27391 .41341 m .24474 .37913 L .20957 .39302 L p F P
0 g
s
.671 .717 .883 r
.14401 .37338 m .17491 .40653 L .20957 .39302 L p F P
0 g
s
.565 .477 .715 r
.72134 .66334 m .69504 .56682 L .65658 .57196 L p F P
0 g
s
.565 .477 .715 r
.65658 .57196 m .6821 .66098 L .72134 .66334 L p F P
0 g
s
.597 .527 .746 r
.63183 .50044 m .65658 .57196 L .69504 .56682 L p F P
0 g
s
.621 .585 .792 r
.63183 .50044 m .60735 .44233 L .5698 .45428 L p F P
0 g
s
.647 .628 .818 r
.54498 .40683 m .5698 .45428 L .60735 .44233 L p F P
0 g
s
.646 .636 .826 r
.5698 .45428 m .54498 .40683 L .50778 .42046 L p F P
0 g
s
.597 .527 .746 r
.69504 .56682 m .66994 .49053 L .63183 .50044 L p F P
0 g
s
.627 .579 .781 r
.60735 .44233 m .63183 .50044 L .66994 .49053 L p F P
0 g
s
.452 .473 .79 r
.70812 .75658 m .7321 .81557 L .77283 .83243 L p F P
0 g
s
.529 .447 .708 r
.74845 .76858 m .72134 .66334 L .6821 .66098 L p F P
0 g
s
.529 .447 .708 r
.6821 .66098 m .70812 .75658 L .74845 .76858 L p F P
0 g
s
.673 .694 .862 r
.38145 .3707 m .4086 .40785 L .4451 .39359 L p F P
0 g
s
.671 .696 .865 r
.4086 .40785 m .38145 .3707 L .34509 .38545 L p F P
0 g
s
.663 .665 .842 r
.48211 .37886 m .50778 .42046 L .54498 .40683 L p F P
0 g
s
.662 .671 .848 r
.50778 .42046 m .48211 .37886 L .4451 .39359 L p F P
0 g
s
.674 .709 .874 r
.28043 .36477 m .30925 .39964 L .34509 .38545 L p F P
0 g
s
.672 .709 .875 r
.30925 .39964 m .28043 .36477 L .24474 .37913 L p F P
0 g
s
.672 .717 .882 r
.17901 .35942 m .20957 .39302 L .24474 .37913 L p F P
0 g
s
.671 .717 .883 r
.20957 .39302 m .17901 .35942 L .14401 .37338 L p F P
0 g
s
.452 .473 .79 r
.77283 .83243 m .74845 .76858 L .70812 .75658 L p F P
0 g
s
.627 .579 .781 r
.66994 .49053 m .64543 .42945 L .60735 .44233 L p F P
0 g
s
.65 .623 .809 r
.58271 .39264 m .60735 .44233 L .64543 .42945 L p F P
0 g
s
.647 .628 .818 r
.60735 .44233 m .58271 .39264 L .54498 .40683 L p F P
0 g
s
.582 .48 .703 r
.7612 .66173 m .73404 .55889 L .69504 .56682 L p F P
0 g
s
.582 .48 .703 r
.69504 .56682 m .72134 .66334 L .7612 .66173 L p F P
0 g
s
.609 .526 .735 r
.66994 .49053 m .69504 .56682 L .73404 .55889 L p F P
0 g
s
.609 .526 .735 r
.73404 .55889 m .70857 .4787 L .66994 .49053 L p F P
0 g
s
.636 .576 .771 r
.64543 .42945 m .66994 .49053 L .70857 .4787 L p F P
0 g
s
.881 .689 .639 r
.85645 .32125 m .83633 .28883 L .80173 .35814 L p F P
0 g
s
.881 .689 .639 r
.80173 .35814 m .82592 .4217 L .85645 .32125 L p F P
0 g
s
.674 .691 .858 r
.41832 .35538 m .4451 .39359 L .48211 .37886 L p F P
0 g
s
.673 .694 .862 r
.4451 .39359 m .41832 .35538 L .38145 .3707 L p F P
0 g
s
.677 .708 .871 r
.31664 .34989 m .34509 .38545 L .38145 .3707 L p F P
0 g
s
.674 .709 .874 r
.34509 .38545 m .31664 .34989 L .28043 .36477 L p F P
0 g
s
.665 .66 .836 r
.51962 .3636 m .54498 .40683 L .58271 .39264 L p F P
0 g
s
.663 .665 .842 r
.54498 .40683 m .51962 .3636 L .48211 .37886 L p F P
0 g
s
.675 .718 .881 r
.21453 .34502 m .24474 .37913 L .28043 .36477 L p F P
0 g
s
.672 .717 .882 r
.24474 .37913 m .21453 .34502 L .17901 .35942 L p F P
0 g
s
.553 .453 .696 r
.78954 .77587 m .7612 .66173 L .72134 .66334 L p F P
0 g
s
.553 .453 .696 r
.72134 .66334 m .74845 .76858 L .78954 .77587 L p F P
0 g
s
.672 .724 .888 r
.11198 .34023 m .14401 .37338 L .17901 .35942 L p F P
0 g
s
.636 .576 .771 r
.70857 .4787 m .68401 .41526 L .64543 .42945 L p F P
0 g
s
.656 .62 .801 r
.62095 .37773 m .64543 .42945 L .68401 .41526 L p F P
0 g
s
.65 .623 .809 r
.64543 .42945 m .62095 .37773 L .58271 .39264 L p F P
0 g
s
.624 .528 .726 r
.77344 .54728 m .74761 .4645 L .70857 .4787 L p F P
0 g
s
.624 .528 .726 r
.70857 .4787 m .73404 .55889 L .77344 .54728 L p F P
0 g
s
.646 .577 .763 r
.68401 .41526 m .70857 .4787 L .74761 .4645 L p F P
0 g
s
.599 .484 .694 r
.80147 .65511 m .77344 .54728 L .73404 .55889 L p F P
0 g
s
.599 .484 .694 r
.73404 .55889 m .7612 .66173 L .80147 .65511 L p F P
0 g
s
.676 .688 .853 r
.45569 .33943 m .48211 .37886 L .51962 .3636 L p F P
0 g
s
.674 .691 .858 r
.48211 .37886 m .45569 .33943 L .41832 .35538 L p F P
0 g
s
.498 .489 .777 r
.74845 .76858 m .77283 .83243 L .81444 .84514 L p F P
0 g
s
.68 .707 .868 r
.35339 .33434 m .38145 .3707 L .41832 .35538 L p F P
0 g
s
.677 .708 .871 r
.38145 .3707 m .35339 .33434 L .31664 .34989 L p F P
0 g
s
.667 .656 .83 r
.55763 .34782 m .58271 .39264 L .62095 .37773 L p F P
0 g
s
.665 .66 .836 r
.58271 .39264 m .55763 .34782 L .51962 .3636 L p F P
0 g
s
.678 .718 .879 r
.25059 .3301 m .28043 .36477 L .31664 .34989 L p F P
0 g
s
.675 .718 .881 r
.28043 .36477 m .25059 .3301 L .21453 .34502 L p F P
0 g
s
.674 .724 .887 r
.14733 .32575 m .17901 .35942 L .21453 .34502 L p F P
0 g
s
.672 .724 .888 r
.17901 .35942 m .14733 .32575 L .11198 .34023 L p F P
0 g
s
.498 .489 .777 r
.81444 .84514 m .78954 .77587 L .74845 .76858 L p F P
0 g
s
.646 .577 .763 r
.74761 .4645 m .72303 .39951 L .68401 .41526 L p F P
0 g
s
.663 .619 .795 r
.65967 .36198 m .68401 .41526 L .72303 .39951 L p F P
0 g
s
.656 .62 .801 r
.68401 .41526 m .65967 .36198 L .62095 .37773 L p F P
0 g
s
.642 .536 .718 r
.81296 .5305 m .78691 .44691 L .74761 .4645 L p F P
0 g
s
.642 .536 .718 r
.74761 .4645 m .77344 .54728 L .81296 .5305 L p F P
0 g
s
.66 .582 .756 r
.72303 .39951 m .74761 .4645 L .78691 .44691 L p F P
0 g
s
.678 .684 .849 r
.49357 .32296 m .51962 .3636 L .55763 .34782 L p F P
0 g
s
.676 .688 .853 r
.51962 .3636 m .49357 .32296 L .45569 .33943 L p F P
0 g
s
.575 .46 .685 r
.83114 .77725 m .80147 .65511 L .7612 .66173 L p F P
0 g
s
.575 .46 .685 r
.7612 .66173 m .78954 .77587 L .83114 .77725 L p F P
0 g
s
.621 .494 .686 r
.84174 .64118 m .81296 .5305 L .77344 .54728 L p F P
0 g
s
.621 .494 .686 r
.77344 .54728 m .80147 .65511 L .84174 .64118 L p F P
0 g
s
.683 .706 .865 r
.39065 .31807 m .41832 .35538 L .45569 .33943 L p F P
0 g
s
.68 .707 .868 r
.41832 .35538 m .39065 .31807 L .35339 .33434 L p F P
0 g
s
.671 .653 .824 r
.59615 .3315 m .62095 .37773 L .65967 .36198 L p F P
0 g
s
.667 .656 .83 r
.62095 .37773 m .59615 .3315 L .55763 .34782 L p F P
0 g
s
.682 .718 .877 r
.28719 .31455 m .31664 .34989 L .35339 .33434 L p F P
0 g
s
.678 .718 .879 r
.31664 .34989 m .28719 .31455 L .25059 .3301 L p F P
0 g
s
.856 .704 .693 r
.83633 .28883 m .81553 .2553 L .77816 .30407 L p F P
0 g
s
.856 .704 .693 r
.77816 .30407 m .80173 .35814 L .83633 .28883 L p F P
0 g
s
.677 .724 .886 r
.18321 .31083 m .21453 .34502 L .25059 .3301 L p F P
0 g
s
.674 .724 .887 r
.21453 .34502 m .18321 .31083 L .14733 .32575 L p F P
0 g
s
.66 .582 .756 r
.78691 .44691 m .76239 .38145 L .72303 .39951 L p F P
0 g
s
.674 .623 .789 r
.69887 .34513 m .72303 .39951 L .76239 .38145 L p F P
0 g
s
.663 .619 .795 r
.72303 .39951 m .69887 .34513 L .65967 .36198 L p F P
0 g
s
.685 .56 .708 r
.85163 .5013 m .82592 .4217 L .78691 .44691 L p F P
0 g
s
.685 .56 .708 r
.78691 .44691 m .81296 .5305 L .85163 .5013 L p F P
0 g
s
.694 .6 .747 r
.76239 .38145 m .78691 .44691 L .82592 .4217 L p F P
0 g
s
.679 .681 .845 r
.53194 .30594 m .55763 .34782 L .59615 .3315 L p F P
0 g
s
.678 .684 .849 r
.55763 .34782 m .53194 .30594 L .49357 .32296 L p F P
0 g
s
.678 .527 .673 r
.88009 .60525 m .85163 .5013 L .81296 .5305 L p F P
0 g
s
.678 .527 .673 r
.81296 .5305 m .84174 .64118 L .88009 .60525 L p F P
0 g
s
.673 .73 .893 r
.07873 .30695 m .11198 .34023 L .14733 .32575 L p F P
0 g
s
.685 .704 .862 r
.42843 .30122 m .45569 .33943 L .49357 .32296 L p F P
0 g
s
.683 .706 .865 r
.45569 .33943 m .42843 .30122 L .39065 .31807 L p F P
0 g
s
.545 .505 .762 r
.78954 .77587 m .81444 .84514 L .8566 .85168 L p F P
0 g
s
.676 .652 .819 r
.63515 .31451 m .65967 .36198 L .69887 .34513 L p F P
0 g
s
.671 .653 .824 r
.65967 .36198 m .63515 .31451 L .59615 .3315 L p F P
0 g
s
.694 .6 .747 r
.82592 .4217 m .80173 .35814 L .76239 .38145 L p F P
0 g
s
.686 .718 .874 r
.32434 .29832 m .35339 .33434 L .39065 .31807 L p F P
0 g
s
.682 .718 .877 r
.35339 .33434 m .32434 .29832 L .28719 .31455 L p F P
0 g
s
.6 .471 .676 r
.87276 .76986 m .84174 .64118 L .80147 .65511 L p F P
0 g
s
.6 .471 .676 r
.80147 .65511 m .83114 .77725 L .87276 .76986 L p F P
0 g
s
.701 .636 .78 r
.73845 .32654 m .76239 .38145 L .80173 .35814 L p F P
0 g
s
.674 .623 .789 r
.76239 .38145 m .73845 .32654 L .69887 .34513 L p F P
0 g
s
.545 .505 .762 r
.8566 .85168 m .83114 .77725 L .78954 .77587 L p F P
0 g
s
.681 .726 .884 r
.21965 .29541 m .25059 .3301 L .28719 .31455 L p F P
0 g
s
.677 .724 .886 r
.25059 .3301 m .21965 .29541 L .18321 .31083 L p F P
0 g
s
.682 .679 .841 r
.57081 .28847 m .59615 .3315 L .63515 .31451 L p F P
0 g
s
.679 .681 .845 r
.59615 .3315 m .57081 .28847 L .53194 .30594 L p F P
0 g
s
.687 .702 .858 r
.46672 .28373 m .49357 .32296 L .53194 .30594 L p F P
0 g
s
.685 .704 .862 r
.49357 .32296 m .46672 .28373 L .42843 .30122 L p F P
0 g
s
.675 .731 .892 r
.11443 .29197 m .14733 .32575 L .18321 .31083 L p F P
0 g
s
.673 .73 .893 r
.14733 .32575 m .11443 .29197 L .07873 .30695 L p F P
0 g
s
.685 .655 .814 r
.67463 .29679 m .69887 .34513 L .73845 .32654 L p F P
0 g
s
.676 .652 .819 r
.69887 .34513 m .67463 .29679 L .63515 .31451 L p F P
0 g
s
.701 .636 .78 r
.80173 .35814 m .77816 .30407 L .73845 .32654 L p F P
0 g
s
.688 .718 .871 r
.36203 .28132 m .39065 .31807 L .42843 .30122 L p F P
0 g
s
.686 .718 .874 r
.39065 .31807 m .36203 .28132 L .32434 .29832 L p F P
0 g
s
.68 .517 .659 r
.91089 .72691 m .88009 .60525 L .84174 .64118 L p F P
0 g
s
.68 .517 .659 r
.84174 .64118 m .87276 .76986 L .91089 .72691 L p F P
0 g
s
.685 .726 .882 r
.25666 .27926 m .28719 .31455 L .32434 .29832 L p F P
0 g
s
.681 .726 .884 r
.28719 .31455 m .25666 .27926 L .21965 .29541 L p F P
0 g
s
.685 .678 .837 r
.61017 .2705 m .63515 .31451 L .67463 .29679 L p F P
0 g
s
.682 .679 .841 r
.63515 .31451 m .61017 .2705 L .57081 .28847 L p F P
0 g
s
.821 .713 .746 r
.81553 .2553 m .794 .22061 L .75467 .25537 L p F P
0 g
s
.821 .713 .746 r
.75467 .25537 m .77816 .30407 L .81553 .2553 L p F P
0 g
s
.707 .665 .806 r
.71453 .27769 m .73845 .32654 L .77816 .30407 L p F P
0 g
s
.685 .655 .814 r
.73845 .32654 m .71453 .27769 L .67463 .29679 L p F P
0 g
s
.591 .523 .747 r
.83114 .77725 m .8566 .85168 L .89887 .84971 L p F P
0 g
s
.688 .7 .856 r
.50551 .26573 m .53194 .30594 L .57081 .28847 L p F P
0 g
s
.687 .702 .858 r
.53194 .30594 m .50551 .26573 L .46672 .28373 L p F P
0 g
s
.678 .732 .891 r
.15069 .27659 m .18321 .31083 L .21965 .29541 L p F P
0 g
s
.675 .731 .892 r
.18321 .31083 m .15069 .27659 L .11443 .29197 L p F P
0 g
s
.591 .523 .747 r
.89887 .84971 m .87276 .76986 L .83114 .77725 L p F P
0 g
s
.691 .717 .868 r
.40025 .26362 m .42843 .30122 L .46672 .28373 L p F P
0 g
s
.688 .718 .871 r
.42843 .30122 m .40025 .26362 L .36203 .28132 L p F P
0 g
s
.707 .665 .806 r
.77816 .30407 m .75467 .25537 L .71453 .27769 L p F P
0 g
s
.689 .727 .879 r
.29423 .26242 m .32434 .29832 L .36203 .28132 L p F P
0 g
s
.685 .726 .882 r
.32434 .29832 m .29423 .26242 L .25666 .27926 L p F P
0 g
s
.692 .679 .833 r
.65003 .25198 m .67463 .29679 L .71453 .27769 L p F P
0 g
s
.685 .678 .837 r
.67463 .29679 m .65003 .25198 L .61017 .2705 L p F P
0 g
s
.689 .698 .853 r
.54479 .24732 m .57081 .28847 L .61017 .2705 L p F P
0 g
s
.688 .7 .856 r
.57081 .28847 m .54479 .24732 L .50551 .26573 L p F P
0 g
s
.737 .597 .701 r
.93662 .80421 m .91089 .72691 L .87276 .76986 L p F P
0 g
s
.737 .597 .701 r
.87276 .76986 m .89887 .84971 L .93662 .80421 L p F P
0 g
s
.683 .732 .888 r
.18754 .26056 m .21965 .29541 L .25666 .27926 L p F P
0 g
s
.678 .732 .891 r
.21965 .29541 m .18754 .26056 L .15069 .27659 L p F P
0 g
s
.693 .714 .865 r
.43899 .24514 m .46672 .28373 L .50551 .26573 L p F P
0 g
s
.691 .717 .868 r
.46672 .28373 m .43899 .24514 L .40025 .26362 L p F P
0 g
s
.711 .687 .825 r
.69033 .2323 m .71453 .27769 L .75467 .25537 L p F P
0 g
s
.692 .679 .833 r
.71453 .27769 m .69033 .2323 L .65003 .25198 L p F P
0 g
s
.693 .727 .877 r
.33237 .24475 m .36203 .28132 L .40025 .26362 L p F P
0 g
s
.689 .727 .879 r
.36203 .28132 m .33237 .24475 L .29423 .26242 L p F P
0 g
s
.691 .697 .85 r
.58458 .22849 m .61017 .2705 L .65003 .25198 L p F P
0 g
s
.689 .698 .853 r
.61017 .2705 m .58458 .22849 L .54479 .24732 L p F P
0 g
s
.687 .734 .886 r
.22496 .24388 m .25666 .27926 L .29423 .26242 L p F P
0 g
s
.683 .732 .888 r
.25666 .27926 m .22496 .24388 L .18754 .26056 L p F P
0 g
s
.694 .714 .864 r
.47823 .22646 m .50551 .26573 L .54479 .24732 L p F P
0 g
s
.693 .714 .865 r
.50551 .26573 m .47823 .22646 L .43899 .24514 L p F P
0 g
s
.778 .716 .794 r
.73092 .20978 m .75467 .25537 L .794 .22061 L p F P
0 g
s
.711 .687 .825 r
.75467 .25537 m .73092 .20978 L .69033 .2323 L p F P
0 g
s
.778 .716 .794 r
.794 .22061 m .77171 .18469 L .73092 .20978 L p F P
0 g
s
.697 .728 .874 r
.37105 .22637 m .40025 .26362 L .43899 .24514 L p F P
0 g
s
.693 .727 .877 r
.40025 .26362 m .37105 .22637 L .33237 .24475 L p F P
0 g
s
.698 .698 .847 r
.62486 .20924 m .65003 .25198 L .69033 .2323 L p F P
0 g
s
.691 .697 .85 r
.65003 .25198 m .62486 .20924 L .58458 .22849 L p F P
0 g
s
.695 .713 .862 r
.51797 .20723 m .54479 .24732 L .58458 .22849 L p F P
0 g
s
.694 .714 .864 r
.54479 .24732 m .51797 .20723 L .47823 .22646 L p F P
0 g
s
.692 .735 .884 r
.26298 .22645 m .29423 .26242 L .33237 .24475 L p F P
0 g
s
.687 .734 .886 r
.29423 .26242 m .26298 .22645 L .22496 .24388 L p F P
0 g
s
.697 .727 .874 r
.41026 .20728 m .43899 .24514 L .47823 .22646 L p F P
0 g
s
.697 .728 .874 r
.43899 .24514 m .41026 .20728 L .37105 .22637 L p F P
0 g
s
.715 .706 .84 r
.66562 .18901 m .69033 .2323 L .73092 .20978 L p F P
0 g
s
.698 .698 .847 r
.69033 .2323 m .66562 .18901 L .62486 .20924 L p F P
0 g
s
.696 .712 .861 r
.55823 .18765 m .58458 .22849 L .62486 .20924 L p F P
0 g
s
.695 .713 .862 r
.58458 .22849 m .55823 .18765 L .51797 .20723 L p F P
0 g
s
.696 .736 .882 r
.30158 .20822 m .33237 .24475 L .37105 .22637 L p F P
0 g
s
.692 .735 .884 r
.33237 .24475 m .30158 .20822 L .26298 .22645 L p F P
0 g
s
.731 .713 .834 r
.70671 .1661 m .73092 .20978 L .77171 .18469 L p F P
0 g
s
.715 .706 .84 r
.73092 .20978 m .70671 .1661 L .66562 .18901 L p F P
0 g
s
.699 .726 .871 r
.45 .18771 m .47823 .22646 L .51797 .20723 L p F P
0 g
s
.697 .727 .874 r
.47823 .22646 m .45 .18771 L .41026 .20728 L p F P
0 g
s
.702 .713 .858 r
.59899 .16768 m .62486 .20924 L .66562 .18901 L p F P
0 g
s
.696 .712 .861 r
.62486 .20924 m .59899 .16768 L .55823 .18765 L p F P
0 g
s
.731 .713 .834 r
.77171 .18469 m .74861 .14747 L .70671 .1661 L p F P
0 g
s
.699 .737 .88 r
.34075 .18917 m .37105 .22637 L .41026 .20728 L p F P
0 g
s
.696 .736 .882 r
.37105 .22637 m .34075 .18917 L .30158 .20822 L p F P
0 g
s
.699 .725 .87 r
.49025 .16771 m .51797 .20723 L .55823 .18765 L p F P
0 g
s
.699 .726 .871 r
.51797 .20723 m .49025 .16771 L .45 .18771 L p F P
0 g
s
.719 .72 .85 r
.64025 .14681 m .66562 .18901 L .70671 .1661 L p F P
0 g
s
.702 .713 .858 r
.66562 .18901 m .64025 .14681 L .59899 .16768 L p F P
0 g
s
.701 .737 .879 r
.38047 .1695 m .41026 .20728 L .45 .18771 L p F P
0 g
s
.699 .737 .88 r
.41026 .20728 m .38047 .1695 L .34075 .18917 L p F P
0 g
s
.7 .725 .869 r
.53101 .14749 m .55823 .18765 L .59899 .16768 L p F P
0 g
s
.699 .725 .87 r
.55823 .18765 m .53101 .14749 L .49025 .16771 L p F P
0 g
s
.686 .707 .864 r
.6819 .12344 m .70671 .1661 L .74861 .14747 L p F P
0 g
s
.719 .72 .85 r
.70671 .1661 m .6819 .12344 L .64025 .14681 L p F P
0 g
s
.703 .737 .878 r
.42072 .14924 m .45 .18771 L .49025 .16771 L p F P
0 g
s
.701 .737 .879 r
.45 .18771 m .42072 .14924 L .38047 .1695 L p F P
0 g
s
.705 .727 .867 r
.57231 .12682 m .59899 .16768 L .64025 .14681 L p F P
0 g
s
.7 .725 .869 r
.59899 .16768 m .57231 .12682 L .53101 .14749 L p F P
0 g
s
.686 .707 .864 r
.74861 .14747 m .72467 .10889 L .6819 .12344 L p F P
0 g
s
.702 .737 .878 r
.46151 .12862 m .49025 .16771 L .53101 .14749 L p F P
0 g
s
.703 .737 .878 r
.49025 .16771 m .46151 .12862 L .42072 .14924 L p F P
0 g
s
.722 .733 .86 r
.61412 .10534 m .64025 .14681 L .6819 .12344 L p F P
0 g
s
.705 .727 .867 r
.64025 .14681 m .61412 .10534 L .57231 .12682 L p F P
0 g
s
.704 .737 .877 r
.50283 .10768 m .53101 .14749 L .57231 .12682 L p F P
0 g
s
.702 .737 .878 r
.53101 .14749 m .50283 .10768 L .46151 .12862 L p F P
0 g
s
.648 .702 .885 r
.65634 .08124 m .6819 .12344 L .72467 .10889 L p F P
0 g
s
.722 .733 .86 r
.6819 .12344 m .65634 .08124 L .61412 .10534 L p F P
0 g
s
.708 .738 .875 r
.5447 .08638 m .57231 .12682 L .61412 .10534 L p F P
0 g
s
.704 .737 .877 r
.57231 .12682 m .5447 .08638 L .50283 .10768 L p F P
0 g
s
.648 .702 .885 r
.72467 .10889 m .69983 .06886 L .65634 .08124 L p F P
0 g
s
.725 .745 .867 r
.58711 .06424 m .61412 .10534 L .65634 .08124 L p F P
0 g
s
.708 .738 .875 r
.61412 .10534 m .58711 .06424 L .5447 .08638 L p F P
0 g
s
.622 .702 .903 r
.62997 .0396 m .65634 .08124 L .69983 .06886 L p F P
0 g
s
.725 .745 .867 r
.65634 .08124 m .62997 .0396 L .58711 .06424 L p F P
0 g
s
.622 .702 .903 r
.69983 .06886 m .67404 .0273 L .62997 .0396 L p F P
0 g
s
.4 g
.25 Mabswid
.67245 0 m
.70644 .58546 L
s
.75408 .133 m
.80324 .70505 L
s
.82659 .25114 m
.88704 .80858 L
s
.89142 .35678 m
.9603 .8991 L
s
.67378 .02282 m
.93008 .43661 L
s
.68078 .14345 m
.94555 .54979 L
s
.68848 .27609 m
.96226 .67199 L
s
.69699 .42266 m
.98035 .80434 L
s
.70644 .58546 m
1 .94814 L
s
0 g
.67245 0 m
.92713 .41497 L
s
.92713 .41497 m
1 .94814 L
s
1 .94814 m
.70644 .58546 L
s
.70644 .58546 m
.67245 0 L
s
.4 g
.08492 .26784 m
.67378 .02282 L
s
.06637 .38492 m
.68078 .14345 L
s
.04619 .51228 m
.68848 .27609 L
s
.02415 .65132 m
.69699 .42266 L
s
0 .80374 m
.70644 .58546 L
s
.08846 .24555 m
0 .80374 L
s
.26841 .16989 m
.21392 .73764 L
s
.4625 .08828 m
.44837 .6652 L
s
.67245 0 m
.70644 .58546 L
s
0 g
.08846 .24555 m
0 .80374 L
s
0 .80374 m
.70644 .58546 L
s
.70644 .58546 m
.67245 0 L
s
.67245 0 m
.08846 .24555 L
s
.08846 .24555 m
.67245 0 L
s
.08846 .24555 m
.09356 .25076 L
s
gsave
.07825 .23514 -102.841 -18.5625 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 22.5625 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.062 moveto
%%IncludeResource: font Times
%%IncludeFont: Times
/Times findfont 16.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(0.85) show
1.000 setlinewidth
grestore
.26841 .16989 m
.27316 .17542 L
s
gsave
.25892 .15883 -91.3723 -18.5625 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 22.5625 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.062 moveto
%%IncludeResource: font Times
%%IncludeFont: Times
/Times findfont 16.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(0.9) show
1.000 setlinewidth
grestore
.4625 .08828 m
.46682 .09415 L
s
gsave
.45386 .07654 -97.6736 -18.5625 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 22.5625 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.062 moveto
%%IncludeResource: font Times
%%IncludeFont: Times
/Times findfont 16.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(0.95) show
1.000 setlinewidth
grestore
.67245 0 m
.67627 .00621 L
s
gsave
.66483 -0.01242 -71.943 -18.5625 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 22.5625 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.062 moveto
%%IncludeResource: font Times
%%IncludeFont: Times
/Times findfont 16.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(1) show
1.000 setlinewidth
grestore
.125 Mabswid
.12339 .23086 m
.12641 .23402 L
s
.15884 .21596 m
.16182 .21916 L
s
.19482 .20083 m
.19776 .20407 L
s
.23134 .18547 m
.23423 .18875 L
s
.30604 .15406 m
.30885 .15742 L
s
.34426 .138 m
.34701 .14139 L
s
.38306 .12168 m
.38576 .12512 L
s
.42247 .10511 m
.42512 .10859 L
s
.50316 .07118 m
.50569 .07475 L
s
.54447 .05381 m
.54695 .05742 L
s
.58645 .03616 m
.58886 .03981 L
s
.6291 .01823 m
.63145 .02191 L
s
gsave
.30907 .06148 -98.9655 -18.5625 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 22.5625 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding ISOLatin1Encoding def
        currentdict
    end

    newfontname exch definefont pop
} def
63.000 15.062 moveto
%%IncludeResource: font Times
%%IncludeFont: Times
%%BeginResource: font Times-MISO
%%BeginFont: Times-MISO
/Times /Times-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Times-MISO
%%IncludeFont: Times-MISO
/Times-MISO findfont 16.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 15.062 moveto
%%IncludeResource: font Times-MISO
%%IncludeFont: Times-MISO
/Times-MISO findfont 16.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(cos) show
%%IncludeResource: font Mathematica1Mono
%%IncludeFont: Mathematica1Mono
/Mathematica1Mono findfont 16.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(q) show
101.250 15.062 moveto
%%IncludeResource: font Times-MISO
%%IncludeFont: Times-MISO
/Times-MISO findfont 16.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{504, 504},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40007h0001n1000`40n007h00OoooonOoo003ooooi
ool00?ooooWoo`00oooonOoo003ooooiool00?ooooWoo`00oooonOoo003ooooiool00?ooooWoo`00
oooonOoo003ooooiool00?ooooWoo`00oooonOoo003ooooiool00?ooooWoo`00oooonOoo003ooooi
ool00?ooocSoo`D00;coo`00oooo>_oo00<00?ooool0_?oo003oooljool00`00ooooo`2lool00?oo
oc[oo`03003ooooo0;coo`00oooo>_oo00<00?ooool0_?oo003oooljool00`00ooooo`2lool00?oo
oc[oo`03003ooooo0;coo`00oooo>_oo00<00?ooool04?oo0`00ZOoo003oooljool00`00ooooo`0?
ool01@00oooooooo0000Z?oo003ooolhool00`00ool0000Aool01000oooooooo0P00Yooo003oooli
ool2000@ool00`00ooooo`03ool00`00ooooo`2Uool00?oood_oo`03003ooooo00?oo`03003ooooo
0:Goo`00ooooBooo00<00?ooool00ooo00<00?ooool0YOoo003ooom;ool00`00ooooo`03ool00`00
ooooo`2Uool00?oood_oo`03003ooooo00?oo`03003ooooo0:Goo`00oooo@Ooo0P002?oo00<00?oo
ool00_oo00<00?ooool0Y_oo003oooloool200000ooo00000008ool01@00oooooooo0000Z?oo003o
oolmool20003ool30008ool3002Yool00?oooc[oo`<000Goo`03003oo`000;Coo`00oooo>?oo0P00
2?oo00<00?oo0000]?oo003oooleool3000:ool01000ooooo`00/ooo003ooolcool2000=ool01@00
oooooooo0000/_oo003ooolaool2000?ool01@00oooooooo0000/_oo003oool^ool4000@ool00`00
ooooo`02ool00`00ooooo`2_ool00?ooobcoo`8000Coo`03003ooooo00goo`03Hb`00?oo00?oo`03
003ooooo0:koo`00oooo:_oo0P005?oo0V</00?oo`00Hb`00ooo00<00?ooool0[_oo003ooolWool3
000Dool2Hb`2ool200000f</ooooo`02ool00`00ooooo`2]ool00?ooobGoo`8001Coo`=S;0@00004
WI`00000Hb`2ool2002_ool00?ooob;oo`<001Coo`9S;0<000FMW0800003Hbcooooo00;oo`03003o
oooo0:coo`00SOoo0`001?oo10000ooo10001?oo0`00NOoo0P005?oo0f</00<009fLWI`01ifL00@0
09fL001S;0Coo`03003ooooo0:_oo`00S?oo0P000_oo0P000_oo00D00?ooooooo`0000;oo`05003o
ooooool00002ool01@00oooooooo0000M_oo0`004_oo00@006</Hb`000bMW005002MW000oomS;003
ool00`00ooooo`2[ool008coo`03003ooooo00;oo`03003oo`0000Goo`03003oo`0000;oo`800003
ool00?oo00Coo`03003ooooo073oo`<000?oo`03003ooooo00goo`03001S;6</00nMW005002MW9fL
001S;004ool00`00ooooo`2Zool008coo`03003ooooo00Coo`03003ooooo00?oo`03003ooooo00<0
00;oo`03003ooooo00Coo`03003ooooo06goo`8001Coo`=S;003002MW9fL00jMW006002MW9fLWI`0
06</0ooo00<00?ooool0Z_oo002<ool00`00ooooo`04ool00`00ooooo`03ool01000ool000001?oo
2000KOoo0P005?oo0V</0kgk00<009fLWI`03YfL00L009fLWIbMW000oomS;003ool00`00ooooo`2Y
ool008coo`03003ooooo00;oo`03003oo`0000Coo`800003ool00?oo00;oo`03003oo`0000Koo`03
003ooooo06Soo`<001?oo`=S;0Jmn`03002MW9fL00fMW003002MW9fL00:MW003001S;?oo00?oo`03
003ooooo0:Soo`00SOoo10000ooo10000ooo10000ooo00<00?ooool00ooo00<00?ooool07ooo0`00
0ooo0P001?oo00<00?ooool00ooo1000<Ooo0P005?oo0V</0P002;gk00<009fLWI`039fL00<009fL
WI`00ifL00<006</ool00_oo00<00?ooool0Z?oo002Sool01@00oooooooo00008Ooo00D00?oooooo
o`0000?oo`03003ooooo00?oo`03003ooooo00Koo`03003ooooo02_oo`<001Coo`9S;08000Zmn`03
002MW9fL00bMW003002MW9fL00>MW003001S;?oo00?oo`03003ooooo0:Ooo`00Y?oo0`008_oo00@0
0?ooooooo`8000Woo`03003ooooo00Koo`03003ooooo02Soo`<001?oo`=S;08000fmn`03002MW9fL
00^MW003002MW9fL00BMW003001S;?oo00;oo`800:Soo`00b?oo00<00?ooool00ooo00<00?ooool0
2?oo00<00?ooool01Ooo00<00?ooool09_oo0P000ooo00<00?ooool03_oo0V</0_oo00<00;gk_O/0
3[gk00<009fLWI`02YfL00<009fLWI`019fL00L00?ooHbcoo`00ool0002Xool00<Soo`03003ooooo
00?oo`03003ooooo00Coo`<00003ool00?oo00Koo`03003ooooo02?oo`<001?oo`=S;0;oo`80016m
n`03002MW9fL00ZMW003002MW9fL00FMW003001S;?oo00?oo`03003ooooo0:Goo`00b?oo00<00?oo
ool00ooo00<00?ooool00ooo0P000_oo0`001Ooo0P008ooo0P005?oo0V</0ooo0P005;gk00<009fL
WI`02IfL00<009fLWI`01YfL00<006</ool00_oo00<00?ooool0YOoo0038ool00`00ooooo`03ool0
0`00ooooo`03ool00`00ooooo`02ool20003ool3000Rool2000Dool2Hb`4ool00`00_O^mn`0E_O/0
0`00WIbMW009WI`00`00WIbMW005WI`00`00Hbcoo`03ool00`00ooooo`2Tool00<Soo`03003ooooo
00?oo`03003ooooo00?oo`03003ooooo00;oo`800003ool000000080023oo`<001?oo`=S;0Coo`80
0003]O/00;gk01Jmn`03002MW9fL00RMW003002MW9fL00FMW0800003Hbcooooo00;oo`03003ooooo
0:?oo`00b?oo00<00?ooool00_oo00<00?ooool01?oo00<00?ooool00_oo00D00?ooooooo`0001oo
o`<001Coo`9S;0Goo`8000>en`03002mnkgk01Fmn`03002MW9fL00RMW004002MW9fLWI`40002YI`0
0`00Hbcoo`02ool00`00ooooo`2Sool00<Woo`05003oooooool00006ool01000oooooooo0P000ooo
00<00?ooool06ooo0`004ooo0f</1Ooo0P001[Gk00<00;gk_O/05Kgk00<009fLWI`01ifL10001ZFL
00<006</ool00ooo00<00?ooool0X_oo003:ool30008ool40005ool4000Gool20003ool00`00oooo
o`0>ool2Hb`6ool20009]O/00`00_O^mn`0E_O/00`00WIbMW004WI`3000:YI`00`00Hbcoo`03ool0
0`00ooooo`2Qool00?Ooo`8001Coo`9S;0Koo`8000^en`03002mnkgk01Fmn`03002MW9fL00@000:U
W003002UW:FL00RUW003003oof</00?oo`03003ooooo0:7oo`00m?oo0`004ooo0f</1_oo0P003[Gk
00<00;gk_O/05Kgk0P001ZFL00<00:FLYI`02JFL00<006</ool00ooo00<00?ooool0X?oo003bool2
000Dool2Hb`7ool2000A]O/00`00_O^mn`0B_O/200000kgK002UW005YI`00`00YIbUW00:YI`01000
Hbcooooo0P00X_oo003`ool2000Cool3Hb`7ool2000C]O/00`00_O^mn`0@_O/20003_M/00`00YIbU
W004YI`00`00YIbUW00:YI`00`00Hbcoo`03ool00`00ooooo`2Oool00>goo`<001?oo`9S;0Soo`80
01Jen`03002mnkgk00jmn`03002mfkgK00Bmf`03002UW:FL00>UW003002UW:FL00^UW003001S;?oo
00?oo`03003ooooo09koo`00jooo0`0000=S;?ooool04?oo0V</2?oo0P000[Gl00<00;Gk]O/05;Gk
00<00;gk_O/02kgk0P001kgK00<00:FLYI`00jFL00<00:FLYI`02jFL00<00?ooHb`00ooo00<00?oo
ool0W_oo003Xool30002ool00f</003oo`0>ool3Hb`8ool20004]O`00`00]O^en`0E]O/00`00_O^m
n`08_O/2000:_M/00`00YIbUW002YI`00`00YIbUW00<YI`00`00Hbcoo`03ool00`00ooooo`2Mool0
0>Koo`8000Goo`03Hb`006</00coo`9S;0Woo`8000Neo003002enkGk01Ben`03002mnkgk00Jmn`80
00fmf`05002UW:FLYI`0000>YI`00`00oomS;003ool00`00ooooo`2Mool00>Coo`8000Ooo`04Hbco
ooooHb`8ool3Hb`9ool2000:]O`00`00]O^en`0D]O/00`00_O^mn`04_O/00`00_M^mf`0=_M/01@00
YIbUW:FL00003jFL00<00?ooHb`00ooo00<00?ooool0W?oo003Qool30009ool01F</ooooooooHb`0
1Ooo0V</2_oo0P003;Gl00<00;Gk]O/05KGk00@00;gk_O^mn`80016mf`04002UW:FL000@YI`00`00
Hbcoo`03ool00`00ooooo`2Kool00=ooo`8000coo`05HbcooooooomS;003ool2Hb`:ool2000?]O`0
0`00]O^en`0D]O/01000_O/000004kgK00@00:FLYI`0012UW003003oof</00?oo`03003ooooo09_o
o`00gOoo0P003_oo00=S;?ooool00_oo0f</2_oo0P004[Gl00<00;Gk]O/05;Gk00<00;gK_M/05;gK
00<00:FL00004JFL00<006</ool00ooo00<00?ooool0V_oo003Jool300000ooo003oo`0=ool00f</
ooooo`02Hb`00oooHbcoo`08ool2000D]O`00`00]O^en`0B]O/200000kGK002mf`0D_M/01000YIbU
W0004:FL00<00?ooHb`00ooo0P00Vooo003Hool2000Cool3Hb`4ool00f</ooooo`04ool200000kGl
002eo00D]O`00`00]O^en`0?]O/20003]M/00`00_M^mf`0D_M/00`00YI`0000@YI`200001_ooHbco
o`00ool009_oo`00eOoo0`004ooo0f</1_oo00=S;?ooool00_oo0P000kGl00<00;Gl]O`05;Gl00<0
0;Gk]O/03;Gk0P001[GK00<00;gK_M/05;gK0P003:FL10000Zfk00<006</ool00ooo00<00?ooool0
V?oo003Cool2000Dool2Hb`2ool00f</ooooo`05ool016</ool000001[Gl00<00;Gl]O`05;Gl00<0
0;Gk]O/02KGk0P002;GK00<00;gK_M/05;gK0P002JFL0`001Zfk00<00?ooHb`00ooo00<00?ooool0
2ooo10001_oo0`00P?oo003Aool2000Cool3Hb`4ool00f</ooooo`05ool20009]O`00`00]Obeo00C
]O`00`00]O^en`07]O/2000;]M/00`00_M^mf`0D_M/00`00YIbUW004YI`3000:[K/00`00Hbcoo`03
ool00`00ooooo`0>ool00`00ooooo`02ool01@00oooooooo0000Oooo003>ool3000Cool2Hb`7ool0
0f</ooooo`03ool2000;]O`00`00]Obeo00D]O`00`00]O^en`04]O/2000>]M/00`00_M^mf`0C_M/0
0`00YIbUW004000>[K/00`00Hbcoo`03ool00`00ooooo`0>ool01@00oooooooo00000ooo0P00O_oo
003<ool2000Dool2Hb`9ool016</oooooooo0P003[Gl00<00;Gl]O`05;Gl00@00;Gk]O^en`80012e
f`03002mfkgK01>mf`<001:]^`03003oof</00?oo`03003ooooo00koo`04003ooooo0005ool00`00
ooooo`1lool00<[oo`8000;oo`03003ooooo00koo`=S;0_oo`04Hbcoo`00000A]O`00`00]Obeo00C
]O`01000]O/000004kGK00<00;gK_M/04KgK0P0000>mf`00[K/04Zfk00<006</ool00ooo00<00?oo
ool03Ooo00@00?ooool000Goo`03003ooooo07coo`00aooo0`004ooo0V</3_oo00=S;000]O`04[Gl
00<00;Gl]O`04kGl0P005KGK00<00;gK_M/03kgK0P0000Bmf`00_M/001B]^`03001S;?oo00;oo`03
003ooooo00coo`8000;oo`03003ooooo00?oo`03003ooooo07coo`00aOoo0P004ooo0f</3_oo0P00
00=S;;Gl]O`04kGl00<00;Gl]O`04;Gl0P000[GK00<00;GK]M/04kGK00<00;gK_M/03KgK00D00;gK
_M^mf`0000:mf`03002]^jfk016]^`03003oof</00?oo`03003ooooo00Woo`<000?oo`03003ooooo
00?oo`03003ooooo07coo`00`ooo0P004ooo0V</3ooo0P000[Gl00=S;;Gl]O`05;Gl00<00;Gl]O`0
3KGl0P001;GK00<00;GK]M/04kGK00<00;gK_M/02kgK0P001;gK00@00;gK_M/001B]^`03001S;?oo
00;oo`03Hb`00?oo00Ooo`@000Coo`03003ooooo00?oo`03003ooooo07coo`00POoo0`000ooo0P00
1?oo00<00?ooool0<?oo0`004ooo0V</3ooo0P001;Gl00=S;000]O`05KGl00<00;Gl]O`02[Gl0P00
1kGK00<00;GK]M/04kGK00<00;gK_M/02;gK0P001[gK00D00;gK_M^mf`0001B]^`06001S;6</Hbco
o`002Ooo00<00?ooool01?oo00<00?ooool00_oo00<00?ooool0OOoo0020ool01@00oooooooo0000
0ooo00<00?ooool00ooo00<00?ooool0;?oo0`004ooo0f</3ooo0P001KGl00=S;;Gl00005[Gl00<0
0;Gl]O`02;Gl0P002[GK00<00;GK]M/04kGK00<00;gK_M/01KgK0P002;gK00D00;gK_M^mf`0001B]
^`03001S;?oo00;oo`03Hb`00?oo00Ooo`03003ooooo00Goo`05003oooooool0001oool0083oo`04
003oooooool20009ool00`00ooooo`0Yool3000Cool2Hb`@ool20007]O`016</]Obeo0005[Gl00<0
0;Gl]O`01KGl0P003;GK00<00;GK]M/04kGK00<00;gK_M/01;gK00<00;gK_M/02;gK00<00;gK_M/0
0[gK00<00:fk[K/04Zfk00L006</ooooof</ool00008ool40004ool30020ool007ooo`03003ooooo
00?oo`03003ooooo00Soo`03003ooooo02Koo`8000?oo`03003ooooo00goo`=S;13oo`8000Veo005
Hbbeo;Gl]O`0000F]O`00`00]Obeo002]O`2000?]M/00`00]M^ef`0C]M/01000_M^mfkgK0P002kgK
00<00;gK_M/00[gK00<00:fk[K/04jfk00H006</oomS;?oo002Cool007ooo`03003ooooo00?oo`03
003ooooo00Coo`<00003ool00?oo02Goo`8001?oo`9S;17oo`8000^eo003Hbbeo;Gl00:eo003002e
o;Gl01>eo003002eo;Gl008001:ef`03002efkGK01:ef`04002mf`00000=_M/00`00Hbbmf`03_M/0
0`00[K^]^`0B[K/01000Hbcoof</0_oo00<00?ooool0T?oo001oool00`00ooooo`03ool00`00oooo
o`03ool20002ool3000Rool3000Cool2Hb`Aool2000=]O`00f</]Obeo003]O`00`00]Obeo00B]O`3
000D]M/00`00]M^ef`0B]M/2000=_M/2Hb`00`00_M]S;004_M/00`00[K^]^`0A[K/200001F</oomS
;?oo0000T_oo001oool00`00ooooo`03ool00`00ooooo`03ool00`00ooooo`02ool2000Pool2000C
ool3Hb`Aool2000?]O`00f</]Obeo003]O`00`00]Obeo00@]O`20002]M/00`00]M^ef`0C]M/00`00
]M^ef`0?]M/20002]K/00`00_M^mf`07_M/3Hb`3_M/00`00Hbbmf`03_M/00`00[K^]^`0?[K/20002
_KX00f</oomS;002ool00`00ooooo`2?ool007ooo`03003ooooo00?oo`03003ooooo00?oo`03003o
oooo00;oo`8001koo`8001?oo`9S;1;oo`8000:eo003002eo;Gl00beo003Hbbeo;Gl00Beo003002e
o;Gl00feo08000Bef`03002efkGK01Bef`03002efkGK00bef`8000Be^`03002mfkgK00Fmf`9S;0Jm
f`03002mff</00Bmf`03002]^jfk00b]^`8000Bm^P03001S;6</00?oo`03003ooooo08koo`00Oooo
00<00?ooool00_oo00<00?ooool01?oo00<00?ooool00_oo00<00?ooool06_oo0`004_oo0f</4_oo
0P001KGl00<00;Gl]O`02kGl00=S;;Gl]O`01KGl00<00;Gl]O`02[Gl0P001kGK00<00;GK]M/05;GK
00<00;GK]M/02KGK0P001kFk00@00;gK_M^mf`=S;0Rmf`04002mfkgKHb`3_M/00`00[K^]^`0:[K/2
0007_KX00`00Hbcoo`02ool00`00ooooo`2>ool0083oo`05003oooooool00006ool01000oooooooo
0P006_oo0P000_oo00<00?ooool03_oo0V</4ooo0P001kGl00<00;Gl]O`02kGl00=S;;Gl]O`01[Gl
00<00;Gl]O`01kGl0P002[GK00<00;GK]M/04kGK00<00;GK]M/01kGK0P002[Fk0f</2kgK00@00;gK
_M]S;0Bmf`03002]^jfk00N]^`8000Vm^P03001S;?oo00?oo`03003ooooo08goo`00POoo0`002?oo
10006?oo0`004ooo0V</4ooo0P002[Gl00<00;Gl]O`02[Gl00=S;;Gl]O`01kGl00<00;Gl]O`01;Gl
0P003;GK00<00;GK]M/05;GK00<00;GK]M/01;GK0P002[Fk0V</00<00;gK_M/02kgK00D00;gK_M^m
ff</00>mf`03002]^jfk00F]^`8000bm^P9S;0;oo`8008ooo`00Y_oo0P004ooo0f</4ooo0P003KGl
00<00;Gl]O`02KGl00=S;;Gl]O`01kGl00<00;Gl]O`00[Gl0P003kGK00<00;GK]M/05;GK00@00;GK
]M^ef`8000Ve^`=S;0>e^`03002mfkgK00Zmf`03002mfkgK00:mf`05HbbmfkgK_M/00004[K/2000>
_KX00f</001S;003ool00`00ooooo`2<ool00:Coo`8001?oo`9S;1Coo`80012eo003002eo;Gl00Re
o003Hbbeo;Gl00Reo004002eo000000B]M/00`00]M^ef`0C]M/01000]M/000002KFk0V</1kFk00<0
0;gK_M/02KgK00<00;gK_M/00[gK00ES;;gK_M^mf`0000:]^`80012m^P03Hb`006</00Coo`03003o
oooo08_oo`00XOoo0`004_oo0f</5?oo0P004kGl00<00;Gl]O`01kGl00=S;;Gl]O`02;Gl0P005;GK
00<00;GK]M/04kGK0P002;Fk0f</2KFk00<00;gK_M/02KgK00<00;gK_M/00kgK00AS;;gK_M^mf`80
01:m^P04Hbbm^P00Hb`3ool00`00ooooo`2;ool009koo`<001?oo`9S;1Goo`8001Feo003002eo;Gl
00Neo003Hbbeo;Gl00Jeo08000:ef`03002efkGK01:ef`03002efkGK012ef`8000:e^`03002e^kFk
00>e^`9S;0fe^`03002mfkgK00Rmf`03002mfkgK00Bmf`05Hb`00000]KX0000B_KX016</_KX006</
1?oo00<00?ooool0R_oo002Mool3000Bool2Hb`Eool20002]O`00`00]Obeo00C]O`00`00]Obeo006
]O`00f</]Obeo004]O`20004]M/00`00]M^ef`0C]M/00`00]M^ef`0=]M/20004]K/00`00]K^e^`03
Hb`?]K/00`00_M^mf`08_M/00`00_M^mf`03_M/00`00Hbbe^P02]KX00`00_KZm^P0@_KX01F</_KZm
^P00Hb`01?oo00<00?ooool0ROoo002Jool300001?ooHbcoo`003_oo0f</5Ooo0P001KGl00<00;Gl
]O`04kGl00<00;Gl]O`01KGl00=S;;Gl]O`00[Gl0P001kGK00<00;GK]M/04kGK00<00;GK]M/02[GK
0P001kFk0V</4kFk00<00;gK_M/01kgK00@00;gK_M^mf`8000:e^P05Hbbe^[Fj]KX0000A_KX01V</
_KZm^[fj001S;0?oo`03003ooooo05;oo`<003Coo`00V?oo0P001?oo00AS;?oooomS;0_oo`9S;1Ko
o`8000Neo003002eo;Gl01Beo003002eo;Gl00Beo003Hbbeo;Gl008000Zef`03002efkGK01:ef`03
002efkGK00Ref`8000Je^`=S;003002e^kFk01>e^`03002mfkgK00Jmf`04002mf`000004]KX01F</
]KZe^[Fj00004[fj00ES;;fj_KX006</00Coo`03003ooooo03coo`03003ooooo017oo`04003ooooo
ool2000:ool00`00ooooo`0Uool009Goo`<000Koo`04HbcoooooHb`8ool3Hb`Fool2000:]O`00`00
]Obeo00D]O`00`00]Obeo003]O`00f</0000000<]M/00`00]M^ef`0C]M/00`00]M^ef`05]M/20005
]K/3Hb`4]K/00`00]K^e^`0B]K/00`00_M^mf`06_M/20007]KX01F</]KZe^[Fj00004Kfj00IS;;fj
_KZm^P00Hb`3ool00`00ooooo`0`ool00`00ooooo`08ool00`00ooooo`0Bool00`00ooooo`03ool0
0`00ooooo`08ool00`00ooooo`0Tool009?oo`8000Woo`05HbcooooooomS;005ool2Hb`Gool2000=
]O`00`00]Obeo00C]O`00`00]Obeo002]O`00`00Hbbef`0>]M/00`00]M^ef`0C]M/00`00]M^ef`02
]M/20005]K/2Hb`8]K/00`00]K^e^`0B]K/00`00_M^mf`04_M/20009]KX01F</]KZe^[Fj00004;fj
00IS;;fj_KZm^P00Hb`4ool00`00ooooo`0_ool00`00ooooo`08ool00`00ooooo`0Bool00`00oooo
o`03ool00`00ooooo`08ool00`00ooooo`0Tool0093oo`<000[oo`03Hbcooooo00?oo`03Hbcooooo
009S;1Ooo`80012eo003002eo;Gl01>eo006002eo000002eff</4;GK00<00;GK]M/04[GK00<00;GK
]M/00P001;Fk0f</2[Fk00<00;Fk]K/04kFk00@00;gK_M^mf`800003]KX00;Fj00Re^P05Hbbe^[Fj
]KX0000@_KX00f</_KZm^P02_KX01`00Hbcooooo003oo`00033oo`@000Koo`03003ooooo00Goo`H0
00;oo`<000Coo`D000coo`03003ooooo02?oo`00S_oo0`003?oo00=S;?ooool00ooo0f</5ooo0P00
4kGl00<00;Gl]O`04[Gl0P000kGK00=S;;GK]M/03[GK00<00;GK]M/04[GK0`001;Fk0V</3[Fk00<0
0;Fk]K/04[Fk00@00;gK000000>e^P03002e^[Fj00Re^P05Hbbe^[Fj]KX0000?_KX00f</_KZm^P02
00000lFi001S;003ool00`00ooooo`0]ool00`00ool00002ool00`00ooooo`03ool00`00ooooo`04
ool20003ool00`00ooooo`020002ool20002ool00`00ooooo`0>ool00`00ooooo`0Sool008coo`80
00?oo`03003ooooo00Woo`04Hbcoooooool2Hb`2ool00f</ooooo`0Cool2000E]O`00`00]Obeo00@
]O`20002]M/01000]M^eff</4KGK00<00;GK]M/03kGK0P000[Fk00<00;FkHb`00V</4;Fk00<00;Fk
]K/04[Fk0P001[Fj00<00;Fj]KX02;Fj00AS;;Fj]KX000nm^P03Hb`0000000?5^@03001S;?oo00?o
o`03003ooooo02_oo`04003ooooo0003ool00`00ooooo`02ool00`00ooooo`04ool00`00ooooo`02
ool01000ooooo`001?oo00<00?ooool00`003_oo00<00?ooool08ooo002:ool2000Aool016</oomS
;6</1Ooo00=S;?ooool04?oo0P000[Gl00<00;Gl]O`04kGl00<00;Gl]O`03KGl0P001;GK00@00;GK
]M]S;1:ef`03002efkGK00bef`8000Be^`9S;1Be^`03002e^kFk00ne^`8000:eV`03002e^[Fj00>e
^P03002e^[Fj00Re^P05Hbbe^[Fj]KX0000>_KX00f</aKW5^@04aKT00`00Hbcoo`02ool00`00oooo
o`0[ool01000ooooo`000ooo00<00?ooool00_oo00<00?ooool01?oo00<00?ooool00_oo00@00?oo
000000Ooo`04003ooooo000=ool00`00ooooo`0Sool008Ooo`<001;oo`=S;0Soo`03Hbcooooo00go
o`8000Feo003002eo;Gl01>eo003002eo;Gl00Zeo08000Nef`03002eff</01>ef`03002efkGK00Ve
f`8000>e^`=S;003]K/00;Fk01>e^`03002e^kFk00fe^`8000BeV`03002e^[Fj00>e^P03002e^[Fj
00Ve^P04Hbbe^[Fj000<_KX200000f</aKW5^@04aKT00`00oomS;003ool00`00ooooo`0Zool01@00
oooooooo00000_oo00<00?ooool00_oo00<00?ooool01?oo00<00?ooool00_oo00<00?oo00001P00
0_oo00D00?ooooooo`0000coo`03003ooooo02?oo`00QOoo0P004ooo0V</00?oof</ool02?oo00=S
;?ooool02ooo0P001kGl00<00;Gl]O`05;Gl00<00;Gl]O`01kGl0P002[GK00<006</]M/04[GK00<0
0;GK]M/01kGK0P000kFk0V</1KFk00<00;Fk]K/04[Fk00<00;Fk]K/02[Fk0P001kFK00<00;Fj]KX0
0[Fj00<00;Fj]KX02[Fj00AS;;Fj]KX000Zm^P040035^LFiHb`7aKT00`00Hbcoo`03ool00`00oooo
o`0Zool01P00ooooo`00ool000Goo`03003ooooo00Coo`03003ooooo00;oo`04003ooooo0003ool0
1@00oooooooo00000ooo00<00?ooool02_oo00<00?ooool08ooo0022ool3000Bool3Hb`3ool00f</
ooooo`08ool00f</ooooo`08ool2000:]O`00`00]Obeo00D]O`00`00]Obeo004]O`2000<]M/00`00
Hbbef`0C]M/00`00]M^ef`04]M/20002]K/3Hb`7]K/00`00]K^e^`0C]K/00`00]K^e^`07]K/20009
]I/00`00]KZe^P02]KX00`00]KZe^P0:]KX01F</]KZe^[Fj00001kfj0P000lFi00=S;<FiaKT01/Fi
00<006</ool00_oo00<00?ooool0:ooo10001_oo00<00?ooool01Ooo1@000_oo1@000ooo1P002ooo
00<00?ooool08ooo0020ool2000Cool2Hb`6ool00f</ooooo`09ool00f</ooooo`05ool2000=]O`0
0`00]Obeo00C]O`00`00]Obeo002]O`2000?]M/00f</]M^ef`0C]M/01000]M^efkGK0P0000Be^f</
HbaS;0^e^`03002e^kFk01:e^`03002e^kFk00Fe^`8000beV`05002e^[Fj]KX0000=]KX016</]KZe
^P001Kfj0P001LFi00=S;<FiaKT01/Fi00<006</ool00ooo00<00?ooool0;?oo00<00?ooool01_oo
00<00?ooool02?oo00<00?ooool06?oo00<00?ooool09?oo001nool20002ool00`00ooooo`0>ool2
Hb`8ool00f</ooooo`0:ool00f</ooooo`02ool2000@]O`00`00]Obeo00C]O`01000]O`000004KGK
00=S;000]M/04kGK00D00;GK00000;Fk009S;0ne^`03002e^kFk01:e^`03002e^kFk00:e^`8000je
V`05002e^[Fj]KX0000>]KX016</]KZe^P000kfj00<00<FiaKT01/Fi00=S;<FiaKT01/Fi00<006</
ool00ooo00<00?ooool0;?oo00<00?ooool01Ooo00<00?ooool02?oo00<00?ooool06?oo00<00?oo
ool09?oo001kool3000Bool3Hb`:ool00f</ooooo`0;ool016</ool000004kGl00<00;Gl]O`04[Gl
0P004kGK00=S;000]M/04kGK0P000f</4KFk00<00;Fk]K/04[Fk00<00;Fk]K/00P004KFK00@00;Fj
]KX000je^P05Hbbe^[Fj002m^P020009aKT00f</aKW5^@06aKT01000oomS;?oo0`00;_oo00<00?oo
ool01_oo00<00?ooool01ooo00<00?ooool05ooo00<00?ooool09Ooo000]ool30003ool20003ool4
0003ool4000fool2000Cool2Hb`<ool00f</ooooo`0<ool2000E]O`00`00]Obeo00@]O`20002]M/0
0`00]M^ef`0@]M/00f</]M/0000A]M/20002Hb`00`00]K^e^`0B]K/00`00]K^e^`0A]K/3000C]I/0
1000]KZe^P003kFj00AS;;Fj]KX000_5^@03Hbc5^LFi00O5^@03001S;?oo00?oo`03003ooooo03go
o`80047oo`00;?oo00D00?ooooooo`0000?oo`04003ooooo0004ool00`00ooooo`04ool00`00oooo
o`0aool2000Bool3Hb`>ool00f</ooooo`0:ool20002]O`00`00]Obeo00C]O`00`00]Obeo00=]O`2
0004]M/00`00]M^ef`0@]M/016</]M^ef`003[GK00@006</HbaS;0:e^`03002e^kFk01:e^`03002e
^kFk00ne^`8000:eV`03002eVkFK01:eV`03002e^P00012e^P03Hb`0000000_5^@03Hbc5^LFi00S5
^@03001S;?oo00;oo`03003ooooo083oo`00;?oo00@00?ooooooo`8000Goo`03003ooooo00;oo`03
003ooooo00Goo`03003ooooo02goo`<001;oo`9S;17oo`03Hbcooooo00Soo`8000Feo003002eo;Gl
01>eo003002eo;Gl00Zeo08000Nef`03002efkGK00nef`05HbbefkGK]M/0000;]M/00`00HbaS;006
]K/00`00]K^e^`0B]K/00`00]K^e^`0<]K/20004]I/00`00]I^eV`0B]I/00`00]KX0000>]KX20000
16</]GZeNP002/Fi00=S;<FiaKT02<Fi00<006</ool00ooo00<00?ooool0Oooo000[ool00`00oooo
o`03ool00`00ooooo`03ool00`00ooooo`02ool00`00ooooo`05ool00`00ooooo`0[ool2000Cool2
Hb`Cool00f</ooooo`05ool30008]O`00`00]Obeo00C]O`00`00]Obeo007]O`2000:]M/00`00]M^e
f`0>]M/01F</]M^efkGK00002KGK0f</2KFk00<00;Fk]K/04[Fk00<00;Fk]K/02KFk0P001kFK00<0
0;FK]I/04[FK0P003;Fj0P000kEj00=S;;Ej00002/Fi00=S;<FiaKT02LFi00<006</ool00ooo00<0
0?ooool0O_oo000[ool00`00ooooo`03ool00`00ooooo`04ool00`00ool000020007ool00`00oooo
o`0Xool300000ooo003oo`0?ool3Hb`Eool00f</ooooo`03ool2000;]O`00`00]Obeo00D]O`00`00
]Obeo004]O`2000=]M/00`00]M^ef`0=]M/00f</]M^ef`02]M/00`00]M^ef`04]M/2Hb`<]K/00`00
]K^e^`0B]K/00`00]K^e^`07]K/20009]I/00`00]I^eV`0B]I/2000:]KX20006]GX00f</]GX00009
aKT00f</aKW5^@09aKT00`00oomS;003ool00`00ooooo`1nool002_oo`03003ooooo00?oo`03003o
oooo00Goo`<000Ooo`8002Soo`8001?oo`9S;1Soo`04Hbcoooooool2000>]O`00`00]Obeo00C]O`0
0`00]Obeo002]O`2000?]M/00`00]M^ef`0=]M/00f</]M^ef`03]M/00`00]M^ef`03Hb`?]K/00`00
]K^e^`0B]K/00`00]K^e^`04]K/2000<]I/00`00]I^eV`0B]I/00`00]KZe^P06]KX20008]GX00f</
]GX00009aKT00f</aKW5^@0:aKT00`00Hbcoo`03ool00`00ooooo`1mool002_oo`03003ooooo00?o
o`03003ooooo00Coo`<000Koo`<002Ooo`8001;oo`=S;1[oo`04Hbcoo`00000A]O`00`00]Obeo00C
]O`01000]O`000004[GK00<00;GK]M/03;GK00=S;;GK]M/00kGK0f</00<00;Fk]K/04;Fk00<00;Fk
]K/04[Fk00@00;Fk]K^e^`8000neV`03002eVkFK016eV`03002e^[Fj00Be^P8000^eNP03HbbeNP00
00S5^@03Hbc5^LFi00W5^@04003E]`00Hb`4ool00`00ooooo`1lool002_oo`03003ooooo00?oo`03
003ooooo00?oo`8000;oo`8000;oo`@002Goo`<001;oo`9S;1coo`03001S;00001Beo003002eo;Gl
01:eo08001Fef`03002efkGK00^ef`04HbbefkGK]M/2Hb`2000D]K/00`00]K^e^`0B]K/01000]K/0
00004KFK00<00;FK]I/04KFK0P000kFj0P003KEj00=S;;Ej00002<Fi00=S;<FiaKT02<Fi00H00=Fg
eKL00?ooHb`3ool00`00ooooo`1lool002_oo`03003ooooo00;oo`03003ooooo00Coo`03003ooooo
00;oo`05003oooooool0000Uool2000Cool2Hb`Lool200000jgLHb`0000E]O`00`00]Obeo00?]O`2
0002]M`00`00]M^ef`0C]M/00`00]M^ef`0:]M/4Hb`20002]K/00`00]K^e^`0B]K/00`00]K^e^`0A
]K/2000D]I/00`00]I^eV`0@]I/200000kFj0000000@]GX00f</]GX00007aKT00f</aKW5^@06aKT2
0004eKL01000Hbcooooo0`00OOoo000/ool01@00oooooooo00001_oo00<00?ooool00_oo00D00?oo
ooooo`0002?oo`8001;oo`=S;1coo`8000>]g003Hbb]g00001Feo003002eo;Gl00beo08000Feg003
002efkGK01:ef`03002efkGK00Vef`9S;003]M/0000000Be^`03002e^kFk01>e^`03002e^kFk00je
^`8000:eV`03002eVkFK016eV`03002eVkFK016eV`8001>eNP03Hb`00<Fi00K5^@03Hbc5^LFi00G5
^@03003E]mFg00GE]`03001S;?oo00;oo`03003ooooo00coo`D000Coo`<000Goo`<005_oo`00;Ooo
0`002?oo10001Ooo10007?oo0`0000?oo`00ool03ooo0V</7Ooo0P001JgL00AS;:gL[M`001Feo003
002eo;Gl00Reo0<000Neg003002efkGK01>ef`03002efkGK00Fef`AS;08000Ne^`03002e^kFk01>e
^`03002e^kFk00^e^`8000BeV`03002eVkFK01:eV`03002eVkFK00jeV`<00003]GX00;Ej016eNP03
HbbeNP0000O5^@03Hbc5^LFi00?5^@03003E]mFg00KE]`03001S;?oo00?oo`03003ooooo00goo`03
003ooooo00?oo`05003oooooool00003ool01@00oooooooo0000F_oo001Oool2000Bool3Hb`Mool2
0006[M`00f</[Mb]g002[M`00`00]Obeo00B]O`00`00]Obeo006]O`2000;]M`00`00]M^ef`0C]M/0
0`00]M^ef`02]M/2Hb`01;GK000006</2[Fk00<00;Fk]K/04[Fk00<00;Fk]K/02;Fk0`001kFK00<0
0;FK]I/04KFK00<00;FK]I/03;FK0P000[Ej00<00;Ej00004kEj00=S;000aKT01/Fi00=S;<FiaKT0
0/Fi00<00=FgeKL02=Fg00<006</ool00ooo00<00?ooool03?oo00<00?ooool00ooo00@00?oooooo
o`8000;oo`04003oooooool2001Iool005coo`<001;oo`9S;1koo`8000R]g003Hbb]g:gL00>]g003
002eo;Gl01:eo003002eo;Gl00>eo08000jeg003002efkGK01>ef`05001S;6</Hbbef`020002]K/0
0f</]K^e^`08]K/00`00]K^e^`0C]K/00`00]K^e^`05]K/2000:]I/00`00]I^eV`0B]I/00`00]I^e
V`09]I/20004]GX01000]GZeNP004kEj00=S;000aKT01LFi00ES;<FiaKW5^@0000_E]`03003oof</
00?oo`03003ooooo00coo`03003ooooo00;oo`03003ooooo00?oo`03003oo`0000Goo`03003ooooo
05Ooo`00F_oo0P004ooo0V</7_oo0P002ZgL00=S;:gL[M`01:gL00<00;Gl]O`04[Gl00<00;Gl]O`0
0P004KGL00<00;GK]M/04KGK0V</0[GK0P001;Fk00=S;;Fk]K/02KFk00<00;Fk]K/04kFk00<00;Fk
]K/00[Fk0P003KFK00<00;FK]I/04[FK00<00;FK]I/01[FK0P001[Ej00@00;Ej]GX001>eNP03Hb`0
0<Fi00G5^@04Hbc5^LFi000=eKL00`00Hbcoo`03ool00`00ooooo`0;ool00`00ooooo`02ool00`00
ooooo`03ool00`00ool00005ool00`00ooooo`1Gool005Ooo`<001;oo`=S;1goo`<000b]g003Hbb]
g:gL00B]g003002eo;Gl01:eo0<001Beg003002efkGK00fef`=S;0:ef`8000Je^`03Hbbe^kFk00Ze
^`03002e^kFk01:e^`03002e^kFk0080012eV`03002eVkFK016eV`03002eVkFK00BeV`8000ReNP05
002eN[Ej]GX0000C]GX00f</0035^@04aKT00f</aKT0000?eKL00`00Hbcoo`02ool00`00ooooo`0;
ool00`00ooooo`02ool00`00ooooo`03ool00`00ool00005ool00`00ooooo`1Gool005Koo`8001;o
o`9S;1koo`8000:]g003002]g:gL00Z]g003Hbb]g:gL00F]g003002eo;Gl00neo08000:eg003002e
g;GL01:eg003002efkGK00Zef`=S;0>ef`800003]K/00;Fk00Fe^`03Hbbe^kFk00Ze^`03002e^kFk
01:e^`<001:eV`03002eVkFK01:eV`04002eVkFK]I/2000:]GX01@00]GZeN[Ej00005;Ej00=S;<Fi
aKT00lFi00=S;000eKL03mFg00<00?ooHb`00ooo00<00?ooool02_oo00<00?ooool00_oo00<00?oo
ool00ooo00<00?oo00001Ooo00<00?ooool0Eooo001Fool2000?ool3Hb`Nool20005[M`00`00[Mb]
g009[M`00f</[Mb]g006[M`00`00]Obeo00<]O`20005]M`00`00]Mbeg00B]M`00`00]M^ef`07]M/2
Hb`4]M/20003]K/00`00]K^e^`04]K/00f</]K^e^`0;]K/00`00]K^e^`0?]K/20002]I/00`00]I^e
V`0A]I/00`00]I^eV`0A]I/01000]I/000003;Ej00<00;Ej]GX00[Ej00<00;Ej]GX04KEj00=S;000
aKT00/Fi00<006</eKL04MFg00<006</ool00ooo00<00?ooool02Ooo00<00?ooool00_oo00<00?oo
ool00ooo00<00?oo00001Ooo00<00?ooool0Eooo001Fool00`00ool0000<ool2Hb`Oool20008[M`0
0`00[Mb]g008[M`00f</[Mb]g007[M`00`00]Obeo009]O`20007]M`00`00]Mbeg00C]M`00`00]M^e
f`03]M/3Hb`4]M/20006]K/00`00]K^e^`03]K/00f</]K^e^`0<]K/00`00]K^e^`0<]K/20004]I/0
0`00]I^eV`0A]I/00`00]I^eV`0A]I/2000>]GX00`00]GZeNP02]GX00`00]GZeNP0B]GX01V</aKW5
^@00eKMS;1?E]`06001S;?oooomS;0002ooo00<00?ooool00_oo00<00?ooool00_oo00@00?ooool0
00Coo`03003ooooo05Soo`00E_oo00@00?oooomS;0Woo`9S;1ooo`8000^]g003002]g:gL00N]g003
Hbb]g:gL00R]g003002eo;Gl00Jeo08000Zeg003002eg;GL01>eg003002efkGK009S;0Fef`8000Ve
^`03002e^kFk00:e^`03Hbbe^kFk00fe^`03002e^kFk00Ve^`8000NeV`03002eVkFK016eV`03002e
VkFK00jeV`8000:]NP03002eN[Ej00^eNP03002eN[Ej00>eNP03002eN[Ej01:eNP05Hb`00=FgeKMS
;00CeKL01`00HbaS;6</oomS;00000Soo`03003oo`0000Goo`05003oooooool00003ool01@00oooo
oooo0000F_oo001Eool00`00ooooo`02ool00f</ooooo`03ool3Hb`Oool2000>[M`00`00[Mb]g006
[M`00f</[Mb]g009[M`00`00]Obeo003]O`2000=]M`00`00]Mbeg00B]M`3Hb`5]M/2000;]K/00`00
]K^e^`02]K/00f</]K^e^`0=]K/00`00]K^e^`07]K/20009]I/00`00]I^eV`0A]I/00`00]I^eV`0<
]I/20004[GX00`00]GZeNP0;]GX00`00]GZeNP03]GX00`00]GZeNP0B]GX01F</eKOE]mFgHb`05=Fg
00L006</ooooof</ool00008ool20006ool30005ool3001Kool005Goo`03003ooooo00?oo`03Hbco
oooo009S;23oo`80016]g003002]g:gL00F]g003Hbb]g:gL00Z]g003002eo;Gl0080012eg003002e
g;GL00neg09S;003]M`00;GK00>ef`8000je^`05002e^kFk]K]S;00@]K/00`00]K^e^`04]K/2000<
]I/00`00]I^eV`0A]I/00`00]I^eV`09]I/20007[GX00`00]GZeNP0:]GX00`00]GZeNP04]GX00`00
]GZeNP0?]GX200001KEIHbcE]mFgHb`04mFg00P00=ff001S;?ooHbcoo`00M_oo001Eool00`00oooo
o`03ool3Hb`Oool3000C[M`00`00[Mb]g004[M`00f</[Mb]g00;[M`3000C]M`00`00]Mbeg00;]M`3
Hb`4]M`01000]M/000004KFk00@00;Fk]K]S;16e^`04002e^kFk]K/2000?]I/00`00]I^eV`0A]I/0
0`00]I^eV`06]I/20009[GX00`00]GZeNP0:]GX00`00]GZeNP04]GX00`00]GZeNP0=]GX20003]ET0
1@00HbcE]mFgHb`04MFg00L00=ffgKH006</oomS;002ool00`00ooooo`1cool005Goo`04003ooooo
ool2Hb`2ool00f</ooooo`0Kool20002[M`00`00[Mb]g00B[M`00`00[Mb]g003[M`00f</[Mb]g009
[M`20002[M`00`00]Mbeg00A]M`00`00]Mbeg008]M`3Hb`7]M`2000C]K/01000]K^e^f</4KFk00@0
0;Fk0000016eV`03002eVkFK016eV`03002eVkFK00BeV`8000b]NP03002eN[Ej00VeNP03002eN[Ej
00FeNP03002eN[Ej00ZeNP8000JeF@04HbcE]mFgHb`AeKL02P00gKKM]]ff001S;?ooHbcoo`00MOoo
001Eool20002Hb`5ool00f</ooooo`0Gool30005[M`00`00[Mb]g00B[M`00`00[Mb]g002[M`00f</
[Mb]g007[M`20005[M`00`00]Mbeg00A]M`00`00]Mbeg005]M`2Hb`8]M`20002]K/00`00]K^e^`0A
]K/00`00]K]S;00A]K/2000D]I/00`00]I^eV`0A]I/01000]I^eVkFK0P003Zej00<00;Ej]GX02KEj
00<00;Ej]GX01KEj00<00;Ej]GX02;Ej0P002;EI00@006</eKMS;13E]`03003M]]ff00;M]P04001S
;?ooHb`2ool00`00ooooo`1bool005Goo`03001S;?oo00Ooo`03Hbcooooo01Coo`8000V]g003002]
g:gL01:]g005002]g:gL[MaS;007[M`20007[M`00`00]Mbeg00B]M`01000]Mbeg;GL0f</2;GL0P00
1;Fk00<00;Fk]K/04[Fk00<006</]K/03[Fk0P000ZfK00<00;FK]I/04KFK00<00;FK]I/04KFK00@0
0;FK0000016]NP03002eN[Ej00ReNP03002eN[Ej00JeNP03002eN[Ej00JeNP03002eFKEI00VeF@03
001S;6</00oE]`03003M]]ff00CM]P03001S;6</00?oo`03003ooooo077oo`00E?oo00<00?ooHb`0
2Ooo00=S;?ooool04?oo0`003:gL00<00:gL[M`04ZgL00@00:gL[MaS;0F]g08000Z]g003002eg;GL
01:eg003001S;6</00Reg0<000Ne^`03002e^kFk01:e^`03Hbbe^kFk00^e^`8000B]V`03002eVkFK
01:eV`03002eVkFK012eV`8001>]NP03002eN[Ej00ReNP03002eN[Ej00JeNP03002eN[Ej00BeNP80
00beF@03001S;6</00kE]`03003M]]ff00KM]P03001S;?oo00;oo`03003ooooo077oo`00:_oo0`00
0ooo0P001?oo00<00?ooool00ooo10005?oo00@00?oooomS;0Woo`03Hbcooooo00goo`80012]g003
002]g:gL01:]g003002]g6</00>]g08000f]g003002eg;GL00neg0=S;003002eg;GL00Feg08000^e
^`03002e^kFk016e^`03Hbbe^kFk00Re^`<000N]V`03002eVkFK016eV`03002eVkFK00jeV`8000:]
NP03002]NZej016]NP03002eN[Ej00ReNP03002eN[Ej00JeNP04002eN[Ej]GX2000?]ET00`00HbcE
]`0<eKL00`00gKKM]P07gKH01000Hbcooooo0`00L_oo000Yool01@00oooooooo00000ooo00<00?oo
ool00_oo00<00?ooool01ooo00<00?ooool04Ooo00D00?ooooooof</00Woo`03Hbcooooo00Woo`<0
01>]g003002]g:gL01:]g003001S;:gL0080012]g003002eg;GL00beg09S;0>eg003002eg;GL00>e
g08000je^`03002e^kFk012e^`03Hb`00;Fk00Je^`8000^]V`03002eVkFK016eV`03002eVkFK00^e
V`8000B]NP03002]NZej01:]NP03002eN[Ej00JeNP03Hb`00;Ej00NeNP04002eNP00000A]ET00`00
HbcE]`0<eKL00`00gKKM]P08gKH2Hb`3ool00`00ooooo`1`ool002Woo`04003oooooool20008ool0
0`00ooooo`07ool00`00ooooo`0@ool00`00ooooo`02ool00f</ooooo`07ool00f</ooooo`06ool2
0002[M`00`00[Mb]g00A[M`00`00[Mb]g00B[M`00`00Hb`0000C[M`00`00]Mbeg008]M`3Hb`6]M`0
0`00]Mbeg002000@]K/00`00]K^e^`0@]K/00f</]K/00004]K/2000=[I/00`00]I^eV`0B]I/00`00
]I^eV`07]I/30007[GX00`00[GZ]NP0A[GX00`00]GZeNP03]GX3Hb`00kEj001S;007]GX2000D]ET2
Hb`;eKL00`00gKKM]P09gKH00f</001S;003ool00`00ooooo`1_ool002Soo`03003ooooo00?oo`03
003ooooo00Koo`03003ooooo00Ooo`03003ooooo013oo`03003ooooo00?oo`03Hbcooooo00Ooo`03
Hbcooooo00;oo`<000F]g003002]g:gL016]g003002]g:gL00n]g0800003[MaS;00001>]g003002e
g;GL00Jeg09S;0Veg0<001>e^`03002e^kFk00ne^`03Hbbe^`0000:e^`80012]V`03002eVkFK016e
V`03002eVkFK00FeV`8000Z]NP03002]NZej01:]NP03002eN[Ej009S;0BeNP03001S;;Ej00BeNP80
00:]>P03002eFKEI016eF@03HbcE]f</00WE]`03003M]]ff00[M]P03Hb`006</00Coo`03003ooooo
06koo`00:?oo00<00?ooool00ooo00<00?ooool01_oo00<00?ooool01ooo00<00?ooool04?oo00<0
0?ooool01?oo00=S;?ooool01ooo00AS;?oo000000V]g003002]g:gL016]g003002]g:gL00b]g080
00>]g003Hbb]g00001>]g003002eg;GL00:eg0=S;0Veg08000:]^`03002e^kFk01:e^`03002e^kFk
00je^`03Hbbe^`00008001>]V`03002eVkFK016eV`03002eVkFK00:eV`8000f]NP03002]NZej016]
NP=S;0JeNP03002eNV</00:eNP8000B]>P03002eFKEI016eF@03Hb`006</00SE]`03003M]]ff00_M
]P04HbcM]P00Hb`3ool00`00ooooo`1^ool002Soo`03003ooooo00?oo`03003ooooo00Ooo`03003o
oooo00Goo`80017oo`03003ooooo00Koo`03Hbcooooo00Goo`<000b]g003002]g:gL016]g003002]
g:gL00V]g08000B]g005Hbb]g:gL[M`0000C[M`01000HbaS;6</2[GL0P001Jfk00<00;Fk]K/04[Fk
00<00;Fk]K/03KFk00ES;000[K^]^`0001>]V`03002eVkFK016eV`03002eVkFK0080012]NP03002]
NZej00f]NP=S;003[GX00;Ej00JeNP05002eNP00Hb`00007[CX00`00]EVeF@0@]ET016</003E]f</
1mFg00<00=ffgKH03=ff00=S;000Hb`01?oo00<00?ooool0KOoo000Xool00`00ooooo`03ool00`00
ooooo`07ool00`00ooooo`03ool3000Bool30007ool00f</ooooo`02ool2000@[M`00`00[Mb]g00A
[M`00`00[Mb]g006[M`20006[M`01F</[Mb]g:gL00004ZgL0V</00<00;GL]M`02;GL0P001jfk00<0
0;Fk]K/04[Fk00<00;Fk]K/02kFk0P0000ES;:fk[K^]^`0001B]V`03002eVkFK012eV`<001:]NP03
002]NZej00^]NP9S;0B]NP03002eN[Ej00FeNP800003[CYS;:dj00N]>P03002eFKEI012eF@05Hbbe
F@00eKMS;005eKL00`00gKKM]P0=gKH016</gKH006</1?oo00<00?ooool0K?oo000Xool00`00oooo
o`03ool00`00ooooo`07ool01000oooooooo10004ooo00<00?ooool02?oo00AS;000000001>]g003
002]g:gL016]g003002]g:gL00>]g08000R]g003Hbb]g:gL00:]g003002]g:gL00b]g0=S;0>]g003
002eg;GL00Feg08000Z]^`03002e^kFk01:e^`03002e^kFk00Re^`8000:]^`03Hbb]^jfk00:]^`03
002]VjfK01:]V`03002eVkFK00feV`8000:]N`03002]NZej016]NP03002]NZej00N]NP=S;0N]NP03
002eN[Ej00:eNP<000>]>P03Hbb]>Zdj00J]>P03002eFKEI012eF@04Hb`00=FgHb`4eKL00`00gKKM
]P0>gKH01F</gKH00000Hb`00ooo00<00?ooool0K?oo000Xool00`00ooooo`02ool00`00ooooo`03
ool00`00ooooo`03ool01@00oooooooo00005Ooo00<00?ooool01ooo0P000ZgL00<00:gL[M`04JgL
00<00:gL[M`04ZgL00<00:gL[M`00P002ZgL00=S;:gL[M`00jgL00<00:gL[M`02JgL0V</1ZgL00<0
0;GL]M`00kGL0P003Jfk00<00;Fk]K/04[Fk00<00;Fk]K/01KFk0P001:fk00=S;:fk[K/00jfk00<0
0:fK[I/04ZfK00<00;FK]I/02[FK0P001:ek00<00:ej[GX04Jej00<00:ej[GX01:ej0f</2Zej00<0
0;Ej]GX00P000Zdj00<00:dj[CX00Zdj00=S;:dj[CX01Jdj00<00;EI]ET04;EI00ES;;EI003E]f</
00;E]`03003M]]ff00oM]P05Hb`00>FD001S;004ool00`00ooooo`1[ool002Woo`05003oooooool0
0005ool00`00ooooo`03ool01@00oooooooo00005Ooo00<00?ooool01Ooo0P001JgL00<00:gL[M`0
4JgL00<00:gL[M`04JgL0`003:gL00=S;:gL[M`01:gL00<00:gL[M`01JgL0f</2JgL00<00;GL]M`0
0P004:fk00<00;Fk]K/04[Fk00<00;Fk]K/00[Fk0P001Zfk00=S;:fk[K/00jfk00<00:fK[I/04ZfK
00<00;FK]I/02;FK0P001jek00<00:ej[GX04Jej00<00:ej[GX00f</3Jej0`001:dj00<00:dj[CX0
0Zdj00=S;:dj[CX01Zdj00<00;EI]ET03kEI00=S;;EI00000]Fg00=S;000gKH04Mff00IS;000iICU
U000Hb`2ool2001]ool002[oo`<000Ooo`H000Coo`@0017oo`03003ooooo00;oo`<000R]g003002]
g:gL016]g003002]g:gL00j]g08000:]g003002]g:gL00Z]g003Hbb]g:gL00F]g003002]g:gL00:]
g09S;0b]g0<001>]^`03002e^kFk016e^`03002e^kFk008000R]^`03Hbb]^jfk00B]^`03002]VjfK
01:]V`03002eVkFK00FeV`8000Z]N`03002]NZej012]NP03001S;6</00j]NP800003[CX00:dj00B]
>P03002]>Zdj00>]>P03Hbb]>Zdj00F]>P03002eFKEI00neF@07HbbeFKEI003E]f</00004]ff00IS
;>FDiICUU000Hb`4ool00`00ooooo`1Zool005;oo`04003oooooool2000<[M`00`00[Mb]g00A[M`0
0`00[Mb]g00:[M`30005[M`00`00[Mb]g009[M`00f</[Mb]g006[M`01000HbaS;6</3:gL0P000Zfl
00<00:fk[K/04Jfk00<00;Fk]K/04KFk0`002Zfk00=S;:fk[K/01Jfk00<00:fK[I/04ZfK00<00;FK
]I/00[FK0P003:ek00<00:ej[GX03Zej0f</00<00:ej[GX02jej0P001:dj00<00:dj[CX00Zdj00<0
0:dj[CX01:dj00=S;:dj[CX01Jdj00<00;EI]ET03[EI00MS;;EI]ET00=Fg001S;00BgKH00f</iICU
U002iI@00`00Hbcoo`03ool00`00ooooo`1Yool005;oo`04003oo`00000?[M`00`00[Mb]g00A[M`0
0`00[Mb]g007[M`20009[M`00`00[Mb]g008[M`00f</[Mb]g005[M`2Hb`00`00[Mb]g009[M`30005
[K`00`00[K^]^`0A[K/00`00]K^e^`0>]K/20002[K/00`00[K^]^`08[K/00f</[K^]^`05[K/00`00
[I^]V`0B[I/00`00]I^eV`02000?[G/00`00[GZ]NP0:[GX3Hb`4[GX00`00[GZ]NP07[GX30006[CX0
0`00[CZ]>P02[CX00`00[CZ]>P05[CX00f</[CZ]>P04[CX00`00]EVeF@0>]ET01F</]EVeFKEI0000
0]ff00=S;=ffgKH03]ff00<006</iI@01>FD00<006</ool00_oo00<00?ooool0JOoo001Bool2000B
[M`00`00[Mb]g00A[M`00`00[Mb]g003[M`3000;[M`00`00[Mb]g008[M`00f</[Mb]g002[M`3Hb`2
[M`00`00[Mb]g007[M`20009[K`00`00[K^]^`0A[K/00`00]K^e^`0;]K/20004[K/00`00[K^]^`08
[K/00f</[K^]^`06[K/00`00[I^]V`0A[I/3000B[G/00`00[GZ]NP06[GX3Hb`7[GX00`00[GZ]NP05
[GX2000:[CX01@00[CZ]>Zdj00001jdj00=S;:dj[CX01Jdj00<00;EI]ET03KEI00=S;;EI00000P00
0]ff00=S;=ffgKH03]ff00<006</iI@01>FD00<006</ool00ooo00<00?ooool0J?oo001Bool2000C
[M`00`00[Mb]g00A[M`00`00[Mb]g002000?[M`00`00[Mb]g007[M`00f</[MaS;002Hb`6[M`00`00
[Mb]g003[M`3000;[K`00`00[K^]^`0B[K/00`00]K^e^`08]K/20007[K/00`00[K^]^`07[K/00f</
[K^]^`07[K/00`00[I^]V`0>[I/20002[G/00`00[G^]N`0@[G/00`00[GZ]NP04[GX2Hb`;[GX00`00
[GZ]NP02[GX2000<[CX01@00[CZ]>Zdj00002:dj00=S;:dj[CX01:dj00<00;EI]ET03KEI00=S;000
]AP00[DH00@00=ffgKIS;0kM]P04003UU>FDHb`5iI@00`00Hbcoo`02ool00`00ooooo`1Xool005;o
o`<001>]g003002]g:gL012]g0<001:]g003002]g:gL00J]g09S;0Z]g003002]g:gL008000n]_003
002]^jfk01:]^`03002e^kFk00Fe^`8000Z]^`03002]^jfk00J]^`03Hbb]^jfk00R]^`03002]VjfK
00^]V`8000B]N`03002]Njek016]N`03002]NZej00=S;0f]NP03002]NZej008000n]>P04002]>Zdj
0009[CX00f</[CZ]>P04[CX00`00]EVeF@0:]ET200000f</]ARe6002]AP01@00gKKM]]ffHb`03Mff
00@00>FDiIAS;0KUU003001S;?oo00;oo`03003ooooo06Ooo`00D_oo00@00?ooool001>]g003002]
g:gL00f]g08000:]g003002]g:gL016]g003002]g:gL00:]g0=S;0b]g0<001:]_003002]^jfk016]
^`03002e^kFk00>e^`8000f]^`03002]^jfk00F]^`03Hbb]^jfk00R]^`03002]VjfK00R]V`<000N]
N`03002]Njek012]N`=S;12]NP<0016]>P04002]>Zdj0009[CX00f</[CZ]>P04[CX00`00]EVeF@08
]ET20002]AP00f</]ARe6003]AP01000gKKM]V</3=ff00D00>FDiICUU6</00KUU003001S;?oo00?o
o`03003ooooo06Koo`00D_oo00D00?ooooooo`0001>]g003002]g:gL00Z]g08000F]g003002]g:gL
016]g004002]g6</Hb`2[M`00f</[Mb]g008[M`20002[K`00`00[Kb]_00A[K`00`00[K^]^`0A[K/0
0`00]K^e^`02000?[K/00`00[K^]^`05[K/00f</[K^]^`09[K/00`00[I^]V`05[I/2000:[G/00`00
[G^]N`0>[G/2Hb`00jek002]NP0>[GX200000jdj002]>P0B[CX01000[CZ]>P002Jdj00=S;:dj[CX0
1:dj00<00;EI]ET01[EI00<00;DH]AP00[DH00=S;;DH]AP00kDH00D00=ffgKKM]V</00_M]P05003U
U>FDiIAS;007iI@00`00Hbcoo`02ool00`00ooooo`1Vool0057oo`03003ooooo00?oo`03002]g:gL
016]g003002]g:gL00J]g0<000R]g003002]g:gL00n]g0=S;0B]g003Hbb]g:gL00J]g08000F]_003
002]_:fl016]_003002]^jfk012]^`<001:]^`03002]^jfk00B]^`03Hbb]^jfk00Z]^`03002]VjfK
00:]V`8000f]N`03002]Njek00Z]N`=S;0B]N`03002]NZej00Z]NP8000B]>P03002]>Zdj012]>P04
002]>Zdj000:[CX00f</[CZ]>P03[CX00`00]EVeF@04]ET20005]AP00f</]ARe6003]AP00`00gKKM
]P02gKH00f</gKKM]P07gKH00`00iICUU002iI@00f</iICUU005iI@01`00Hbcooooo003oo`0006Oo
o`00DOoo00<00?ooool01?oo00<00:gL[M`04JgL00<00:gL[M`00jgL0P003:gL00<00:gL[M`03:gL
0V</0jgL00D00:gL[Mb]g6</00J]g08000N]_003002]_:fl01:]_003002]^jfk00f]^`8000:]^`03
002]^jfk016]^`03002]^jfk00>]^`03Hbb]^jfk00Z]^`03002]VjfK0080012]N`03002]Njek00J]
N`=S;0N]N`03002]NZej00R]NP8000J]>P03002]>Zdj016]>P03002]>P0000Z]>P03Hbb]>Zdj00B]
>P04002eFKEI]ET20008]AP00f</]ARe6003]AP01@00gKKM]]ffHb`02Mff00<00>FDiI@00^FD00=S
;>FDiI@01^FD00<006</ool00ooo00<00?ooool0I?oo001Aool00`00ooooo`04ool00`00[Mb]g00B
[M`00`00[Mb]g002000?[M`00`00[Mb]g008[M`3Hb`5[M`01@00[Mb]g:gLHb`01:gL0P002Zfl00<0
0:fl[K`04Zfl00<00:fk[K/02Jfk0`001:fk00<00:fk[K/04Zfk00<00:fk[K/00Zfk00=S;:fk[K/0
2Zfk0`004Zek00<00:ek[G/00jek0f</2jek00<00:ej[GX01Jej0P002Jdj00<00:dj[CX04:dj00<0
0:dj00002jdj00=S;:dj[CX00jdj00@00;EI000000Ze6003Hbbe6;DH00>e6003003M]]ff00;M]P03
HbcM]]ff00GM]P03003UU>FD00?UU003HbcUU>FD00KUU0800005Hbcoooooool0001Vool0057oo`03
003ooooo00Goo`03002]g:gL016]g0<001:]g003002]g:gL00F]g09S;0V]g004002]g:gLHb`2[M`2
000=[K`00`00[Kb]_00A[K`00`00[K^]^`07[K/20008[K/00`00[K^]^`0A[K/00`00[K^]^`02[K/0
0f</[K^]^`08[K/20002[I/00`00[G^]N`0A[G/00`00[G^]N`02Hb`?[G/00`00[GZ]NP02[GX2000;
[CX00`00[CZ]>P0A[CX2000<[CX00f</[CZ]>P02[CX2000<]AP00f</]ARe6004]AP01@00gKKM]]ff
Hb`01mff00<00>FDiI@00nFD00=S;>FDiI@01NFD00@00>Eb001S;0Coo`03003ooooo06?oo`00DOoo
00<00?ooool01_oo00<00:gL[M`03ZgL0P000ZgL00<00:gL[M`04:gL00<00:gL[M`00ZgL0f</3:gL
00<00:gLHb`00P004:fl00<00:fl[K`04Jfl00<00:fk[K/00jfk0`002jfk00<00:fk[K/04Jfk00D0
0:fk[K^]^f</00N]^`<000B]V`03002]Njek016]N`=S;16]N`03002]NZej008000j]>P03002]>Zdj
012]>P8000b]>P03Hbb]>Zdj00800003[?T00;DH00^e6003Hbbe6;DH00Be6003003M]]ff00;M]P03
HbcM]]ff00?M]P03003UU>FD00CUU003HbcUU>FD00GUU005003UL^Eb001S;003ool00`00ooooo`1S
ool0057oo`03003ooooo00Ooo`03002]g:gL00Z]g0<000F]g003002]g:gL012]g004001S;6</Hb`?
[M`200000f</[Kb]_00A[K`00`00[Kb]_00A[K`00`00[K^]^`02000?[K/00`00[K^]^`0A[K/00`00
[K]S;006[K/20008[I/00`00[G^]N`0=[G/3Hb`00jek002]N`0A[G/3000@[CX00`00[CZ]>P0A[CX0
0`00[CZ]>P0;[CX00f</002/n@03[?T00`00]ARe6009]AP00f</]ARe6004]AP00`00gKKM]P03gKH0
0f</gKKM]P02gKH00`00iICUU005iI@00f</iICUU004iI@01@00iG;ULP00Hb`01?oo00<00?ooool0
H_oo001@ool00`00ooooo`09ool00`00[Mb]g007[M`20009[M`00`00[Mb]g00>[M`2Hb`00`00[Mb]
g00=[M`20002[K`00f</[Kb]_00A[K`00`00[Kb]_00A[K`3000A[K/00`00[K^]^`0B[K/00`00Hbb]
^`02[K/3000;[I/00`00[G^]N`09[G/3Hb`4[G/00`00[G^]N`0>[G/20002[CX00`00[CZ]>P0?[CX0
0`00[CZ]>P0@[CX00`00[CZ]>P09[CX200000jciHbb/n@03[?T00`00]ARe6009]AP00f</]ARe6005
]AP00`00gKKM]P02gKH01F</gKKM]]ff00002>FD00=S;>FDiI@01>FD00H00>EbiG;ULP00Hb`4ool0
0`00ooooo`1Qool0053oo`<000[oo`03002]g:gL00>]g0<000^]g003002]g:gL00^]g0=S;0>]g003
002]g:gL00Z]g08000B]_003Hb`00:fl01:]_003002]_:fl00j]_08000:]^`03002]^jfk012]^`03
002]^jfk016]^`03001S;:fk008000j]V`03002]Njek00N]N`9S;0R]N`03002]Njek00^]N`8000B]
>P03002]>Zdj00n]>P03002]>Zdj012]>P8000R]>P8000>/n@03Hbb/nJci00>/n@03002e6;DH00Re
6003Hbbe6;DH00Fe6003003M]]ff00?M]P04HbcM]]ff0008iI@00f</iICUU003iI@00`00iG;ULP03
iG800`00Hbcoo`02ool2001Rool0053oo`03003ooooo00_oo`03002]g:gL008000n]g003002]g:gL
00R]g09S;0N]g003002]g:gL00J]g0<000F]_004Hbb]_:fl000C[K`00`00[Kb]_00;[K`20004[K/0
0`00[K^]^`0A[K/00`00[K^]^`0@[K/00`00Hb`0000A[I/00`00[G^]N`03[G/3Hb`;[G/00`00[G^]
N`07[G/30007[CX00`00[CZ]>P0?[CX00`00[CZ]>P0?[CX20006[CX20006[?T00f</[?V/n@02[?T0
0`00]ARe6008]AP00f</]ARe6006]AP00`00gKKM]P03gKH00f</003UU008iI@00f</iICUU003iI@0
0`00iG;ULP03iG801`00Hbcooooo003oo`0000goo`D000;oo`@000Koo`<0047oo`00D?oo00<00?oo
ool03?oo0P004ZgL00<00:gL[M`01:gL0f</2ZgL00<00:gL[M`00jgL0P002:fl00AS;:fl[K`001B]
_003002]_:fl00R]_08000N]^`03002]^jfk016]^`03002]^jfk00f]^`800003[I]S;00001:]V`03
002]Nf</009S;0j]N`03002]Njek00F]N`8000Z]>P03002]>Zdj00n]>P03002]>Zdj00n]>P03002]
>P0000>]>P8000V/n@03Hbb/nJci00:/n@03002e6;DH00Ne6003Hbbe6;DH00Je6003003M]]ff00?M
]P03Hb`00>FD00SUU003HbcUU>FD00?UU003003UL^Eb00CULP03001S;?oo00;oo`03003ooooo00go
o`03003ooooo00Koo`03003ooooo00;oo`05003oooooool00010ool0053oo`03003ooooo00goo`03
002]g:gL016]g004002]g:gL[M`2Hb`>[M`00`00[Mb]g002000:[K`01F</[Kb]_:fl00005:fl00<0
0:fl[K`01Jfl0P002Zfk00<00:fk[K/04:fk00<00:fk[K/02jfk0P000jfK00=S;000[I/04JfK0V</
4Zek00<00:ek[G/00Zek0P003Jdj00<00:dj[CX03jdj00<00:dj[CX03Zdj00@00:dj002]>P8000b/
n@05Hbb/nJci[?T00009]AP00f</]ARe6007]AP00`00gKKM]P02gKH00`00HbcUU008iI@00f</iICU
U002iI@00`00iG;ULP06iG800`00Hbcoo`02ool00`00ooooo`0<ool00`00ooooo`07ool01@00oooo
oooo00000ooo0P00?ooo001@ool00`00ooooo`0>ool00`00[Mb]g00A[M`3Hb`@[M`3000<[K`00f</
[Kb]_002[K`00`00[Kb]_00A[K`00`00[Kb]_003[K`2000=[K/00`00[K^]^`0@[K/00`00[K^]^`08
[K/20005[I/00f</[I/0000>[I/3Hb`00jfK002]N`0A[G/00`00[G^]N`02000?[CX00`00[CZ]>P0?
[CX00`00[CZ]>P0>[CX01000[CX000003Zci00=S;:ci[?T00Zci00<00;DH]AP01[DH00=S;;DH]AP0
1kDH00<00=ffgKH00]ff00<00>FDHb`02>FD00=S;>FDiI@00^FD00<00>EbiG801^Eb00<006</ool0
0ooo00<00?ooool02ooo00<00?ooool01ooo00@00?ooool000Goo`03003ooooo03goo`00Cooo00<0
0?ooool04?oo00<00:gL[M`03ZgL0V</00>]g000[M`03ZgL0P000Zfl00<00:fl[K`02Zfl00=S;:fl
[K`00jfl00<00:fl[K`04Jfl00<00:fl[K`00P003jfk00<00:fk[K/04Jfk00<00:fk[K/01Jfk0P00
1jfK00AS;:fK[I/000Z]V`=S;0F]V`03002]Njek00n]N`<001:]>P03002]>Zdj00n]>P03002]>Zdj
00f]>P800003[?T00:ci00j/n@05Hbb/nJci[?T00009]AP00f</]ARe6006]AP01@00gKKM]]ff0000
0^FD00=S;>FDiI@01^FD00=S;>FDiI@00^FD00<00>EbiG801nEb00<006</ool00_oo00<00?ooool0
2ooo00<00?ooool01ooo00@00?ooool000Goo`03003ooooo03goo`00Cooo00<00?ooool04Ooo00<0
0:gL[M`02ZgL0f</1:gL00<00:gL[M`02JgL0`001Jfl00<00:fl[K`02Jfl00=S;:fl[K`01:fl00<0
0:fl[K`04:fl0`004Zfk00<00:fk[K/04Jfk00<00:fk[K/00Zfk0P002JfK00AS;:fK[I/000N]V`=S
;0V]V`03002]Njek00b]N`8000:]FP03002]>Zdj012]>P03002]>Zdj00n]>P03002]>Zdj00^]>P<0
00>/n@03002/nJci00f/n@05Hbb/nJci[?T00008]AP00f</]ARe6007]AP01000gKKM]P000nFD00=S
;>FDiI@01NFD00ES;>FDiICUU00000[ULP03001S;?oo00?oo`03003ooooo00[oo`03003ooooo00Ko
o`8000;oo`03003ooooo00?oo`03003ooooo03goo`00Cooo00<00?ooool04_oo00<00:gL[M`01ZgL
0f</1jgL00<00:gL[M`01jgL0P002Jfl00<00:fl[K`02:fl00=S;:fl[K`01Jfl00<00:fl[K`03Jfl
0P000Zfk00<00:fk[K/04Jfk00<00:fk[K/04:fk00<00:fk[K/00P002jfK00ES;:fK[I^]V`0000B]
V`9S;0b]V`03002]Njek00V]N`<000B]FP03002]>Zdj016]>P03002]>Zdj00n]>P03002]>Zdj00N]
>P<000:/n@05002/nJci[?T0000?[?T01F</[?V/nJci00002;DH00=S;;DH]AP01kDH00<00=ff0000
1NFD00=S;>FDiI@01NFD00AS;>FDiI@000_ULP03001S;?oo00?oo`03003ooooo00Woo`03003ooooo
00Coo`<000?oo`03003ooooo00?oo`03003ooooo03goo`00Cooo00<00?ooool04ooo00<00:gL[M`0
0jgL0V</2jgL00<00:gL[M`00jgL0`003:fl00<00:fl[K`01jfl00=S;:fl[K`01Zfl00<00:fl[K`0
2Jfl0`001Jfk00<00:fk[K/04Jfk00<00:fk[K/03jfk0`003JfK00=S;:fK[I/00ZfK00@006</HbaS
;0n]V`03002]Njek00J]N`8000R]FP03002]>Zdj012]>P03002]>Zdj00n]>P03002]>Zdj00F]>P80
00F/n@03002/nJci00:/n@03002/nJci00f/n@04Hbb/nJci0008]AP00f</]ARe6008]AP20005iI@0
0f</iICUU005iI@016</iICUU0002nEb0P0000IS;?ooooooo`00Hb`:ool00`00ooooo`02ool40004
ool00`00ooooo`03ool00`00ooooo`0mool004ooo`8001Goo`03002]g6</009S;0j]g003002]g:gL
0080012]_003002]_:fl00J]_003Hbb]_:fl00J]_003002]_:fl00N]_08000R]^`03002]^jfk016]
^`03002]^jfk00f]^`8000:]V`03002]VjfK00^]V`03Hbb]VjfK00=S;1>]V`03002]Njek00:]N`<0
00Z]FP03002]>Zdj016]>P03002]>Zdj00n]>P03002]>Zdj00:]>P8000R/n@05002/nJci[?T0000@
[?T016</[?V/n@001kDH00=S;;DH]AP02;DH00<00>FDiI@01NFD00=S;>FDiI@01>FD00AS;>FDiI@0
00_ULP8000AS;003ool00?oo00Woo`03003ooooo00?oo`03003ooooo00Coo`03003ooooo00;oo`03
003ooooo03koo`00Cooo00<00?ooool05?oo0V</4JgL0`004jfl00<00:fl[K`01Jfl00=S;:fl[K`0
1jfl00<00:fl[K`00jfl0`002jfk00<00:fk[K/04Jfk00<00:fk[K/02Jfk0`001:fK00<00:fK[I/0
2jfK0f</0jfK00<00:fK[I/04:fK00<00:ek[G/00P003ZeJ00<00:dj[CX04:dj00<00:dj[CX03jdj
00<00:dj[CX00P002Zci00<00:ci[?T00Zci00<00:ci[?T03Jci00AS;:ci[?T000Ne6003Hbbe6;DH
00Je60<000SUU003HbcUU>FD00?UU003HbcUU00000cULP04003UD@00Hb`3ool00`00ooooo`06ool0
0`00ool00005ool00`00ooooo`05ool01@00oooooooo0000@?oo001>ool00`00ooooo`0Bool3Hb`2
ool00`00[Mb]g00;[M`30002[K`00`00[Kb]_00A[K`00`00[Kb]_005[K`00f</[Kb]_008[K`00`00
[Kb]_002000?[K/00`00[K^]^`0A[K/00`00[K^]^`06[K/20008[I/00`00[I^]V`08[I/3Hb`6[I/0
0`00[I^]V`0?[I/3000@[EX00`00[CZ]>P0A[CX00`00[CZ]>P0>[CX3000<[?T00`00[?V/n@02[?T0
0`00[?V/n@0>[?T016</[?V/n@001[DH00=S;;DH]AP01;DH0P000kCG00<00>FDiI@01NFD00=S;>FD
iI@00nFD00=S;>FD00002nEb00D00>EAiE7UDF</00?oo`03Hb`00?oo00Ooo`8000Koo`@000Coo`<0
047oo`00C_oo00<00?ooool04?oo0V</1_oo00<00:gL[M`01jgL0`001Zfl00<00:fl[K`04Jfl00<0
0:fl[K`00jfl00=S;:fl[K`02Jfl0`004Zfk00<00:fk[K/04Jfk00<00:fk[K/00Zfk0`002jfK00<0
0:fK[I/01:fK0f</0ZfK00=S;:fK[I/01JfK00<00:fK[I/03:fK0P000ZeJ00<00:eJ[EX03jeJ00<0
0:dj[CX04:dj00<00:dj[CX03:dj0P0000>Tn@00[?T03:ci00<00:ci[?T00jci00<00:ci[?T03Zci
00=S;:ci00001[DH00=S;;DH]AP00kDH00<00;CG]=L00kCG00<00>FDiI@01^FD00=S;>FDiI@00^FD
00=S;>FD00002nEb00H00>EAiE7UD@00Hb`2ool00f</ool0001Kool004koo`03003ooooo00goo`=S
;0Woo`03002]g:gL00B]g08000Z]_003002]_:fl016]_003002]_:fl00:]_003Hbb]_:fl00J]_0<0
00:]^`03002]^jfk012]^`03002]^jfk016]^`03002]^jfk008000n]V`03002]VjfK00=S;0F]V`03
Hbb]VjfK00F]V`03002]VjfK00V]V`<000B]FP03002]FZeJ00n]FP03002]>Zdj016]>P03002]>Zdj
00R]>P<000BTn@03002/nJci00Z/n@03002/nJci00>/n@03002/nJci00j/n@04Hbb/nJci0005]AP0
16</]ARe6;DH0P001kCG00<00>FDiI@01^FD00IS;>FDiICUU6</000<iG800`00iE7UD@02iE401P00
HbcoooooHb`005_oo`00C_oo00<00?ooool02ooo0V</3Ooo00<00:gL[M`00`003Jfl00<00:fl[K`0
4Jfl00D00:fl[Kb]_6</00F]_0<000J]^`03002]^jfk012]^`03002]^jfk012]^`<0016]V`03001S
;6</00R]V`03Hbb]VjfK00J]V`03002]VjfK00J]V`8000R]FP03002]FZeJ00n]FP03002]>Zdj012]
>P03002]>Zdj00J]>P8000NTn@03002/nJci00Z/n@03002/nJci00B/n@03002/nJci00j/n@03Hbb/
n@0000Fe6004Hbbe60000009]=L00`00iICUU006iI@01V</iICUU>FDHb`000cULP03003UDNEA00?U
D@06HbcoooooHbcoo`00F_oo001>ool00`00ooooo`08ool3Hb`@ool2000A[K`00`00[Kb]_00A[K`0
1000[Kb]_6</0jfl0P002Zfk00<00:fk[K/04:fk00<00:fk[K/03Jfk0P000ZfK00<00:fK[I/03JfK
0f</00<00:fK[I/01jfK00=S;:fK[I/01jfK00<00:fK[I/00ZfK0`002ZeJ00<00:eJ[EX04:eJ00<0
0:dj[CX04:dj00<00:dj[CX00Zdj0`002ZCi00<00:ci[?T02Jci00<00:ci[?T01:ci00<00:ci[?T0
3jci00=S;:ci00001;DH00=S;000]=L02[CG00<00>FDiI@01nFD00ES;>FDiIAS;00000cULP03003U
DNEA00?UD@04001S;?ooHb`2ool00`00ooooo`1Gool004koo`03003ooooo00Goo`=S;1Coo`03002]
_:fl012]_003002]_:fl016]_003002]_6</00<000b]^`03002]^jfk016]^`03002]^jfk00V]^`<0
00B]V`03002]VjfK00Z]V`=S;0B]V`03002]VjfK00J]V`03Hbb]VjfK00N]V`03002]VjfK008000j]
FP03002]FZeJ00n]FP03002]>Zdj012]>P03002]>Zdj008000fTn@03002/nJci00V/n@03002/nJci
00B/n@03002/nJci00n/n@03Hbb/n@0000>e6003001S;;CG00bde`03003UU>FD00KUU005HbcUU>FD
iIAS;00<iG800`00iE7UD@04iE400`00HbaS;002ool00`00ooooo`1Gool004koo`03003ooooo00?o
o`9S;1Soo`03002]_:fl012]_003002]_:fl012]_0800003Hbb]^jfk00j]^`03002]^jfk016]^`03
002]^jfk00J]^`8000R]V`03002]VjfK00J]V`=S;0R]V`03002]VjfK00F]V`03Hbb]VjfK00N]V`<0
012]FP03002]FZeJ012]FP03002]>Zdj00n]>P<0012Tn@03002/nJci00R/n@03002/nJci00F/n@03
002/nJci00n/n@03Hbb/n@00008000:de`03HbbdekCG00Zde`03003UU>FD00OUU004HbcUU000Hb`;
iG800`00iE7UD@06iE42Hb`01?oo003oo`00F?oo001=ool01000oooooooo0f</6ooo00<00:fl[K`0
4:fl00<00:fl[K`03Jfl0P000Zfl00=S;:fk[K/03jfk00<00:fk[K/04:fk00<00:fk[K/00jfk0`00
2jfK00<00:fK[I/00jfK0V</2jfK00<00:fK[I/01JfK00=S;:fK[I/01JfK0P000ZeK00<00:eJ[EX0
3jeJ00<00:eJ[EX03jeJ00<00:dj[CX03:dj0`0000>Tn@00Y?T04:Ci00<00:ci[?T02:ci00<00:ci
[?T01Jci00<00:ci[?T04:ci00=S;000]=L00kCG00=S;;CG]=L02[CG00<00>FDiI@02>FD00=S;000
Hb`02nEb00<00>EAiE401^EA00<006</ool00_oo00<00?ooool0E_oo001=ool01000oomS;6</7ooo
00<00:fl[K`04:fl00<00:fl[K`02Zfl0P001:fl00=S;000[K/04:fk00<00:fk[K/04:fk00<00:fk
[K/00P003jfK00<00:fKHb`00V</3ZfK00<00:fK[I/01:fK00=S;:fK[I/00ZfK0`001:eK00<00:eJ
[EX03jeJ00<00:eJ[EX04:eJ00<00:dj[CX02Jdj0P001JCi00<00:CiY?T03jCi00<00:ci[?T01jci
00<00:ci[?T01Zci00<00:ci[?T03Jci0P0000>/^6</]=L00kCG00=S;;CG]=L02kCG00<00>FDiI@0
1nFD00=S;000Hb`02nEb00<00>EAiE401nEA00=S;?ooool00_oo00<00?ooool0EOoo001=ool00`00
Hbcoo`0Qool00`00[Kb]_00@[K`00`00[Kb]_006[K`30006[K`00f</[K`0000A[K/00`00[K^]^`0?
[K/3000@[I/3Hb`B[I/00`00[I^]V`03[I/00f</[I^]V`020008[E/00`00[EZ]FP0?[EX00`00[EZ]
FP0?[EX00`00[CZ]>P06[CX30007Y?T00`00Y?VTn@0?Y?T00`00[?V/n@07[?T00`00[?V/n@06[?T0
0`00[?V/n@0;[?T20003[;P00f</002de`02]=L00f</]=Nde`0;]=L00`00iICUU007iI@00`00HbaS
;00;iG800`00iE7UD@07iE42Hb`4ool00`00ooooo`1Dool004goo`03001S;?oo02;oo`03002]_:fl
012]_003002]_:fl00>]_08000V]_003Hbb]_00001:]^`03002]^jfk00b]^`8000:]V`03002]VjfK
00^]V`=S;0:]V`03002]VjfK016]V`03002]VjfK00:]V`03Hb`0000000Z]F`03002]FZeJ012]FP03
002]FZeJ00n]FP03002]>Zdj00:]>P<000^Tn@03002TnJCi00nTn@03002/nJci00J/n@03002/nJci
00N/n@03002/nJci00R/n@8000J/^005HbbdekCG]=MS;00>]=L00`00iICUU006iI@00`00iG9S;00;
iG800`00iE7UD@07iE400`00HbaS;003ool00`00ooooo`1Dool003koo`D000[oo`03003oof</02;o
o`03002]_:fl016]_003002]_:fl008000Z]_005Hbb]_:fl[K`0000B[K/00`00[K^]^`09[K/20004
[I/00`00[I^]V`09[I/2Hb`6[I/00`00[I^]V`0@[I/00`00[I^]V`0200000f</[E^]F`0;[E/00`00
[EZ]FP0?[EX00`00[EZ]FP0?[EX00`00[CZ]>P02000>Y?T00`00Y?VTn@0?Y?T00`00[?V/n@06[?T0
0`00[?V/n@07[?T00`00[?V/n@06[?T20008[;P01@00HbbdekCGHb`03[CG00<00>FDiI@01^FD00<0
0>EbHb`02nEb00<00>EAiE401nEA00<006</Hb`01?oo00<00?ooool0Dooo0010ool00`00ooooo`0:
ool01000ooooof</8_oo00<00:fl[K`04:fl0`003:fl00=S;:fl[K`00Zfl00<00:fk[K/03jfk00<0
0:fk[K/01Zfk0`001jfK00<00:fK[I/01JfK0f</2JfK00<00:fK[I/03jfK0`000ZeK00=S;:eK[E/0
3:eK00<00:eJ[EX03jeJ00<00:eJ[EX03ZeJ0`004JCi00<00:CiY?T03jCi00<00:ci[?T01Jci00<0
0:ci[?T02:ci00<00:ci[?T00jci0P002jbh00AS;;CG]=MS;0jde`03003UU>FD00KUU004003ULV</
Hb`9iG800`00iE7UD@08iE401000Hb`006</0ooo00<00?ooool0Dooo0010ool00`00ooooo`09ool0
0`00ooooo`02ool00f</ooooo`0Pool00`00[Kb]_00=[K`20002[K`00`00[Kb]_00:[K`00f</[Kb]
_003[K`00`00[K^]^`0?[K/00`00[K^]^`03[K/2000;[I/01000[I^]VjfK0f</3:fK00<00:fK[I/0
3JfK0P000Zek00@00:eK[E]S;0j]F`03002]FZeJ00n]FP03002]FZeJ00b]FP800003YAX00:Ci016T
n@03002TnJCi00nTn@03002/nJci00F/n@03002/nJci00R/n@04002/nJci[?T2000=[;P01000Hbbd
ef</3kCG00<00>FDiI@01>FD00H00>EbiG9S;>EbHb`8iG800`00iE7UD@08iE401@00Hbc]D000Hb`0
0ooo00<00?ooool0D_oo0010ool00`00ooooo`09ool00`00ooooo`03ool00f</ooooo`0Pool00`00
[Kb]_00:[K`20005[K`00`00[Kb]_009[K`00f</[Kb]_004[K`00`00[K^]^`0?[K/00`00[K^]^`02
000>[I/00`00HbaS;00@[I/00`00[I^]V`09[I/30004[G/01000[E^]Ff</3jeK00<00:eJ[EX03jeJ
00<00:eJ[EX02:eJ0`001:DJ00<00:CiY?T03jCi00<00:CiY?T04:Ci00<00:ci[?T01:ci00<00:ci
[?T02Jci0`003jbh00@00;CGHbaS;0nde`03003UU>FD00CUU004003UL^EbiG82Hb`8iG800`00iE7U
D@08iE401@00Hbc]D000Hb`00_oo00<00?oo0000Dooo0010ool00`00ooooo`09ool00`00ooooo`04
ool00f</ooooo`0Pool00`00[Kb]_006[K`30008[K`00`00[Kb]_008[K`00f</[Kb]_005[K`00`00
[K^]^`0>[K/3000>[I/3Hb`C[I/00`00[I^]V`06[I/20008[G/00`00[E]S;00?[E/00`00[EZ]FP0?
[EX00`00[EZ]FP06[EX20007YAX00`00Y?VTn@0@Y?T00`00Y?VTn@0?Y?T00`00[?V/n@05[?T00`00
[?V/n@07[?T2000B[;P00`00HbaS;00@]=L00`00iICUU003iI@01`00iG;UL^EbHbcULV</00OULP03
003UDNEA00SUD@06001S;>e@kE0006</0ooo00<00?ooool0DOoo0010ool00`00ooooo`09ool00`00
ooooo`05ool2Hb`Qool00`00[Kb]_003[K`2000;[K`00`00[Kb]_008[K`00f</[Kb]_006[K`00`00
[K^]^`0:[K/30002[I/00`00[I^]V`09[I/3Hb`3[I/00`00[I^]V`0A[I/00`00[I^]V`02[I/3000:
[G/00`00[E]S;00@[E/00`00[EZ]FP0?[EX00`00[EZ]FP02[EX3000:YAX00`00Y?VTn@0?Y?T00`00
Y?VTn@0@Y?T00`00[?V/n@04[?T00`00[?V/n@04[?T300000jBH002/^00A[;P00`00]=MS;00@]=L0
0`00iICUU002iI@00`00iG;ULP02iG8016</iG;ULV</1^Eb00<00>EAiE401nEA00<00>e@Hb`00ne@
00=S;?ooool00_oo00<00?ooool0D?oo0010ool00`00ooooo`09ool00`00ooooo`07ool00f</oooo
o`0Oool00`00[Kb]_002000>[K`00`00[Kb]_007[K`00f</[Kb]_006[K`00`00[K^]^`07[K/30006
[I/00`00[I^]V`05[I/3Hb`7[I/00`00[I^]V`0@[I/00`00[I^]V`02000>[G/00`00Hbb]F`0?[E/0
0`00[EZ]FP0?[EX00`00[EZ]FP02000=YAX00`00Y?VTn@0@Y?T00`00Y?VTn@0?Y?T00`00[?V/n@04
[?T00`00[?V/n@02[?T20005Y9P00`00[;R/^00?[;P01000]=Ndef</3kCG00<00>FDiI@00^FD00<0
0>EbiG800^Eb00AS;>EbiG9S;0KULP03003UDNEA00OUD@03003]D6</00?]D003001S;?oo00?oo`03
003ooooo04ooo`00@?oo00<00?ooool02Ooo0P002Ooo00=S;?ooool07ooo0P004Jfl00<00:fl[K`0
1Zfl00=S;:fl[K`01jfl00<00:fk[K/01:fk0P002JfK00<00:fK[I/00jfK0V</2jfK00<00:fK[I/0
3jfK0`004Jek00=S;:eK[E/03jeK00<00:eJ[EX03ZeJ0`004:DJ00<00:CiY?T03jCi00<00:CiY?T0
4:Ci00<00:ci[?T00jci00<00:ci00000P001jBH00<00:bh[;P04:bh00<00;CGHb`04;CG00D00>FD
iICUU00000CULP05HbcUL^EbiG9S;004iG800`00iE7UD@08iE401000kE3]D6</0ne@00<006</ool0
0_oo00<00?ooool0Cooo0010ool00`00ooooo`08ool00`00ooooo`0:ool00f</ooooo`0Oool00`00
[Kb]_00@[K`00`00[Kb]_005[K`00f</[Kb]_008[K`00`00[K^]^`03000<[I/00`00[I]S;002Hb`=
[I/00`00[I^]V`0=[I/20002[G/00`00[G^]N`0?[G/00f</[E^]F`0@[E/00`00[EZ]FP0:[EX30002
YAX00`00YAZU6P0>YAX00`00Y?VTn@0@Y?T00`00Y?VTn@0?Y?T00`00[?V/n@02[?T3000;Y9P00`00
[;R/^00?[;P01000]=MS;6</3kCG00@00>FDiI@000GULP03HbcUL^Eb00;ULP05HbcUL^EbiG80000:
iE401000kE3]D6</0ne@00<006</ool00ooo00<00?ooool0C_oo000nool00`00ool0000:ool00`00
ooooo`0;ool00f</ooooo`0Oool00`00[Kb]_00@[K`00`00[Kb]_004[K`00f</[Kb]_008[K`3000>
[I/3Hb`A[I/00`00[I^]V`09[I/30004[G/00`00[G^]N`0?[G/00f</002]F`0@[E/00`00[EZ]FP07
[EX30005YAX00`00YAZU6P0?YAX00`00Y?VTn@0?Y?T00`00Y?VTn@0@Y?T01P00[?T00000Y9P000bT
V003002/^:bh00n/^005002def</]=MS;00>]=L01000iICUU0001NEb00=S;>EbiG800^Eb00ES;>Eb
iG;ULP0000[UD@04003]D>e@Hb`4kE000`00Hbcoo`02ool00`00ooooo`1>ool003ooo`8000[oo`03
003ooooo00coo`03Hbcooooo01ooo`03002]_:fl012]_003002]_:fl00:]_003Hbb]_:fl00N]_080
00:]V`03002]VjfK00V]V`=S;0>]V`03002]VjfK00n]V`03002]VjfK00J]V`8000R]N`03002]Njek
00j]N`03Hbb]N`00016]F`03002]FZeJ00>]FP<000VU6P03002U6ZDJ00jU6P03002TnJCi00nTn@03
002TnJCi00nTn@<000>TV003002TV:BH00ZTV003002/^:bh012/^004001S;;CGHb`?]=L00`00iI@0
0005iG800f</iG;ULP03iG8016</iG;ULP002^EA00@00>e@kE1S;0C]D003001S;?oo00;oo`8004oo
o`00Booo00<00?ooool03Ooo00=S;?ooool07ooo00<00:fl[K`04:fl00D00:fl[Kb]_6</00N]_080
00F]V`03002]VjfK00J]V`9S;0J]V`03002]VjfK012]V`03002]VjfK00:]V`<000^]N`03002]Njek
00f]N`03Hbb]N`00016]F`03002]FZeJ00<000bU6P03002U6ZDJ00nU6P03002TnJCi00nTn@03002T
nJCi00^Tn@<00003Y9P00:BH00>TV003002TV:BH00^TV003002/^:bh00n/^005001S;;CG]=MS;00>
]=L00`00iI@00005iG800f</iG;ULP04iG800f</iG80000:iE401000kE3]D6</1>e@0P0000ES;000
003oo`0004ooo`00Booo00<00?ooool03_oo00=S;?ooool07_oo00=S;000[K`04:fl00D00:fl[Kb]
_6</00B]_0<000N]V`03002]VjfK00>]V`=S;0V]V`03002]VjfK00n]V`03002]VjfK008000j]N`03
002]Njek00f]N`04Hbb]Njek000@[E/3000@YAX00`00YAZU6P0>YAX00`00Y?VTn@0?Y?T00`00Y?VT
n@09Y?T20005Y9P01@00Y9RTV:BH00003JBH00<00:bh[;P03jbh00<006</]=L00[CG00=S;;CG]=L0
3;CG00<00>EbiG801NEb00=S;>EbiG800nEb00=S;>Eb00002^EA00@00>e@kE1S;0C]D004003eS`00
Hb`2ool00`00ooooo`1=ool004_oo`03003ooooo00ooo`03Hbcooooo01goo`03Hbcoo`00016]_004
002]_:flHb`2[K`2000;[I/00`00[I]S;002Hb`=[I/00`00[I^]V`0>[I/3000A[G/00`00[G^]N`0<
[G/016</[G^]N`003JeK0`000ZDj00<00:DJYAX03ZDJ00<00:DJYAX03jDJ00<00:CiY?T03jCi00<0
0:CiY?T01JCi0`001jBH00D00:BHY9QS;00000fTV003002/^:bh012/^003HbbdekCG00:de`03Hbbd
ekCG00^de`03003UL^Eb00GULP03HbcUL^Eb00CULP03HbcUDNEA00WUD@04003]D>e@Hb`4kE001000
mHoeSf</0_oo00<00?ooool0COoo001:ool00`00ooooo`0Aool00f</ooooo`0Kool016</ooooo`00
4Zfl00<00:flHb`00P003JfK0V</4JfK00<00:fK[I/02ZfK0`000Zek00<00:ek[G/04:ek00<00:ek
[G/02Zek00=S;:ek[G/00Zek00<00:eK[E/01jeK0`001JDj00<00:DJYAX03jDJ00<00:DJYAX03ZDJ
00<00:CiY?T03jCi00<00:CiY?T00ZCi0`002ZBH0f</00>TV000Hb`03JBH00<00:bh[;P03jbh00=S
;;CG]=L00[CG00=S;;CG]=L02KCG0`001nEb00=S;>EbiG801>Eb00=S;>EAiE402NEA00@00>e@kE1S
;0C]D005003eSoF?001S;002ool00`00ooooo`1<ool004[oo`03003ooooo01;oo`03Hbcooooo01[o
o`05Hbcoooooool0000A[K`200000f</[I^]V`0:[I/3Hb`2[I/00`00[I^]V`0?[I/00`00[I^]V`06
[I/30005[G/00`00[G^]N`0@[G/00`00[G^]N`0:[G/00f</[G^]N`03[G/00`00[E^]F`03[E/30009
YCX00`00YAZU6P0>YAX00`00YAZU6P0?YAX00`00Y?VTn@0?Y?T01000Y?T000002ZBH0f</00JTV000
Y9RTV000Hb`=Y9P00`00[;R/^00?[;P00f</]=Nde`03]=L00f</]=Nde`06]=L20003[7H00`00iG;U
LP04iG800f</iG;ULP04iG800`00HbcUD@09iE401@00kE3]D>e@Hb`00ne@00P00?F?mHoeS`00Hbco
o`00C_oo001:ool00`00ooooo`0Cool00f</ooooo`0Iool00f</ooooo`02ool00`00[Kb]_00;[K`3
0002[I/00f</[I^]V`07[I/3Hb`5[I/00`00[I^]V`0?[I/00`00[I^]V`04[I/20009[G/00`00[G^]
N`0@[G/00`00[G^]N`09[G/00f</[G^]N`03[G/00`00[E^]F`03000<YCX00`00YAZU6P0?YAX00`00
YAZU6P0>YAX00`00Y?VTn@0>Y?T30009Y9P3Hb`4Y9P01P00Y9RTV000Y9QS;0fTV003002/^:bh00j/
^003Hb`00;CG00Bde`03HbbdekCG00Bde`03002/MZaf00>/MP03003UL^Eb00CULP03HbcUL^Eb00CU
LP03003UDF</00WUD@05003]D>e@kE1S;003kE002000mHoeSoF?001S;?oo001>ool004[oo`<001Co
o`03Hbcooooo01Soo`03Hbcooooo00?oo`03002]_:fl00N]_0<000F]V`03Hb`00:fK00B]V`=S;0V]
V`03002]VjfK00n]V`03002]VjfK00<000b]N`03002]Njek012]N`03002]Njek00R]N`03Hbb]Njek
00>]N`<0012U>P03002U6ZDJ00jU6P03002U6ZDJ00nU6P03002TnJCi00ZTn@<00003Y;P00:BH00JT
V0=S;0RTV003002TV00000:TV003HbbTV:BH00ZTV003002/^:bh00j/^003Hb`00;CG00Bde`03Hbbd
ekCG00:de`8000J/MP03003UL^Eb00CULP03HbcUL^Eb00CULP03003UDF</00WUD@05003]D>e@kE1S
;003kE000`00mHoeS`02mHl00`00Hb`0001>ool004[oo`03003ooooo01Goo`03Hbcooooo01Ooo`03
Hbcooooo00Coo`03002]_:fl00B]_08000R]V`03Hbb]V`0000:]V`9S;0f]V`03002]VjfK00j]V`<0
012]N`03002]Njek012]N`03002]Njek00N]N`04Hbb]Njek[G/20002YCX00`00YCZU>P0>YCX00`00
YAZU6P0?YAX00`00YAZU6P0>YAX00`00Y?VTn@07Y?T30005Y;P01000Y9RTV:BH0f</2jBH00<00:BH
00000jBH00=S;:BHY9P02JBH00<00:bh[;P03jbh00=S;;CG]=L01;CG00AS;;CG]=L000V/MP03003U
L^Eb00?ULP03HbcUL^Eb00CULP04003UDNEAHb`8iE401@00kE3]D>e@Hb`00ne@00<00?F?mHl00oF?
00=S;000ool0COoo001:ool00`00ooooo`0Fool00f</ooooo`0Fool00f</ooooo`05ool00`00[Kb]
_003000:[I/00f</[I]S;002Hb`@[I/00`00[I^]V`0:[I/30002[G/00`00[G^]N`0>[G/00`00[G^]
N`0@[G/00`00[G^]N`07[G/016</[G/000001:Dj00<00:DjYCX03jDj00<00:DJYAX03ZDJ00<00:DJ
YAX03jDJ00<00:CiY?T00jCi0`002:Bh00@006</HbaS;0jTV003002TV00000>TV003HbbTV:BH00ZT
V003002/^:bh00j/^003Hb`00;CG00Fde`03Hb`00:af00V/MP03003UL^Eb00?ULP03HbcUL^Eb00?U
LP03003UDNEA00;UD@03HbcUDNEA00GUD@05003]D>e@kE1S;003kE000`00mHoeS`03mHl2001>ool0
04[oo`03003ooooo01Ooo`9S;1Koo`03Hbcooooo00Koo`8000b]V`=S;003[I/00:fK016]V`03002]
VjfK00J]V`<000F]N`03002]Njek00n]N`03002]Njek012]N`03002]Njek00F]N`03001S;00000NU
>P03002U>ZDj00jU>P03002U6ZDJ00nU6P03002U6ZDJ00jU6P03002TnJCi00<000VT^0=S;003002T
V:BH00nTV08000BTV003HbbTV:BH00VTV003002/^:bh00j/^003Hb`00;CG00>de`800003Hbb/MZaf
00V/MP03003UL^Eb00?ULP03HbcUL^Eb00?ULP03003UDNEA00;UD@03HbcUDNEA00CUD@03003]D>e@
00;]D005Hbc]D>e@kE000006mHl00`00ooooo`1<ool004Woo`03003ooooo01[oo`03Hbcooooo01?o
o`03Hbcooooo00Ooo`03002]VjfK00N]V`AS;0B]V`03002]VjfK00n]V`03002]VjfK00B]V`8000V]
N`03002]Njek00n]N`03002]Njek012]N`03002]Njek00:]N`800003YCYS;:Dj00NU>P03002U>ZDj
00nU>P03002U6ZDJ00jU6P03002U6ZDJ00jU6P<000VT^0=S;0>T^003002TV:BH00nTV003002TV000
00BTV003HbbTV:BH00VTV003002/^:bh00f/^003Hb`00;CG00:de`05002/MZaf[7IS;00:[7H00`00
iG;ULP04iG800f</iG;ULP02iG800`00iE7UD@03iE400f</iE7UD@03iE400`00kE3]D002kE001F</
kE3]D>e@00001_F?00<00?ooool0C?oo0019ool00`00ooooo`0Kool00f</ooooo`0Aool00f</oooo
o`09ool00`00[I^]V`04[I/2Hb`3[I/00f</[I^]V`03[I/00`00[I^]V`0?[I/00`00[I^]V`03000<
[G/00`00[G^]N`0?[G/00`00[G^]N`0?[G/00`00[G^]N`020003YCX00f</YCZU>P07YCX00`00YCZU
>P0?YCX00`00YAZU6P0>YAX00`00YAZU6P0:YAX300000ibi002T^007Y;P2Hb`6Y;P00`00Y9RTV00@
Y9P20005Y9P00f</Y9RTV008Y9P00`00[;R/^00=[;P00f</[;P000020005[7H00f</[7J/MP08[7H0
0`00iG;ULP03iG800f</iG;ULP02iG800`00iE7UD@04iE400f</iE7UD@02iE400`00kE3]D002kE00
1F</kE3]D>e@00001_F?0P00COoo0019ool00`00ooooo`0Lool00f</ooooo`0@ool00f</ooooo`0:
ool00`00[I^]V`03Hb`5[I/00f</[I^]V`04[I/00`00[I^]V`0>[I/3000@[G/00`00[G^]N`0>[G/0
0`00[G^]N`0?[G/30005YCX00f</YCZU>P07YCX00`00YCZU>P0?YCX00`00YAZU6P0>YAX00`00YAZU
6P07YAX30005W;T00`00Y;RT^002Y;P3Hb`9Y;P00`00Y9RTV00?Y9P20005Y9P00f</Y9RTV008Y9P0
0`00[;R/^00=[;P00f</00000007[7H00f</[7J/MP08[7H00`00iG;ULP03iG800f</iG;ULP02iG80
0`00iE7UD@04iE400f</iE7UD@02iE400`00kE3]D003kE0016</kE3]D0001_F?0P00COoo0019ool0
0`00ooooo`0Mool00f</ooooo`0?ool00f</ooooo`0:ool3Hb`8[I/00f</[I^]V`05[I/00`00[I^]
V`0:[I/30002[I/00`00[G^]N`0>[G/00`00[G^]N`0?[G/00`00[G^]N`0;[G/30002YE/00`00YCZU
>P03YCX00f</YCZU>P08YCX00`00YCZU>P0?YCX00`00YAZU6P0>YAX00`00YAZU6P03YAX30008W;T0
0`00Y;QS;002Hb`<Y;P00`00Y9RTV00?Y9P20006Y9P00f</Y9RTV008Y9P00`00[;R/^00;[;P01000
HbbTEP002:af00=S;:af[7H01jaf00<00>EbiG800nEb00=S;>EbiG800^Eb00<00>EAiE401NEA00ES
;>EAiE7UD@0000G]D004Hbc]D>e@0006mHl2001=ool004Woo`03003ooooo01koo`03Hbcooooo00ko
o`03Hbcooooo00Soo`9S;003ool00:fK00R]V`03Hbb]VjfK00F]V`03002]VjfK00N]V`<000F]V`03
002]Njek00n]N`03002]Njek00n]N`03002]Njek00N]N`<000FUF`03002U>ZDj00>U>P03HbbU>ZDj
00RU>P03002U>ZDj00nU>P03002U6ZDJ00jU6P03002U6ZDJ00<000ZL^@=S;12T^003002TV:BH00nT
V003002TV:BH00FTV003HbbTV:BH00NTV003002/^:bh00V/^0800005Y5IS;:AFY5H00008[7H00f</
[7J/MP07[7H00`00iG;ULP02iG801F</iG;UL^Eb00002NEA00AS;>EAiE4000G]D004Hbc]D>e@0006
mHl2001=ool004Woo`03003ooooo01ooo`03Hbcooooo00goo`03Hbcooooo00Goo`=S;0Coo`03002]
VjfK00J]V`03Hbb]VjfK00J]V`03002]VjfK00B]V`8000V]V`03002]Njek00n]N`03002]Njek00n]
N`03002]Njek00>]N`<000VUF`03002U>ZDj00:U>P03HbbU>ZDj00VU>P03002U>ZDj00nU>P03002U
6ZDJ00fU6P<000ZL^@=S;0:L^@03002T^:Bh00jT^003002TV:BH00nTV003002TV:BH00JTV003HbbT
V:BH00JTV003002/^:bh00N/^08000>TEP04HbbTEZAF0008[7H00f</[7J/MP07[7H00`00iG;ULP02
iG801F</iG;UL^Eb00002NEA00AS;>EAiE4000G]D004Hbc]D>e@0006mHl2001=ool004Soo`<0027o
o`03Hbcooooo00coo`03Hbcooooo00;oo`=S;0Soo`03002]VjfK00F]V`03Hbb]VjfK00N]V`03002]
VjfK00<000b]V`03002]Njek00n]N`03002]Njek00j]N`03002]Njek00<000bUF`03002U>ZDj00:U
>P03HbbU>ZDj00ZU>P03002U>ZDj00jU>P03002U6ZDJ00VU6P@00003W=T009bi00NL^@=S;0JL^@03
002T^:Bh00jT^003002TV:BH00jTV08000NTV003HbbTV:BH00NTV003002/^:bh00B/^08000FTEP04
HbbTEZAF0009[7H00f</[7J/MP06[7H00`00iG;ULP02iG801F</iG;UL^Eb00002^EA00=S;>EA0000
1Ne@00AS;>e@kE0000KeS`8004goo`00B?oo00<00?ooool08_oo00=S;?ooool02ooo00=S;?ooHb`0
0V</3?oo00<00:fK[I/01:fK00=S;:fK[I/01jfK0`004:fK00<00:ek[G/03Zek00<00:ek[G/03Zek
0`004:EK00D00:DjYCZU>V</00bU>P03002U>ZDj00nU>P03002U6ZDJ00BU6P@000JLf@03002L^Ibi
00:L^@=S;0VL^@03002T^:Bh00jT^003002TV:BH00jTV08000RTV003HbbTV:BH00JTV003002/^:bh
00:/^08000NTEP05HbbTEZAFY5H00009[7H00f</[7J/MP06[7H01@00iG;UL^EbHb`00nEb00<00>EA
iE402NEA00=S;000kE001>e@00AS;>e@kE0000KeS`8004goo`00B?oo00<00?ooool08ooo00=S;?oo
ool02_oo0V</4?oo00<00:fK[I/00jfK00=S;:fK[I/01:fK0`000ZfK00<00:fK[I/03ZfK00<00:ek
[G/03jek00<00:ek[G/02Zek0`000ZEK00<00:EKYE/03jEK00@00:DjYCYS;0fU>P03002U>ZDj00jU
>P03002U6ZDJ00@000ZLf@03002L^F</009S;0bL^@03002T^:Bh00jT^003002TV:BH00jTV08000VT
V003HbbTV:BH00JTV004002/^000000:Y5H016</Y5JTEP002Zaf00=S;:af[7H01Jaf00D00>EbiG;U
LV</00?ULP03003UDNEA00WUD@03Hb`00>e@00C]D004Hbc]D>e@0006mHl00`00ool0001<ool004So
o`03003ooooo02Coo`03Hbcooooo00Koo`AS;17oo`03002]VjfK00>]V`04Hbb]VjfK[I/30006[I/0
0`00[I^]V`0>[I/00`00[G^]N`0?[G/00`00[G^]N`06[G/30005YE/00`00YE^UF`0?YE/01000YCZU
>V</3JDj00<00:DjYCX03ZDj0`003IcI0f</49bi00<00:BhY;P03ZBh00<00:BHY9P03JBH00<00:BH
00002JBH00=S;:BHY9P01:BH0`003:AF00AS;:AFY5H000Z/MP03Hbb/MZaf00F/MP03003UL^Eb00;U
LP04HbcUL^Eb000<iE400f</kE3]D003kE0016</kE3]D0001_F?00<00?oo0000C?oo0018ool00`00
ooooo`0Uool00f</ooooo`02ool3Hb`2ool00f</ooooo`0Aool01@00[I^]VjfKHb`00ZfK0P002JfK
00<00:fK[I/03jfK00<00:ek[G/03jek00<00:ek[G/00jek0P002JEK00<00:EKYE/03jEK00<00:Dj
Hb`03ZDj00<00:DjYCX02ZDj0`0000>LfP00W=T02YcI0f</0YcI00<009biW;T03Ybi00<00:BhY;P0
3ZBh00<00:BHY9P03JBH00<00:BH00002JBH00AS;:BHY9RTV0<00003W3L00:AF00bTEP05HbbTEZAF
Y5H0000:[7H00f</[7J/MP05[7H01`00iG;UL^EbHbcULP0000gUD@03001S;>e@00C]D003Hbc]D000
00KeS`03003oo`0004coo`00B?oo00<00?ooool09_oo16</1Ooo00=S;?ooool04_oo00@00:fK[I]S
;08000b]V`03002]VjfK00n]V`03002]Njek00j]N`03002]Njek00<000bUF`03002UFjEK00nUF`03
001S;:Dj00fU>P03002U>ZDj00NU>P<000FLfP03002LfIcI00FLf@=S;0JLf@03002L^Ibi00jL^@03
002T^:Bh00jT^003002TV:BH00bTV003002TV00000ZTV003Hb`0000000FL=`03002TEZAF00ZTEP05
HbbTEZAFY5H0000;[7H00f</[7J/MP04[7H01`00iG;UL^EbHbcULP0000gUD@03001S;>e@00C]D003
Hbc]D00000KeS`03003oo`0004coo`00Aooo00<00?ooool09_oo0f</1ooo00=S;?ooool04ooo0P00
00=S;:fK[I/03JfK00<00:fK[I/03ZfK00<00:ek[G/03Zek0`004:EK00<00:EKYE/03ZEK00<006</
YCX03ZDj00<00:DjYCX00jDj0`0029cJ00<009cIW=T00YcI0f</2IcI00<009biW;T03Ybi00<00:Bh
Y;P03ZBh00<00:BHY9P03:BH00@00:BHY9P000NTV0<00003HbbL=i`g00BL=`03002TEZAF00ZTEP05
HbbTEZAFY5H0000;[7H00f</[7J/MP04[7H01`00iG;UL^EbHbcULP0000gUD@03003]D6</00C]D003
Hbc]D00000KeS`03003oo`0004coo`00Aooo00<00?ooool08ooo0f</0ooo0V</1Ooo00=S;?ooool0
5?oo00<006</[I/03jfK00<00:fK[I/03ZfK00<00:ek[G/02Zek0`000ZEK00<00:EKYE/03ZEK00<0
0:EKYE/03jEK00=S;:DjYCX03JDj00<00:DjYCX00`0039cJ00@006</HbaS;0fLf@03002L^Ibi00jL
^@03002T^:Bh00fT^003002TV:BH00bTV004002TV:BH0004Y9P30004W3L00f</W3NL=`03W3L00`00
Y5JTEP0:Y5H00f</Y5JTEP02Y5H00`00[7J/MP09[7H00f</[7J/MP04[7H01P00iG;ULV</iG8000gU
D@03003]D6</00C]D003Hbc]D00000KeS`03003oo`0004coo`00Aooo00<00?ooool08?oo0f</2?oo
00=S;?ooool00_oo00=S;?ooool05Ooo00=S;:fK[I/03ZfK00<00:fK[I/03jfK00<00:ek[G/01Zek
0`001JEK00<00:EKYE/03jEK00<00:EKYE/03ZEK00=S;:DjYCX03JDj0`003IcJ0f</49cI00<009bi
W;T03Ybi00<00:BhY;P03ZBh00<00:BHY9P02jBH00H00:BHY9RTV000Y9P20007W3L00f</W3NL=`04
W3L00`00Y5JTEP09Y5H00f</Y5JTEP02Y5H00`00[7J/MP0:[7H00f</[7J/MP03[7H01P00iG;ULV</
iG8000cUD@05003]D>e@kE1S;003kE000f</kE000006mHl00`00ool0001<ool004Ooo`03003ooooo
01koo`9S;0coo`05HbcooooooomS;00Gool00f</002]V`0?[I/00`00[I^]V`0?[I/00`00[G^]N`03
[G/20009YE/00`00YE^UF`0?YE/00`00YE^UF`0=YE/00f</002U>P09YCX40002W?X00`00W=ZLfP08
W=X3Hb`3W=X00`00W=VLf@0>W=T00`00W;VL^@0>W;T00`00Y;RT^00=Y;P00`00Y9RTV00;Y9P00`00
Y9P00003000:W3L00f</W3NL=`03W3L00`00Y5JTEP09Y5H00f</Y5JTEP02Y5H00`00[7J/MP0:[7H0
0f</[7J/MP03[7H01P00iG;ULV</iG8000cUD@03003]D>e@00;]D006Hbc]D>e@Hbc]D0001_F?00@0
0?ooool004_oo`00Aooo00<00?ooool06ooo0f</3ooo00AS;?oooomS;1Ooo`03Hb`00:fK012]V`03
002]VjfK00j]V`03002]Njek00<000bUF`03002UFjEK00nUF`03002UFjEK00bUF`03HbbUF`0000JU
>P<000JLnP03002LfYcJ00FLfP=S;0JLfP03002LfIcI00jLf@03002L^Ibi00jL^@03002T^:Bh00jT
^003002TV:BH00RTV0@000:L=`03002L=i`g00ZL=`03HbbL=i`g00:L=`03002TEZAF00VTEP03HbbT
EZAF00>TEP03002/MZaf00Z/MP03Hbb/MZaf00:/MP05003UL^EbHb`0000=iE400`00kE3]D002kE00
1V</kE3]D6</kE0000KeS`04003ooooo001;ool004Ooo`8001Woo`=S;1?oo`03Hbcoof</01Ooo`03
Hbcoo`00016]V`03002]VjfK00f]V`<0012UF`03002UFjEK00jUF`03002UFjEK00bUF`03HbbUF`00
00:U>P@000ZLnP04002LfYcJW=X3Hb`:W=X00`00W=VLf@0=W=T00`00W;VL^@0>W;T00`00Y;RT^00>
Y;P00`00Y9RTV004Y9P40003W3L01@00W3NL=i`g000039`g00ES;9`gW3NL=`0000^TEP03HbbTEZAF
00>TEP03002/MZaf00^/MP03Hbb/MZaf00:/MP04003ULV</000=iE400`00kE3]D003kE001F</kE1S
;>e@00001_F?00@00?ooool004_oo`00A_oo0P005ooo0f</5ooo00=S;?ooool05_oo00AS;?ooool0
016]V`03002]VjfK00R]V`@000:]N`03002UFjEK00jUF`03002UFjEK00nUF`03002UFjEK00^UF`03
HbbUF`00008000jLnP04001S;6</Hb`=W=X00`00W=VLf@0>W=T00`00W;VL^@0>W;T00`00Y;RT^00>
Y;P00`00Y9P000030007W3L01@00W3NL=i`g000039`g00=S;9`gW3L00Y`g00<00:AFY5H02:AF00=S
;:AFY5H00jAF00<00:af[7H02jaf00=S;:af[7H00Zaf00@00>EbiG9S;0gUD@03003]D>e@00C]D004
Hbc]D6</0006mHl01000ooooo`00Booo0016ool00`00ooooo`0Dool2Hb`Jool2Hb`Gool01F</oooo
oooo00004:fK00<00:fK[I/01JfK0`001jek00<00:EKYE/03ZEK00<00:EKYE/03jEK00<00:EKYE/0
2:EK0P0000ES;000YAZU6P0000bLnP=S;003002LfYcJ00jLfP03002LfIcI00fLf@03002L^Ibi00jL
^@03002T^:Bh00fT^0<000^L=`05002L=i`gW3L0000=W3L01F</W3NL=i`g00002ZAF00=S;:AFY5H0
1:AF00<00:af[7H02jaf00ES;:af[7J/MP0000;ULP03HbcUDNEA00_UD@03003]D>e@00C]D004Hbc]
D6</0006mHl01000ooooo`00Booo0016ool00`00ooooo`0Aool3Hb`Lool00f</oomS;00Fool00f</
ooooo`02ool00`00[I^]V`0>[I/00`00[I^]V`04000:[G/00`00YE^UF`0?YE/00`00YE^UF`0?YE/0
0`00YE^UF`04YE/30002YAX01F</YAZU6ZDJ00002Ycj0V</0icj00<009cJW=X03YcJ00<009cIW=T0
3YcI00<009biW;T03Ybi00<00:BhY;P02:Bh100000>L=`00W3L02i`g00<009`gW3L00Y`g00<009`g
W3L02i`g00AS;9`gW3L000^TEP03HbbTEZAF00>TEP03002/MZaf00b/MP07Hbb/MZaf[7H00>EbHb`0
3NEA00<00>e@kE001Ne@0V</00<00?F?mHl01?F?00@00?ooool004_oo`00A_oo00<00?ooool03_oo
0f</7ooo00AS;?oooomS;1Coo`03Hbcooooo00Coo`03002]VjfK00f]V`<000n]N`03002UFjEK00nU
F`03002UFjEK00jUF`03002UFjEK00@000FU6P03HbbU6ZDJ00:U6P03002LnYcj00BLnP=S;0JLnP03
002LfYcJ00jLfP03002LfIcI00fLf@03002L^Ibi00jL^@03002T^:Bh00BT^0@000JL=`03002L=i`g
00VL=`03002L=i`g00:L=`03002L=i`g00^L=`05HbbL=i`gW3L0000:Y5H00f</Y5JTEP03Y5H00`00
[7J/MP0<[7H01f</[7J/MZaf003ULV</00gUD@03003]D>e@00K]D003Hb`00?F?00GeS`04003ooooo
001;ool004Koo`03003ooooo00_oo`=S;2;oo`05HbcooooooomS;00Cool00f</ooooo`05ool00`00
[I^]V`0:[I/20002[G/00`00[G^]N`0>[G/00`00YE^UF`0>YE/00`00YE^UF`0>YE/30009YAX00f</
YAZU6P02YAX01000W?ZLnYcj0f</2Icj00<009cJW=X03YcJ00<009cIW=T03YcI00<009biW;T03Ybi
00<00:Bh00000`002Y`g00<009`gW3L02I`g00<009`gW3L00Y`g00<009`gW3L039`g00AS;9`gW3L0
00ZTEP03HbbTEZAF00BTEP03002/MZaf00b/MP03Hbb/MZaf00800003HbcUDNEA00_UD@03003]D>e@
00K]D003Hb`00?F?00GeS`04003ooooo001;ool004Koo`03003ooooo00Woo`9S;2Goo`03Hbcooooo
00;oo`03Hbcooooo013oo`03Hbcooooo00Goo`03002]VjfK00N]V`<000B]N`03002]Njek00n]N`03
002UFjEK00jUF`03002UFjEK00ZUF`<000:U>P03002U6ZDJ00NU6P03HbbU6ZDJ00>U6P=S;0fLnP03
002LfYcJ00jLfP03002LfIcI00fLf@03002L^Ibi00fL^@<000jL=`03002L=i`g00VL=`03002L=i`g
00:L=`03002L=i`g00fL=`03HbbL=`0000ZTEP03HbbTEZAF00BTEP03002/MZaf00f/MP05Hbb/MZaf
001S;00=iE400`00kE3]D006kE02Hb`6mHl01@00oooooooo0000B_oo0016ool00`00ooooo`06ool3
Hb`Wool00f</ooooo`03ool00f</ooooo`0?ool00f</ooooo`06ool00`00[I^]V`03[I/30008[G/0
0`00[G^]N`0>[G/00`00YE^UF`0?YE/00`00YE^UF`06YE/30005YCX00`00YAZU6P07YAX00f</YAZU
6P03Hb`00`00W?ZLnP0=W?X00`00W=ZLfP0>W=X00`00W=VLf@0>W=T00`00W;VL^@08W;T400000iAH
002L=`0?W3L00`00W3NL=`08W3L00`00W3NL=`03W3L00`00W3NL=`0=W3L00f</W3L00009Y5H00f</
Y5JTEP04Y5H00`00[7J/MP0>[7H016</[7H006</3NEA00<00>e@kE001^e@00=S;000Hb`01_F?00@0
0?ooool004[oo`00AOoo00<00?ooool01?oo0f</:_oo00=S;?ooool01?oo00=S;?ooool03_oo00=S
;?ooool01ooo00<00:fK[I/00P003:ek00<00:ek[G/03Zek00<00:EKYE/03jEK00<00:EKYE/00jEK
0P002JDj00<00:DJYAX01ZDJ0f</1:DJ00<009cjW?X03Icj00<009cJW=X03YcJ00<009cIW=T03IcI
00<009biW;T019bi10001YAH00<009`gW3L03I`g00<009`gW3L029`g00<009`gW3L00i`g00<009`g
W3L03I`g00=S;9`g00002JAF00=S;:AFY5H01JAF00<00:af[7H03Jaf00AS;000001S;0gUD@03003]
D>e@00K]D003Hb`006</00KeS`04003ooooo001:ool004Goo`03003ooooo00;oo`9S;2goo`03Hbco
oooo00Goo`03Hbcooooo00goo`03Hbcooooo00Soo`8000n]N`03002]Njek00j]N`03002UFjEK00jU
F`03002UFjEK00<000^U>P03002U6ZDJ00>U6PAS;0JU6P03002LnYcj00fLnP03002LfYcJ00jLfP03
002LfIcI00jLf@03002L^@0000<000ZDF003002L=i`g00fL=`03002L=i`g00RL=`03002L=i`g00>L
=`03002L=i`g00jL=`03Hb`00:AF00RTEP03HbbTEZAF00FTEP03002/MZaf00b/MP05002/=F</[3ES
;00=iE400`00kE3]D007kE000f</mHmS;005mHl01000ooooo`00B_oo0015ool00`00oomS;002Hb`^
ool00f</ooooo`07ool00f</ooooo`0<ool00f</ooooo`09ool00`00[G^]N`0=[G/00`00[G^]N`0?
[G/00`00YE^UF`0=YE/3000?YCX00`00YAYS;002Hb`3YAX00f</YAZU6P05YAX00`00W?ZLnP0=W?X0
0`00W=ZLfP0>W=X00`00W=VLf@0;W=T4000>U5P00`00W3NL=`0>W3L00`00W3NL=`07W3L00`00W3NL
=`04W3L00`00W3NL=`0>W3L00f</Y5JTEP07Y5H00f</Y5JTEP05Y5H00`00[7J/MP0;[7H01`00[3F/
=J`eHb`006</00cUD@03003]D>e@00O]D004HbceSoF?Hb`4mHl01000ooooo`00B_oo0015ool00`00
Hbcoo`0`ool00f</ooooo`08ool2Hb`<ool00f</ooooo`0:ool00`00[G^]N`0=[G/00`00[G^]N`0?
[G/00`00YE^UF`08YE/40002YC/00`00YCZU>P0<YCX3Hb`6YAX00f</YAZU6P06YAX00`00W?ZLnP0<
W?X00`00W=ZLfP0>W=X00`00W=VLf@06W=T50002U5P00`00U5RDF00>U5P00`00W3NL=`0=W3L00`00
W3NL=`07W3L00`00W3NL=`04W3L00`00W3NL=`0?W3L00f</Y5JTEP06Y5H00f</Y5JTEP06Y5H00`00
[7J/MP08[7H20004[3D00f</001S;00<iE400`00kE3]D007kE0016</mHoeSf</1?F?00@00?ooool0
04[oo`00AOoo00<006</ool0<?oo00=S;?ooool02_oo00=S;?ooool02Ooo00=S;?ooool02_oo00<0
0:ek[G/03Zek00<00:ek[G/03Zek00<00:EKYE/01JEK0`001ZDk00<00:DjYCX02JDj0f</0ZDj00<0
0:DJYAX01:DJ00=S;:DJYAX01ZDJ00<009cjW?X03Icj00<009cJW=X03YcJ00<009cIW=T01@0029AH
00<009AHU5P03IAH00<009`gW3L03I`g00<009`gW3L01i`g00<009`gW3L019`g00<009`gW3L03i`g
00=S;:AFY5H01ZAF00=S;:AFY5H01ZAF00<00:af[7H01jaf00<00:`e[3D01J`e0V</2nEA00<00>e@
kE002>e@00ES;?F?mHoeSf</00?eS`04003ooooo001:ool001Woo`D000;oo`8000;oo`H000;oo`@0
01Goo`03003oof</033oo`03Hbcooooo00_oo`03Hbcooooo00Soo`03Hbcooooo00_oo`03002]Njek
00j]N`03002]Njek00j]N`03002UFjEK00@000ZU>`03002U>ZDj00FU>P=S;0FU>P03002U6ZDJ00BU
6P03HbbU6ZDJ00NU6P03002LnYcj00bLnP03002LfYcJ00bLfPD000fDF003002DF9AH00fDF003002L
=i`g00jL=`03002L=i`g00JL=`03002L=i`g00FL=`03002L=i`g00jL=`03001S;:AF00JTEP03HbbT
EZAF00JTEP03002/MZaf00J/MP03002/=J`e00N/=@03HbcUDNEA00WUD@03003]D>e@00S]D003Hbce
SoF?00;eS`04HbceSoF?0003ool00`00ooooo`17ool001_oo`03003ooooo00?oo`05003oooooool0
0004ool00`00ooooo`03ool00`00ooooo`0Aool01000oooooooo0V</;_oo00=S;?ooool03?oo00=S
;?ooool01_oo00=S;?ooool03Ooo00<00:ek[G/03Jek00<00:ek[G/03Zek0`003ZDk00<00:DjYCX0
0ZDj0f</2JDj00<00:DJYAX00jDJ00=S;:DJYAX01jDJ00<009cjW?X03Icj00<009cJW=X01YcJ1@00
0YAi00<009AHU5P03IAH00<009AHU5P03YAH00<009`gW3L03I`g00<009`gW3L01Y`g00<009`gW3L0
1I`g00<009`gW3L03i`g00<006</Y5H01ZAF00=S;:AFY5H01ZAF00<00:af[7H00jaf0P002Z`e00=S
;>EAiE402NEA00<00>e@kE002>e@00=S;?F?mHl00_F?00AS;?F?mHl000?oo`03003ooooo04Ooo`00
6ooo0P002Ooo00<00?ooool02?oo00<00?ooool04?oo00<00?ooool00ooo00=S;?ooool0:ooo00=S
;?ooool03Ooo00=S;?ooool01Ooo00=S;?ooool03_oo00<00:ek[G/03Jek00<00:ek[G/02Zek0`00
0ZEK00<00:DkYC/03JDk00@006</HbaS;0fU>P05002U6ZDJYAYS;00;YAX00`00W?ZLnP0<W?X01000
W=ZLfYcJ1@0029Ai00<009AHU5P03IAH00<009AHU5P03IAH00<009`gW3L03I`g00<009`gW3L01Y`g
00<009`gW3L01I`g00<009`gW3L03i`g00<00:AFHb`01ZAF00=S;:AFY5H01ZAF00<00:af[7H00Zaf
00<00:`e[3D02Z`e0V</2^EA00<00>e@kE002>e@00=S;?F?mHl00oF?00=S;?F?00000ooo00<00?oo
ool0Aooo000Kool20009ool00`00ooooo`08ool00`00ooooo`0@ool00`00ooooo`04ool00f</oooo
o`0Zool00f</ooooo`0>ool00f</ooooo`04ool00f</ooooo`0>ool00`00[G^]N`0>[G/00`00[G^]
N`06[G/30005YE/00`00YC^U>`0;YC/3Hb`00`00YCZU>P0=YCX01@00YAZU6ZDJHb`02jDJ00<009cj
W?X02icj1@003IAi00<009AHU5P03IAH00<009AHU5P03IAH00<009`gW3L03Y`g00<009`gW3L01Y`g
00<009`gW3L01I`g00<009`gW3L03Y`g00<00:AFHb`01ZAF00=S;:AFY5H01ZAF00D00:af[7J/MP00
00f/=@03Hb`006</00WUD@03003]D>e@00S]D003HbceSoF?00?eS`03HbceS`0000?oo`03003ooooo
04Ooo`006ooo10002?oo00<00?ooool01ooo00<00?ooool04?oo00<00?ooool01Ooo00=S;?ooool0
:Ooo00=S;?ooool03ooo00=S;?ooool00ooo00=S;?ooool03ooo00<00:ek[G/03Zek00<00:ek[G/0
0jek0P002JEK00<00:DkYC/01jDk0f</0jDk00<00:DjYCX03ZDj00@00:DJYAYS;0bU6P03002LnYcj
00FLnPD000:DV@03002DNIAi00jDN@03002DF9AH00fDF003002DF9AH00fDF003002L=i`g00fL=`03
002L=i`g00JL=`03002L=i`g00FL=`03002L=i`g00nL=`03002TEV</00FTEP03HbbTEZAF00NTEP<0
00j/=@03Hb`006</00WUD@03003]D>e@00S]D003001S;?F?00CeS`03Hb`00?oo00;oo`03003ooooo
04Ooo`006ooo0P000_oo00<00?ooool01_oo00<00?ooool01Ooo0P004_oo00<00?ooool01_oo00=S
;?ooool09ooo00=S;?ooool04Ooo00=S;?ooool00_oo00=S;?ooool04?oo00<00:ek[G/03Jek00<0
0:ek[G/00`002jEK00<00:DkYC/01:Dk0f</1jDk00<00:DjYCX03JDj00@00:DJYAYS;0bU6P04002L
nYcjW?X40008U9T00`00U7VDN@0=U7T00`00U5RDF00=U5P00`00U5RDF00=U5P00`00W3NL=`0=W3L0
0`00W3NL=`06W3L00`00W3NL=`05W3L00`00W3NL=`0?W3L01000Y5JTEV</1:AF00=S;:AFY5H01ZAF
0P004:`e00AS;000iE5S;0SUD@03003]D>e@00S]D003001S;?F?00GeS`05Hbcoooooool00019ool0
00;oo`D000Coo`8000Coo`D000Goo`8000;oo`03003ooooo00Ooo`03003ooooo00;oo`<001?oo`03
003ooooo00Ooo`03Hbcooooo02Koo`03Hbcooooo01;oo`05HbcooooooomS;00Bool00f</002]N`0=
[G/3000?YE/00`00YC^U>`03Hb`;YC/00`00YCZU>P0=YCX00`00YAYS;00<YAX4000<U9T00`00U7VD
N@0=U7T00`00U5RDF00=U5P00`00U5RDF00=U5P00`00W3NL=`0=W3L00`00W3NL=`06W3L00`00W3NL
=`06W3L00`00W3NL=`0>W3L01@00Y5JTEZAFHb`00jAF00=S;:AFY5H01:AF0P0000>Km@00[3D04:`e
00AS;>EAiE5S;0OUD@03003]D>e@00S]D003001S;?F?00GeS`05Hbcoooooool00019ool000Coo`03
003ooooo00?oo`03003ooooo00;oo`8000Goo`03003ooooo00;oo`8000;oo`03003ooooo00Ooo`03
003ooooo00@001?oo`03003ooooo00Woo`03Hbcooooo02Goo`03Hbcooooo01?oo`04HbcoooooHb`B
ool00f</ool0000:[G/30002YE/00`00YE^UF`0=YE/3Hb`>YC/00`00YCZU>P0=YCX00`00YAYS;008
YAX400000ibj002DV@0>U9T00`00U7VDN@0=U7T00`00U5RDF00=U5P00`00U5RDF00=U5P00`00W3NL
=`0=W3L00`00W3NL=`05W3L00`00W3NL=`06W3L00`00W3NL=`0>W3L01@00Y5JTEZAFHb`00jAF00=S
;:AFY5H00ZAF0P0019_e00<00:`e[3D03Z`e00AS;000iE5S;0OUD@03003]D>e@00S]D003001S;?F?
00GeS`05001S;?ooool00019ool000Coo`03003ooooo00;oo`03003ooooo00?oo`03003ooooo00Wo
o`D000Coo`03003ooooo00;oo`05003oooooool0000Eool3000:ool2Hb`Uool00f</ooooo`0Dool0
0f</oomS;00Bool00f</ool00006[G/40005YE/00`00YE^UF`0;YE/2Hb`00jEK002U>`0?YC/00`00
YCZU>P0=YCX00`00HbbU6P04YAX30006W;X00`00U9VDV@0<U9T00`00U7VDN@0=U7T00`00U5RDF00=
U5P00`00U5RDF00=U5P00`00W3NL=`0=W3L00`00W3NL=`05W3L00`00W3NL=`06W3L00`00W3NL=`0?
W3L01@00Y5JTEZAFHb`00ZAF00=S;:AF00000P001Y_e00<00:`e[3D03Z`e00ES;000iE7UDF</00KU
D@03003]D>e@00S]D003001S;?F?00GeS`03003oof</00;oo`03003ooooo04Koo`001?oo00@00?oo
ooooo`8000Coo`03003ooooo00Soo`03003oo`0000Soo`04003oooooool20003ool00`00ooooo`0C
ool00`00ooooo`0<ool00f</ooooo`0Rool00f</ooooo`0Eool2Hb`Bool016</ooooo`000Zek0`00
2ZEK00<00:EKYE/01jEK0f</1:EK00<00:DkYC/03JDk00<00:DjYCX03JDj00<006</YAX010002Ibj
00<009BIU9T03IBI00<009AiU7T03IAi00<009AHU5P039AH00<009AHU5P03YAH00<009`gW3L039`g
00<009`gW3L01I`g00<009`gW3L01Y`g00<009`gW3L03i`g00<00:AFY5H00ZAF00AS;:AFHb`000VK
m@03002/=J`e00j/=@03Hb`00>EA00;UD@03HbcUDNEA00?UD@03003]D>e@00S]D003001S;?F?00Ge
S`03003oof</00;oo`03003ooooo04Koo`001?oo00<00?ooool00P001Ooo00<00?ooool02Ooo0P00
2Ooo10001Ooo10004Ooo00<00?ooool03Ooo00=S;?ooool08Ooo00=S;?ooool05_oo00=S;?ooool0
4?oo00AS;?ooooooo`8000jUF`03002UFjEK00>UF`=S;0RUF`03002U>jDk00fU>`03002U>ZDj00^U
>P800003Hb`009bj00fL^P03002DVIBI00bDV@03002DNIAi00fDN@03002DF9AH00fDF003002DF9AH
00fDF003002L=i`g00fL=`03002L=i`g00BL=`03002L=i`g00NL=`03002L=i`g00jL=`03002TEZAF
00:TEP03001S;6</00ZKm@03002/=J`e00j/=@03Hbb/=@0000?UD@03HbcUDNEA00;UD@03003]D>e@
00S]D003001S;?F?00GeS`06003oooooHbcoo`00B?oo0004ool50005ool00`00ooooo`0bool00`00
ooooo`0>ool00f</ooooo`0Pool00f</ooooo`0Eool00f</oomS;00Aool00f</ooooo`02ool00`00
YE^UF`0<YE/00`00YE^UF`03Hb`;YE/00`00YC^U>`0>YC/00`00YCZU>P05YCX50002W=X00f</002L
^P0=W;X00`00U9VDV@0<U9T00`00U7VDN@0=U7T00`00U5RDF00=U5P00`00U5RDF00=U5P00`00W3NL
=`0=W3L00`00W3NL=`04W3L00`00W3NL=`07W3L00`00W3NL=`0?W3L01@00Y5H00000VoD00V</2i_e
00<00:`e[3D03J`e00=S;:`e00000nEA00=S;>EAiE400^EA00<00>e@kE002>e@00<006</mHl01OF?
00H00?ooooooof</0018ool000Coo`8000?oo`03003ooooo00;oo`03003ooooo03;oo`03003ooooo
00ooo`03Hbcooooo01ooo`03Hbcooooo01Goo`04HbcoooooHb`@ool00f</ooooo`02ool00`00YE^U
F`0<YE/3Hb`?YE/00`00YC^U>`0=YC/01000YCZU>ZDj10001icJ00=S;000W;X03Ybj00<009BIU9T0
39BI00<009AiU7T03IAi00<009AHU5P03IAH00<009AHU5P03IAH00<009`gW3L039`g00<009`gW3L0
19`g00<009`gW3L01i`g00<009`gW3L03Y`g0`0019_e00=S;9_eVoD02I_e00<00:`e[3D03J`e00=S
;:`e00001>EA00ES;>EAiE7UD@0000[]D003001S;?F?00GeS`06003ooooooomS;000B?oo0004ool0
0`00ooooo`03ool01@00oooooooo00001ooo00<00?ooool0:Ooo00<00?ooool04Ooo00=S;?ooool0
7_oo00=S;?ooool05Ooo00AS;?ooooooo`9S;0koo`03Hbcooooo00?oo`03002UFjEK00RUF`=S;0:U
F`03002UFjEK00fUF`03002U>jDk00fU>`@000^LfP03HbbLfP0000jL^P03002DVIBI00bDV@03002D
NIAi00fDN@03002DF9AH00fDF003002DF9AH00fDF003002L=i`g00fL=`03002L=i`g00>L=`03002L
=i`g00RL=`03002L=i`g00ZL=`<00003VmH009_e00FKm@03HbbKmI_e00RKm@03002/=J`e00f/=@03
Hbb/=@0000GUD@04HbcUDNEA000:kE000`00mHmS;005mHl01P00ooooooooHb`004Soo`001?oo00<0
0?ooool00ooo00<00?ooool00_oo00<00?ooool01?oo00<00?ooool0:Ooo00<00?ooool04_oo00=S
;?ooool07?oo00=S;?ooool05_oo00=S;?ooool00ooo00=S;?ooool02_oo00=S;?ooool01Ooo00<0
0:EKYE/01:EK0f</1JEK00<00:EKYE/03ZEK00<00:DkYC/02:Dk10000Ycj00<009cJW=X02YcJ00=S
;9cJ00003ibj00<009BIU9T039BI00<009AiU7T03IAi00<009AHU5P039AH00<009AHU5P03IAH00<0
09`gW3L03I`g00<009`gW3L00i`g00<009`gW3L029`g00<009`gW3L01Y`g100019_F00<009_eVoD0
19_e00=S;9_eVoD029_e00<00:`e[3D03J`e00AS;:`e[3D000CUD@03HbcUD@0000_]D003003eSf</
00GeS`06001S;6</Hbcoo`00B?oo0004ool00`00ooooo`02ool20005ool00`00ooooo`03ool00`00
ooooo`0Yool00`00ooooo`0Cool2Hb`Lool00f</ooooo`0Fool00f</ooooo`04ool00f</ooooo`09
ool00f</ooooo`05ool01000YE^UFjEK0f</2JEK00<00:EKYE/03ZEK00<00:DkYC/01:Dk0`001Ycj
00<009cJW=X02YcJ00AS;9cJW=X000jL^P03002DVIBI00bDV@03002DNIAi00fDN@03002DF9AH00fD
F003002DF9AH00fDF003002L=i`g00bL=`03002L=i`g00>L=`03002L=i`g00RL=`03002L=i`g00>L
=`<000VKeP03002KmI_e00>Km@9S;0ZKm@03002/=J`e00f/=@03Hbb/=@0000GUD@03Hb`00>e@00[]
D003003eSf</00GeS`03003ooooo00?oo`03003ooooo04Goo`000_oo1`002?oo1P00:ooo00<00?oo
ool05Ooo00=S;?ooool06Ooo00=S;?ooool05_oo00=S;?ooool01Ooo00=S;?ooool02?oo00=S;?oo
ool01_oo0f</3JEK00<00:EKYE/03JEK00<00:DkYC/010002Ycj00<009cJW=X02IcJ00AS;9cJW=X0
00nL^P03002DVIBI00bDV@03002DNIAi00bDN@03002DF9AH00fDF003002DF9AH00fDF003002L=i`g
00fL=`03002L=i`g00:L=`03002L=i`g00VL=`03002L=`00008000bKeP03002KmI_e00>Km@03HbbK
mF</00VKm@03002/=J`e00f/=@03Hbb/=@0000KUD@03Hbc]D>e@00W]D003003eSf</00GeS`03003o
oooo00?oo`03003ooooo04Goo`00@_oo00<00?ooool05_oo00=S;?ooool06?oo00=S;?ooool05_oo
00=S;?ooool01_oo00=S;?ooool01ooo00=S;?ooool00ooo0f</00?oo`00YE/03JEK00<00:EKYE/0
3JEK0`003Ycj00<009cJW=X02IcJ00ES;9cJW=ZLfP0000jL^P03002DVIBI00bDV@03002DNIAi00fD
N@03002DF9AH00bDF003002DF9AH00fDF003002L=i`g00fL=`03002L=i`g00:L=`03002L=i`g00NL
=`@000nKeP03002KmI_e00>Km@04HbbKmI_eHb`8VoD00`00[3F/=@0=[3D016</[3F/=@001NEA00=S
;>e@kE002Ne@00<00?F?Hb`01OF?00<00?ooool00ooo00<00?ooool0AOoo0012ool00`00ooooo`0G
ool00f</ooooo`0Gool00f</ooooo`0Fool00f</ooooo`07ool00f</ooooo`06ool00f</ooooo`03
Hb`4ool00`00YE^UF`0=YE/00`00YE^UF`08YE/40002YAX00`00W?ZLnP0=W?X00`00W=ZLfP08W=X0
1F</W=ZLfYcJ00003ibj00<009BIU9T039BI00<009AiU7T039Ai00<009AHU5P03IAH00<009AHU5P0
3IAH00<009`gW3L039`g00<009`gW3L00Y`g00@009`gW3NL=`H000:CeP03002KeY_F00jKeP03002K
mI_e00>Km@04HbbKmI_eHb`8VoD00`00[3F/=@0=[3D016</[3F/=@001NEA00<006</kE002Ne@00<0
0?F?Hb`01OF?00<00?ooool00ooo00<00?ooool0AOoo0012ool2000Iool00f</ooooo`0Fool00f</
ooooo`0Fool00f</ooooo`08ool00f</ooooo`05ool3Hb`8ool00`00YE^UF`0=YE/00`00YE^UF`04
YE/30006YAX00`00W?ZLnP0=W?X00`00W=ZLfP08W=X00f</W=ZLfP02W=X00`00W;ZL^P0<W;X00`00
U9VDV@0<U9T00`00U7VDN@0=U7T00`00U5RDF00<U5P00`00U5RDF00=U5P00`00W3NL=`0=W3L00`00
W3NL=`050009TmH00`00VmJKeP0>VmH00`00VoFKm@02VoD01F</VoFKmI_eHb`029_e00<00:`e[3D0
3:`e00AS;:`e[3D000GUD@03003]D6</00W]D003003eSf</00GeS`03003ooooo00?oo`03003ooooo
04Goo`00@Ooo00<00?ooool06_oo00=S;?ooool05Ooo00=S;?ooool05_oo00=S;?ooool02Ooo00AS
;?ooooooo`AS;0_oo`03002UFjEK00bUF`03002UFjEK00@000ZU6P03002LnYcj00fLnP03002LfYcJ
00NLfP03HbbLfYcJ00:LfP03002L^Ybj00fL^P03002DVIBI00^DV@03002DNIAi00fDN@03002DF9AH
00fDF003002DF9AH00bDF003002L=i`g00ZL=`H000:CeP03002CeY?F00VCeP03002KeY_F00jKeP03
002KmI_e00:Km@03HbbKmI_e00:Km@03HbbKmI_e00FKm@03002/=J`e00b/=@04Hbb/=J`e0005iE40
0`00kE1S;009kE000`00mHmS;005mHl00`00ooooo`03ool00`00ooooo`15ool0047oo`03003ooooo
01_oo`9S;1Goo`03Hbcooooo01Koo`03Hbcooooo00[oo`=S;0?oo`03Hbcooooo00Woo`03002UFjEK
00bUF`<000jU6P03002LnYcj00fLnP03002LfYcJ00NLfP03HbbLfYcJ00>LfP03002L^Ybj00bL^P03
002DVIBI00bDV@03002DNIAi00bDN@03002DF9AH00fDF003002DF9AH00fDF004002L=i`gW3L80003
TmH00`00TmJCeP02TmH00`00TmJCeP09TmH00`00VmJKeP0>VmH00`00VoFKm@02VoD00f</VoFKm@03
VoD00f</VoFKm@04VoD00`00[3F/=@0<[3D01F</[3F/=J`e00001>EA00@00>e@kE1S;0S]D004003e
SoF?Hb`4mHl00`00ooooo`03ool00`00ooooo`15ool0047oo`03003ooooo01goo`03Hbcooooo017o
o`03Hbcooooo01Koo`03Hbcooooo00Soo`=S;003oomS;?oo00?oo`03Hbcooooo00[oo`03002UFjEK
00RUF`<000:U>P03002U6ZDJ00fU6P03002LnYcj00fLnP03002LfYcJ00JLfP03HbbLfYcJ00>LfP03
002L^Ybj00fL^P03002DVIBI00^DV@03002DNIAi00fDN@03002DF9AH00bDF003002DF9AH00ZDF0L0
00bCeP05002CeY?FTmH0000;TmH00`00VmJKeP0>VmH00`00VoFKm@02VoD00f</VoFKm@03VoD00f</
VoFKm@05VoD00`00[3F/=@0;[3D01F</[3F/=J`e00001>EA00D00>e@kE3]D6</00K]D005003eSoF?
mHmS;004mHl00`00ooooo`03ool00`00ooooo`15ool0047oo`03003ooooo01koo`03Hbcooooo013o
o`03Hbcooooo01Koo`03Hbcooooo00Goo`=S;0Goo`05HbcooooooomS;00=ool00`00YE^UF`03YE/4
0005YCX00`00YAZU6P0>YAX00`00W?ZLnP0=W?X00`00W=ZLfP05W=X00f</W=ZLfP04W=X00`00W;ZL
^P0<W;X00`00U9VDV@0<U9T00`00U7VDN@0<U7T00`00U5RDF00=U5P01000U5RDF9AH20000h_G00<0
09?FTmH03I?F00D009?FTmJCeP0000bCeP03002KeY_F00jKeP05002KmI_eVoES;006VoD00f</VoFK
m@04VoD00`00[3F/=@0;[3D01F</[3F/=J`e00001>EA00D00>e@kE3]D6</00K]D005003eSoF?mHmS
;004mHl00`00ooooo`04ool00`00ooooo`14ool0047oo`03003ooooo01ooo`03Hbcooooo00ooo`03
Hbcooooo01Koo`03Hbcooooo00;oo`=S;0Woo`04HbcoooooHb`=ool00`00YE^UF`03000:YCX00`00
YAZU6P0=YAX00`00W?ZLnP0=W?X00`00W=ZLfP05W=X00f</W=ZLfP04W=X00`00W;ZL^P0=W;X00`00
U9VDV@0;U9T00`00U7VDN@0=U7T00`00U5RDF005U5P;000<RmL00`00TmJCeP0<TmH01@00TmJCeY?F
000039?F00<009_FVmH03Y_F00D009_eVoFKmF</00NKm@03HbbKmI_e00>Km@03002/=J`e00b/=@05
Hbb/=J`e[3D00003iE400`00kE3]D002kE000f</kE3]D003kE001@00mHoeSoF?Hb`01?F?00<00?oo
ool01?oo00<00?ooool0A?oo0011ool00`00ooooo`0Pool00f</ooooo`0>ool00f</ooooo`0Fool0
0f</oomS;002Hb`=ool00f</oomS;00>ool2000>YCX00`00YAZU6P0=YAX00`00W?ZLnP0=W?X00`00
W=ZLfP04W=X00f</W=ZLfP05W=X00`00W;ZL^P0<W;X00`00U9VDV@0<U9T00`00U7VDN@05U7T?0007
RoL00`00RmN;e`0=RmL00`00TmJCeP0<TmH01@00TmJCeY?F000039?F00<009_FVmH03Y_F00<009_e
VoD00Y_e00=S;9_eVoD01I_e00=S;9_eVoD00Y_e00<00:`e[3D03:`e00ES;:`e[3F/=@0000?UD@03
003]D>e@00?]D003Hbc]D>e@00;]D005003eSoF?mHmS;004mHl00`00ooooo`04ool00`00ooooo`14
ool0043oo`03003ooooo02;oo`03Hbcooooo00goo`03Hbcooooo01Koo`9S;17oo`03Hbcooooo00ko
o`03002U>ZDj00bU>P03002U6ZDJ00fU6P03002LnYcj00fLnP03002LfYcJ00BLfP03HbbLfYcJ00FL
fP03002L^Ybj00fL^P03002DVIBI00BDV@l000N<6003002;mh_g00f;m`03002;eh_G00b;e`03002C
eY?F00fCeP04002CeY?F000<TmH00`00VmJKeP0>VmH00`00VoFKm@02VoD00f</VoFKm@05VoD00f</
VoFKm@03VoD00`00[3F/=@0;[3D01F</[3F/=J`e00000nEA00<00>e@kE001>e@00ES;>e@kE3]D000
00?eS`03HbceSoF?00;eS`03003ooooo00Coo`03003ooooo04Coo`00@?oo00<00?ooool08ooo00=S
;?ooool03?oo00=S;?ooool04ooo16</4_oo0f</3_oo00<00:DjYCX03JDj00<00:DJYAX03JDJ00<0
09cjW?X03Icj00<009cJW=X00icJ00=S;9cJW=X01YcJ00<009bjW;X02Ibj2P001h`h00<008`HS1P0
3H`H00<008_gRoL038_g00<008_GRmL038_G00<009?FTmH03I?F00@009?FTmH000fCeP03002KeY_F
00jKeP05002KmI_eVoES;008VoD00f</VoFKm@02VoD00`00[3F/=@0;[3D01F</[3F/=J`e00000nEA
00<00>e@kE001>e@00ES;>e@kE3]D00000?eS`03HbceSoF?00;eS`03003ooooo00Coo`03003ooooo
04Coo`00@?oo00<00?ooool09?oo0V</3?oo00=S;?ooool04?oo0f</0ooo00=S;?ooool04?oo00AS
;?oooomS;0koo`03002U>ZDj00fU>P03002U6ZDJ00bU6P03002LnYcj00fLnP03002LfYcJ00>LfP03
HbbLfYcJ00JLfP03002L^Ybj00:L^PL000><F@03002<>8`h00b<>003002<68`H00b<6003002;mh_g
00b;m`03002;eh_G00f;e`03002CeY?F00bCeP04002CeP00000=TmH00`00VmJKeP0>VmH01@00VoFK
mI_eHb`02I_e00ES;9_eVoFKm@0000f/=@03Hbb/=J`e00:/=@04003UDNEA0007kE0016</kE3]D000
0oF?00=S;?F?mHl00_F?00<00?ooool01?oo00<00?ooool0A?oo0010ool00`00ooooo`0Vool00f</
ooooo`09ool00f</ooooo`0=ool3Hb`6ool00f</ooooo`0@ool01F</ooooooooHb`03_oo00<00:Dj
YCX03:Dj00<00:DJYAX03JDJ00<009cjW?X03Icj00<009cJW=X00YcJ00=S;9cJW=X019cJ1`002haI
00<008`hS3P02h`h00<008`HS1P038`H00<008_gRoL03H_g00<008_GRmL038_G00<009?FTmH02i?F
0`000Y?F00<009?FTmH02Y?F00<009_FVmH03Y_F00D009_eVoFKmF</00ZKm@04HbbKmI_e000=[3D0
0f</[3F/=@02[3D00`00iE400009kE000f</kE000004mHl01F</mHoeSoF?00001_oo00<00?ooool0
A?oo0010ool00`00ooooo`0Wool00f</ooooo`07ool00f</ooooo`0;ool3Hb`9ool00f</ooooo`0@
ool00f</ooooo`02ool00f</ooooo`0;ool00`00YCZU>P0=YCX00`00YAZU6P0=YAX00`00W?ZLnP0<
W?X00`00W=ZLfP02W=X00f</000000040003U9T00`00S5V<F@0<S5T00`00S3R<>00<S3P00`00S1R<
600<S1P00`00RoN;m`0<RoL00`00RmN;e`0<RmL00`00TmJCeP07TmH40002TmH01@00TmJCeY?F0000
2i?F00<009_FVmH03Y_F00D009_eVoFKmF</00ZKm@05HbbKmI_eVoD0000<[3D00f</[3F/=@02[3D0
0`00iE400009kE000f</kE000004mHl01F</mHoeSoF?00001ooo00<00?ooool0@ooo0010ool2000Y
ool00f</ooooo`06ool00f</ooooo`08ool3Hb`<ool00f</ooooo`0@ool00f</ooooo`03ool00f</
ooooo`0;ool00`00YCZU>P0=YCX00`00YAZU6P0<YAX00`00W?ZLnP0;W?X600000f</U9VDV@08U9T0
0`00S5V<F@0;S5T00`00S3R<>00<S3P00`00S1R<600<S1P00`00RoN;m`0<RoL00`00RmN;e`0<RmL0
0`00TmJCeP04TmH30006TmH01@00TmJCeY?F000039?F00<009_FVmH03Y_F00@009_eVoES;0^Km@04
HbbKmI_e000<[3D00f</[3F/=@03[3D2000:kE000f</003eS`03mHl01F</mHoeSoF?00001ooo00<0
0?ooool0@ooo000oool00`00ooooo`0Zool00f</ooooo`05ool00f</ooooo`05ool3Hb`>ool00f</
ooooo`0Aool00f</ooooo`04ool00f</ooooo`0;ool00`00YCZU>P0=YCX00`00YAZU6P0<YAX00`00
W?ZLnP05W?X50002U;X01@00U9VDVIBIHb`02YBI00<008aIS5T038aI00<008`hS3P02h`h00<008`H
S1P038`H00<008_gRoL038_g00<008_GRmL03H_G00<009?FTmH00`002I?F00<009?FTmH00Y?F00<0
09?FTmH02I?F00<009_FVmH03Y_F00@009_eVoES;0bKm@03HbbKm@0000b/=@03Hbb/=J`e00>/=@80
00_]D003HbceSoF?00;eS`05HbceSoF?mHl00007ool00`00ooooo`13ool003ooo`03003ooooo02_o
o`03Hbcooooo00Coo`03Hbcooooo00;oo`=S;17oo`03Hbcooooo017oo`03Hbcooooo00Goo`03Hbco
oooo00[oo`03002U>ZDj00fU>P03002U6ZDJ00bU6P04002LnYcjW?X40008U;X01000U9VDVF</2iBI
00<008aIS5T02haI00<008`hS3P038`h00<008`HS1P038`H00<008_gRoL038_g00<008_GRmL038_G
0`003I?F00<009?FTmH00Y?F00<009?FTmH029?F00<009_FVmH03Y_F00@009_eVoES;0fKm@03Hb`0
0:`e00b/=@03Hbb/=J`e00:/=@8000_]D003HbceSoF?00;eS`05HbceSoF?mHl00007ool00`00oooo
o`13ool003ooo`03003ooooo02coo`03Hbcooooo00?oo`03Hbcoof</009S;1Coo`03Hbcooooo017o
o`03Hbcooooo00Koo`03Hbcooooo00[oo`03002U>ZDj00fU>P03002U6ZDJ00ZU6PD000bD^P04002D
VIBIHb`;U9T00`00S5V<F@0<S5T00`00S3R<>00;S3P00`00S1R<600<S1P00`00RoN;m`0<RoL00`00
RmN;e`08RmL5000?TmH00`00TmJCeP03TmH00`00TmJCeP08TmH00`00VmJKeP0=VmH01000VoFKmF</
3I_e00=S;9_e00003:`e00=S;:`e[3D00Z`e0P002ne@00<006</mHl00_F?00ES;?F?mHoeS`0000Oo
o`03003ooooo04?oo`00?ooo00<00?ooool0;Ooo0V</0_oo0f</5ooo00=S;?ooool04Ooo00=S;?oo
ool01ooo00=S;?ooool02_oo00<00:DjYCX03JDj00<00:DJYAX01JDJ10000icJ00<009BjU;X039Bj
00<009BIHb`039BI00<008aIS5T02haI00<008`hS3P02h`h00<008`HS1P038`H00<008_gRoL038_g
00<008_GRmL01H_G0`001H_G00<009?FTmH039?F00<009?FTmH019?F00<009?FTmH01i?F00<009_F
VmH03Y_F00<009_eHb`03Y_e00=S;000[3D02j`e00=S;:`e[3D00j`e00<00>e@kE002Ne@00<00?F?
Hb`00_F?00ES;?F?mHoeS`0000Ooo`03003ooooo04?oo`00?ooo00<00?ooool0;_oo0f</00?oof</
ool05ooo00=S;?ooool04Ooo00=S;?ooool02?oo00=S;?ooool02Ooo00<00:DjYCX03JDj00@00:DJ
YAZU6P@000NLfP03002D^YBj00bD^P03002DVF</00bDV@03002<FHaI00^<F@03002<>8`h00b<>003
002<68`H00b<6003002;mh_g00b;m`04002;eh_GRmL30008RmL00`00TmJCeP0=TmH00`00TmJCeP03
TmH00`00TmJCeP07TmH00`00VmJKeP0>VmH01000VoFKmF</3Y_e00=S;:`e[3D02Z`e00=S;:`e[3D0
0j`e00<00>e@kE002Ne@00<00?F?Hb`00_F?00ES;?F?mHoeS`0000Ooo`03003ooooo04?oo`00?ooo
00<00?ooool0:ooo0f</0_oo00=S;?ooHb`06?oo00=S;?ooool04Ooo00=S;?ooool02Ooo00=S;?oo
ool02Ooo00<00:DjYCX02jDj1@0039cJ00<009BjU;X039Bj00<006</U9T02iBI00<008aIS5T038aI
00<008`hS3P02h`h00<008`HS1P038`H00<008_gRoL038_g10002h_G00<009?FTmH03I?F00<009?F
TmH019?F00<009?FTmH01Y?F00<009_FVmH03Y_F00@009_eVoES;0nKm@03Hbb/=J`e00V/=@03Hbb/
=J`e00:/=@8000_]D004003eSoF?Hb`2mHl016</mHoeS`002?oo00<00?ooool0@_oo000nool00`00
ooooo`0Yool3Hb`6ool2Hb`Hool00f</ooooo`0@ool00f</ooooo`0;ool00f</ooooo`09ool00`00
YCZU>P05YCX50003W?X00`00W=ZLfP0;W=X00`00U;ZD^P0<U;X00`00HbbDV@0<U9T00`00S5V<F@0;
S5T00`00S3R<>00<S3P00`00S1R<600;S1P00`00RoN;m`09RoL4000>RmL00`00TmJCeP0=TmH00`00
TmJCeP05TmH00`00TmJCeP06TmH00`00VmJKeP0=VmH01000VoFKmF</3i_e00=S;:`e[3D02J`e00MS
;:`e[3F/=@00[3D0000;kE001`00mHoeSoF?HbceSf</00;eS`03003ooooo00Koo`03003ooooo04;o
o`00?_oo00<00?ooool09_oo0f</2Ooo0V</6?oo00=S;?ooool04?oo00=S;?ooool03?oo00=S;?oo
ool02?oo00@00:DjYCZU>P@000RLnP03002LfYcJ00bLfP03002D^YBj00bD^P03HbbDVIBI00^DV@03
002<FHaI00^<F@03002<>8`h00b<>003002<68`H00b<6003002;mh_g00F;m`<000B;m`03002;eh_G
00b;e`03002CeY?F00bCeP03002CeY?F00JCeP03002CeY?F00FCeP03002KeY_F00jKeP03002KmF</
00nKm@03001S;:`e00V/=@04Hbb/=J`e0003[3D00`00kE3]D008kE001`00mHoeSoF?HbceSf</00;e
S`03003ooooo00Koo`03003ooooo04;oo`00?_oo00<00?ooool08ooo0f</3?oo00=S;?ooHb`05ooo
00=S;?ooool04?oo00=S;?ooool03Ooo00=S;?ooool02?oo0`003Icj00<009cJW=X02icJ00<009Bj
U;X039Bj00=S;9BIU9T039BI00<008aIS5T02haI00<008`hS3P02h`h00<008`HS1P038`H00<008_g
RoL00X_g0`001h_g00<008_GRmL038_G00<009?FTmH03I?F00<009?FTmH01Y?F00<009?FTmH019?F
00<009_FVmH03Y_F00<009_eHb`03i_e00<00:`eHb`02J`e00=S;:`e00001:`e00<00>e@kE002>e@
00<00?F?mHl00_F?0V</0_F?00<00?ooool01_oo00<00?ooool0@_oo000nool00`00ooooo`0Pool3
Hb`?ool00f</ooooo`02Hb`Eool00f</ooooo`0@ool00f</ooooo`0>ool00f</ooooo`08ool00`00
W?ZLnP0<W?X00`00W=ZLfP0<W=X00`00U;ZD^P0:U;X00f</002DV@0=U9T00`00S5V<F@0;S5T00`00
S3R<>00<S3P00`00S1R<600<S1P4000:RoL00`00RmN;e`0<RmL00`00TmJCeP0=TmH00`00TmJCeP07
TmH00`00TmJCeP03TmH00`00VmJKeP0>VmH00`00VoES;00@VoD00`00Hbb/=@08[3D00f</002/=@04
[3D00`00kE3]D008kE000`00mHoeS`03mHl016</mHoeS`002?oo00<00?ooool0@_oo000nool3000M
ool3Hb`Bool00f</ooooo`02ool00f</ooooo`0Bool00f</ooooo`0@ool00f</ooooo`0?ool2Hb`8
ool00`00W?ZLnP0=W?X00`00W=ZLfP0;W=X00`00U;ZD^P0:U;X00f</U;X0000>U9T00`00S5V<F@0;
S5T00`00S3R<>00;S3P00`00S1R<600:S1P3000>RoL00`00RmN;e`0<RmL00`00TmJCeP0<TmH00`00
TmJCeP07TmH00`00TmJCeP04TmH00`00VmJKeP0=VmH00`00VoES;00@VoD00`00[3ES;008[3D00`00
Hbb/=@04[3D00`00kE3]D008kE000`00mHoeS`03mHl2Hb`00oF?003oo`07ool00`00ooooo`12ool0
03koo`03003ooooo01[oo`=S;1Goo`03Hbcooooo00?oo`03Hbcooooo013oo`03Hbcooooo017oo`03
Hbcooooo017oo`03Hbcooooo00Koo`03002LnYcj00bLnP03002LfYcJ00bLfP03002D^YBj00VD^P03
HbbD^P0000jDV@03002<FHaI00^<F@03002<>8`h00b<>003002<68`H00F<60@000:<6003002;mh_g
00b;m`03002;eh_G00b;e`03002CeY?F00bCeP03002CeY?F00RCeP03002CeY?F00>CeP03002KeY_F
00jKeP03001S;9_e00nKm@04002/=J`eHb`5[3D200000j`eHbb/=@05[3D00`00kE3]D007kE000`00
mHoeS`03mHl2Hb`2mHl00`00ooooo`05ool00`00ooooo`12ool003koo`03003ooooo01Ooo`=S;1So
o`03Hbcooooo00Coo`03Hbcooooo00ooo`03Hbcooooo017oo`03Hbcooooo01;oo`03Hbcooooo00Go
o`03002LnYcj00fLnP03002LfYcJ00bLfP03002D^YBj00RD^P04HbbD^YBj000=U9T00`00S5V<F@0;
S5T00`00S3R<>00<S3P00`00S1R<6002S1P30007S1P00`00RoN;m`0;RoL00`00RmN;e`0<RmL00`00
TmJCeP0=TmH00`00TmJCeP08TmH00`00TmJCeP02TmH00`00VmJKeP0>VmH00`00HbbKm@0?VoD01@00
[3F/=J`eHb`00j`e00D00:`e[3F/=F</00J/=@03003]D>e@00O]D003003eSoF?00?eS`05HbceSf</
mHl00007ool00`00ooooo`12ool003goo`03003ooooo01Koo`9S;1_oo`03Hbcooooo00Goo`03Hbco
oooo00koo`03Hbcooooo017oo`03Hbcooooo01?oo`03Hbcooooo00Goo`03002LnYcj00bLnP03002L
fYcJ00bLfP03002D^YBj00RD^P04HbbD^YBj000>U9T00`00S5V<F@0;S5T00`00S3R<>00;S3P00`00
S1P00002000:S1P00`00RoN;m`0<RoL00`00RmN;e`0;RmL00`00TmJCeP0=TmH00`00TmJCeP09TmH0
0`00TmJCeP02TmH00`00VmJKeP0=VmH00`00HbbKm@0@VoD01000[3F/=F</0Z`e00<00:`e[3D00Z`e
00=S;:`e[3D01:`e00<00>e@kE001ne@00<00?F?mHl01?F?00AS;?F?Hb`000Soo`03003ooooo047o
o`00?Ooo00<00?ooool04ooo0f</7Ooo00=S;?ooool01_oo00=S;?ooool03Ooo00=S;?ooool04Ooo
00=S;?ooool05?oo00=S;?ooool01Ooo00<009cjW?X039cj00<009cJW=X039cJ00<009BjU;X01iBj
00ES;9BjU;ZD^P0000fDV@03002<FHaI00^<F@03002<>8`h00Z<>0<000f<6003002;mh_g00b;m`03
002;eh_G00b;e`03002CeY?F00bCeP03002CeY?F00ZCeP05002CeY?FTmH0000?VmH00`00HbbKm@0@
VoD01P00[3F/=J`eHb`000F/=@03Hbb/=J`e00B/=@03003]D>e@00O]D003003eSoF?00CeS`04Hbce
Sf</0008ool00`00ooooo`11ool003goo`03003ooooo013oo`=S;1ooo`03Hbcooooo00Soo`03Hbco
oooo00coo`03Hbcooooo017oo`03Hbcooooo01Goo`03Hbcooooo00Coo`03002LnYcj00bLnP03002L
fYcJ00bLfP03002D^YBj00ND^P05HbbD^YBjU;X0000>U9T00`00S5V<F@0:S5T00`00S3R<>006S3P4
0002S3P00`00S1R<600<S1P00`00RoN;m`0;RoL00`00RmN;e`0<RmL00`00TmJCeP0=TmH00`00TmJC
eP09TmH01@00TmJCeY?F000049_F00=S;9_eVoD03i_e00H00:`e[3F/=@00Hb`5[3D00f</[3F/=@05
[3D00`00kE3]D006kE000`00mHoeS`04mHl016</mHoeSf</2?oo00<00?ooool0@Ooo000mool00`00
ooooo`0=ool3Hb`Rool00f</ooooo`09ool2Hb`<ool00f</ooooo`0Aool00f</ooooo`0Fool00f</
ooooo`04ool00`00W?ZLnP0<W?X00`00W=ZLfP0<W=X00`00U;ZD^P06U;X00f</U;ZD^P02U;X00`00
U9VDV@0;U9T00`00S5V<F@0;S5T00`00S3R<>002S3P30006S3P00`00S1R<600<S1P00`00RoN;m`0;
RoL00`00RmN;e`0<RmL00`00TmJCeP0=TmH00`00TmJCeP0:TmH01000TmJCeP0049_F00<006</VoD0
3i_e00@00:`e[3D000:/=@03Hbb/=J`e00:/=@03Hbb/=J`e00F/=@03003]D>e@00K]D003003eSoF?
00CeS`05HbceSoF?001S;007ool00`00ooooo`11ool003goo`03003ooooo00[oo`=S;2Goo`03Hbco
oooo00_oo`03Hbcooooo00Woo`03Hbcooooo013oo`03Hbcooooo01Soo`03Hbcooooo00Coo`03002L
nYcj00^LnP03002LfYcJ00bLfP03002D^YBj00JD^P03HbbD^YBj00:D^P03002DVIBI00bDV@03002<
FHaI00Z<F@03002<>000008000Z<>003002<68`H00^<6003002;mh_g00b;m`03002;eh_G00^;e`03
002CeY?F00fCeP03002CeY?F00^CeP04002CeY?F000?VmH00`00HbbKm@0@VoD20003[3D00f</[3F/
=@02[3D00f</[3F/=@05[3D00`00kE3]D006kE000`00mHoeS`04mHl01F</mHoeS`00Hb`01ooo00<0
0?ooool0@Ooo000mool00`00ooooo`07ool3Hb`Xool00f</ooooo`0<ool00f</ooooo`08ool00f</
ooooo`0@ool00f</ooooo`0Iool00f</ooooo`03ool00`00W?ZLnP0<W?X00`00W=ZLfP0<W=X00`00
U;ZD^P05U;X00f</U;ZD^P03U;X00`00U9VDV@04U9T>0005S5T3000=S3P00`00S1R<600<S1P00`00
RoN;m`0;RoL00`00RmN;e`0<RmL00`00TmJCeP0<TmH00`00TmJCeP0<TmH00`00TmH0000?VmH00`00
HbbKm@0?VoD20005[3D01F</[3F/=J`eHb`01j`e00<00>e@kE001^e@00<00?F?mHl01?F?00IS;?F?
mHl00?ooHb`6ool00`00ooooo`11ool003coo`03003ooooo00Goo`=S;2_oo`03Hbcooooo00goo`03
Hbcooooo00Ooo`03Hbcooooo013oo`03Hbcooooo01[oo`03Hbcooooo00?oo`03002LnYcj00^LnP03
002LfYcJ00bLfP03002D^YBj00FD^P03Hb`0000000X000N<FP0300246H@I00B46@l000N<>003002<
68`H00^<6003002;mh_g00^;m`03002;eh_G00b;e`03002CeY?F00fCeP03002CeY?F00bCeP80012K
eP03HbbKmI_e00^Km@<00003VmD00:`e00F/=@05Hbb/=J`e[3ES;007[3D00`00kE3]D005kE000`00
mHoeS`04mHl016</mHoeS`000_oo00=S;?ooool00ooo00<00?ooool0@Ooo000lool00`00ooooo`02
ool3Hb`^ool00f</ooooo`0>ool00f</ooooo`06ool00f</ooooo`0@ool00f</ooooo`0Kool00f</
ooooo`02ool00`00W?ZLnP0<W?X00`00W=ZLfP09W=X:00000f</S9Z<VP04S9X00`00S5Z<FP0;S5X0
1000Q1V46H@I0`001X@I00<007_hNoP01G_h2`002X`H00<008_gRoL038_g00<008_GRmL02h_G00<0
09?FTmH03I?F00<009?FTmH039?F0P0049_F00=S;9_eVoD029_e0`0019_E00<00:`e[3D01J`e00AS
;:`e[3ES;0N/=@03003]D>e@00G]D003003eSoF?00CeS`04HbceSoF?0002ool00f</ooooo`04ool0
0`00ooooo`10ool003coo`03003oof</009S;37oo`03Hbcooooo00ooo`03Hbcooooo00Coo`03Hbco
oooo017oo`03Hbcooooo01coo`03Hbcooooo00;oo`03002LnYcj00^LnP03002LfYcJ00:LfPL000>D
fP03002<VXbJ00B<VP03Hbb<VXbJ00B<VP03002<FXaJ00^<FP@000Z46@03001kn7_h00]kn004001k
^7^hNkP80003S1P00`00RoN;m`0;RoL00`00RmN;e`0<RmL00`00TmJCeP0<TmH00`00TmJCeP0=TmH2
000?VmH00f</VoFKm@06VoD20008VmD00`00[3F/=@04[3D016</[3F/=F</1j`e00<00>e@kE001Ne@
00<00?F?mHl01OF?00=S;?F?00000ooo00=S;?ooool00ooo00<00?ooool0@?oo000lool00`00Hbco
o`0cool00f</ooooo`0@ool00f</ooooo`03ool00f</ooooo`0Aool00f</ooooo`0Mool00f</oooo
o`02ool00`00W?ZLnP08W?X7000;U=X00`00S9Z<VP03S9X00f</S9Z<VP05S9X00`00S5Z<FP08S5X4
000<Q1T00`00NoQkn00<NoP00`00NkQk^008NkP7000:RoL00`00RmN;e`0<RmL00`00TmJCeP0=TmH0
0`00TmJCeP0=TmH00`00VmJKeP0=VmH00f</VoFKm@03VoD3000:VmD00`00[3F/=@05[3D00f</[3ES
;007[3D00`00kE3]D005kE000`00mHoeS`05mHl00f</mHl00004ool00f</ooooo`02ool00`00oooo
o`10ool003coo`03001S;?oo03?oo`03Hbcooooo017oo`9S;0?oo`03Hbcooooo017oo`03Hbcooooo
01koo`05Hbcoooooool00004W?X60004W?X00`00U=ZDfP0;U=X00`00S9Z<VP03S9X00f</S9Z<VP05
S9X00`00S5Z<FP04S5X40003S5X00`00Q1V46@0;Q1T00`00NoQkn00;NoP00`00NkQk^00;NkP01000
NiMkUg^G20000h_g00<008_GRmL02h_G00<009?FTmH03I?F00<009?FTmH03I?F0P003Y_F00=S;000
VoD00`003I_E00<00:`e[3D01Z`e0V</2:`e00<00>e@kE001>e@00<00?F?mHl01OF?00=S;?F?0000
1?oo00=S;?ooool00_oo00<00?ooool0@?oo000Jool50002ool20002ool4000Cool01000oomS;6</
<Ooo00=S;?ooool05?oo00AS;?oooomS;1?oo`03Hbcooooo01ooo`9S;0;oo`@000ZLnP03002DfYCJ
00bDfP03002<VXbJ00:<VP03Hbb<VXbJ00J<VP03002<FXaJ00<000R<FP0300246H@I00Z46@03001k
n7_h00]kn003001k^7^h00ak^003001kUg^G00QkU`L000Z;e`03002CeY?F00fCeP03002CeY?F00fC
eP03002KeP0000ZKeP<00003Hb`00000012Ke@03002/=J`e00N/=@03Hbb/=J`e00J/=@03003]D>e@
00C]D003003eSoF?00GeS`03HbceS`0000Goo`05Hbcoooooool00012ool001coo`03003ooooo00?o
o`03003ooooo00Coo`03003ooooo00ooo`03003ooooo00;oo`03Hbcooooo02koo`03Hbcooooo01Go
o`03Hbcoof</01?oo`03Hbcooooo027oo`04Hbcoof</000=W?X00`00U=ZDfP0;U=X00`00S9Z<VP02
S9X00f</S9Z<VP06S9X3000;S5X00`00Q1V46@0:Q1T00`00NoQkn00<NoP00`00NkQk^00;NkP00`00
NiMkU`0;NiL01000PiN3Uh>G20000h_G00<009?FTmH039?F00<009?FTmH03Y?F0P000Y_F20000i>e
00=S;000VmD04I_E00<00:`e[3D01Z`e00=S;:`e[3D01Z`e00<00>e@kE001>e@00<00?F?mHl01OF?
00=S;?F?00001_oo00AS;?ooool004;oo`007?oo00<00?ooool02ooo00<00?ooool03_oo00<00?oo
ool00ooo00=S;?ooool0;Ooo00=S;?ooool05_oo0V</4ooo00=S;?ooool08_oo0V</00<009cjW?X0
2icj00<009CJU=X039CJ00D008bJS9Z<VV</00F<VP<00003S9X008aJ00b<FP0300246H@I00Z46@03
001kn7_h00]kn003001k^7^h00]k^003001kUg^G00akU`030023Uh>G00R3U`/000RCeP03002CeY?F
00FCeP`000^C]@03Hb`009_E016Ke@03002/=J`e00J/=@9S;0N/=@03003]D>e@00C]D003003eSoF?
00GeS`03HbceS`0000Ooo`03Hbcoo`0004;oo`007?oo00<00?ooool02ooo00<00?ooool03_oo00<0
0?ooool01?oo00=S;?ooool0;?oo00=S;?ooool05ooo00=S;?ooool04Ooo00=S;?ooool08ooo00=S
;?oo00003Icj00<009CJU=X02iCJ00H008bJS9Z<VV</S9X40004S9X00`00S5Z<FP0;S5X00`00Q1V4
6@0:Q1T00`00NoQkn00;NoP00`00NkQk^00;NkP00`00NiMkU`0<NiL00`00PiN3U`0;PiL00`00PiJ3
UP05PiH@0008RiH01000TkFC]@0039>e00=S;000VmD049_E00<00:`e[3D01Z`e00=S;:`eHb`01j`e
00<00>e@kE000ne@00<00?F?mHl01OF?00=S;?F?00001ooo00=S;?oo0000@_oo000Lool00`00oooo
o`0;ool00`00ooooo`0>ool00`00ooooo`05ool2Hb`/ool00f</ooooo`0Gool2Hb`Bool00f</oooo
o`0Sool2Hb`00`00W?ZLnP0<W?X00`00U=ZDfP0;U=X00`00S9X000020009S9X00`00S5Z<FP0;S5X0
0`00Q1V46@0:Q1T00`00NoQkn00;NoP00`00NkQk^00;NkP00`00NiMkU`0;NiL00`00PiN3U`0<PiL0
0`00PiJ3UP0<PiH00`00RiJ;UP0=RiH01@00TkFC]I>e00002i>e00=S;000VmD049_E00<00:`e[3D0
1j`e00=S;:`eHb`01Z`e00<00>e@kE000ne@00<00?F?mHl01OF?00=S;?F?00002?oo00=S;?oo0000
@Ooo000Lool00`00ooooo`0:ool2000@ool00`00ooooo`07ool00f</ooooo`0Yool00f</ooooo`0G
ool00f</oomS;00@ool00f</ooooo`0Tool016</oomS;0003Icj00<009CJU=X02YCJ0`0038bJ00<0
08aJS5X02haJ00<008@IQ1T02X@I00<007_hNoP02g_h00<007^hNkP02g^h00<007^GNiL02g^G00<0
08>GPiL038>G00<008>FPiH03H>F00<008^FRiH03H^F00D009>eTkFC]@0000ZC]@03Hb`009_E016K
e@03002/=J`e00J/=@03Hbb/=F</00J/=@03003]D>e@00?]D003003eSoF?00KeS`03Hb`00?oo00So
o`03Hb`00?oo043oo`007?oo00<00?ooool02?oo0`004Ooo00<00?ooool02?oo00=S;?ooool0:?oo
00=S;?ooool05ooo00=S;?ooool00V</3_oo00=S;?ooool09?oo00ES;?oooomS;00000fLnP03002D
fYCJ00JDfP<000:DfP03002<VXbJ00^<VP03002<FXaJ00Z<FP0300246H@I00^46@03001kn7_h00Yk
n003001k^7^h00]k^003001kUg^G00akU`030023Uh>G00^3U`030023UX>F00f3UP03002;UX^F00f;
UP05002C]I>eTkD0000:TkD00f</002Ke@0AVmD00`00[3F/=@06[3D016</[3F/=F</1J`e00<00>e@
kE000ne@00<00?F?mHl01_F?00=S;000ool02?oo00=S;000ool0@?oo000Lool00`00ooooo`06ool4
000Aool00`00ooooo`0:ool2Hb`Xool00f</ooooo`0Fool00f</ooooo`03ool00f</ooooo`0;ool0
0f</ooooo`0Tool01F</oooooooo00003Ycj00<009CJU=X00YCJ0`001ICJ00<008bJS9X02hbJ00<0
08aJS5X02haJ00<008@IQ1T02X@I00<007_hNoP02g_h00<007^hNkP02g^h00<007^GNiL02g^G00<0
08>GPiL02h>G00<008>FPiH03H>F00<008^FRiH03H^F00<009>eTkD00Y>e00<009>eTkD01i>e00=S
;9>e00004I_E00<00:`e[3D01Z`e00ES;:`e[3F/=F</00F/=@03003]D>e@00;]D003003eSoF?00Ke
S`03Hb`00?oo00Soo`03Hb`00?oo043oo`007?oo00<00?ooool01ooo00<00?ooool04Ooo00<00?oo
ool03?oo00=S;?ooool09?oo00=S;?ooool05ooo00=S;?ooool01?oo00=S;?ooool02_oo00=S;?oo
ool09?oo00=S;?ooool00_oo00<009cjW?X02icj1@002ICJ00<008bJS9X02hbJ00<008aJS5X02XaJ
00<008@IQ1T02X@I00<007_hNoP02g_h00<007^hNkP02g^h00<007^GNiL02g^G00<008>GPiL038>G
00<008>FPiH038>F00<008^FRiH03H^F00<009>eTkD00i>e00<009>eTkD01Y>e00=S;9>e00004I_E
00<00:`e[3D01Z`e00=S;:`e[3D00Z`e00=S;:`e[3D00Z`e00<00>e@kE000^e@00<00?F?mHl01_F?
00=S;000ool01?oo16</00?oo`00ool0@?oo000Jool00`00ool00009ool00`00ooooo`0Aool3000=
ool00f</ooooo`0Sool00f</ooooo`0Gool00f</ooooo`05ool00f</ooooo`09ool00f</ooooo`0T
ool00f</ooooo`03ool00`00W?ZLnP08W?X4000<U=X00`00S9Z<VP0;S9X00`00S5Z<FP0;S5X00`00
Q1V46@0:Q1T00`00NoQkn00:NoP00`00NkQk^00;NkP00`00NiMkU`0<NiL00`00PiN3U`0;PiL00`00
PiJ3UP0=PiH00`00RiJ;UP0=RiH00`00TkFC]@03TkD00`00TkFC]@05TkD00f</TkD0000AVmD00`00
[3F/=@06[3D00f</[3F/=@02[3D00f</[3F/=@02[3D00`00kE3]D002kE000`00mHoeS`06mHl016</
003ooooo0f</1Ooo00<00?ooool0?ooo000Kool2000:ool4000?ool00`00ooooo`0>ool00f</oooo
o`0Rool00f</ooooo`0Gool00f</ooooo`06ool00f</ooooo`08ool00f</ooooo`0Tool00f</oooo
o`03ool00`00W?ZLnP05W?X30003W?X00`00U=ZDfP0;U=X00`00S9Z<VP0;S9X00`00S5Z<FP0:S5X0
0`00Q1V46@0:Q1T00`00NoQkn00;NoP00`00NkQk^00:NkP00`00NiMkU`0<NiL00`00PiN3U`0;PiL0
0`00PiJ3UP0=PiH00`00RiJ;UP0=RiH00`00TkFC]@04TkD00`00TkFC]@04TkD00f</TkD0000BVmD0
0`00[3F/=@05[3D00f</[3F/=@03[3D01F</[3F/=J`e00001>e@00<00?F?mHl01_F?00AS;000HbaS
;0Soo`03003ooooo03ooo`00>_oo00<00?ooool03ooo0V</8_oo00=S;?ooool05ooo00=S;?ooool0
1ooo00=S;?ooool01ooo00=S;?ooool09?oo00=S;?ooool01?oo00@009cjW?ZLnP<000NLnP03002D
fYCJ00ZDfP03002<VXbJ00^<VP03002<FXaJ00^<FP0300246H@I00Z46@03001kn7_h00Ykn003001k
^7^h00]k^003001kUg^G00]kU`030023Uh>G00b3U`030023UX>F00b3UP03002;UX^F00f;UP03002C
]I>e00FC]@03002C]I>e00>C]@04HbbC]I>e000AVmD00`00[3F/=@05[3D00f</[3F/=@04[3D01F</
[3F/=J`e00000ne@00<00?F?mHl01_F?00=S;000ool02_oo00<00?ooool0?_oo000jool00`00oooo
o`0Aool00f</ooooo`0Oool00f</ooooo`0Gool00f</ooooo`08ool2Hb`7ool00f</ooooo`0Tool0
0f</ooooo`04ool4000;W?X00`00U=ZDfP0:U=X00`00S9Z<VP0;S9X00`00S5Z<FP0:S5X00`00Q1V4
6@0:Q1T00`00NoQkn00:NoP00`00NkQk^00;NkP00`00NiMkU`0;NiL00`00PiN3U`0<PiL00`00PiJ3
UP0<PiH00`00RiJ;UP0=RiH00`00TkFC]@05TkD00`00TkFC]@03TkD016</TkFC]@004I_E00<00:`e
[3D01J`e00=S;:`e[3D01:`e00ES;:`e[3F/=@0000?]D003003eSoF?00KeS`03Hb`00?oo00[oo`03
003ooooo03koo`00>_oo00<00?ooool04_oo00=S;?ooool07_oo00=S;?ooool05ooo00=S;?ooool0
2_oo00=S;?ooool01?oo00=S;?ooool09?oo00AS;?ooooooo`<00003ool009cj00bLnP03002DfYCJ
00ZDfP03002<VXbJ00^<VP03002<FXaJ00^<FP0300246H@I00V46@03001kn7_h00]kn003001k^7^h
00Yk^003001kUg^G00akU`030023Uh>G00^3U`030023UX>F00f3UP03002;UX^F00f;UP03002C]I>e
00FC]@03002C]I>e00:C]@04HbbC]I>e000AVmD00`00[3F/=@06[3D00f</[3F/=@04[3D016</[3F/
=@000ne@00<00?F?mHl01oF?00=S;?ooool02Ooo00<00?ooool0?_oo000iool00`00ooooo`0Dool0
0f</ooooo`0Mool00f</ooooo`0Gool00f</ooooo`0;ool00f</ooooo`03ool00f</ooooo`0Tool0
16</000000001Ooo00<009cjW?X029cj8P002HaJ00<008@IQ1T02X@I00<007_hNoP02W_h00<007^h
NkP02g^h00<007^GNiL02g^G00<008>GPiL02h>G00<008>FPiH03H>F00<008^FRiH03H^F00<009>e
TkD01Y>e00D009>eTkFC]F</00:C]@03002KeI_E012Ke@03002/=J`e00F/=@03Hbb/=J`e00F/=@03
Hbb/=@0000?]D003003eSoF?00OeS`03Hbcooooo00Woo`03003ooooo03koo`00>Ooo00<00?ooool0
5Ooo0V</7Ooo00=S;?ooool05ooo00=S;?ooool03?oo00=S;?ooool00_oo00=S;?ooool08_oo0`00
2?oo00<009cjW?X00Ycj1P0019EL00<008dlSC`02Xdl00<008ClQ?`02hCl00<007BLM9`01`000haJ
00<008@IQ1T02H@I00<007_hNoP02W_h00<007^hNkP02g^h00<007^GNiL02g^G00<008>GPiL038>G
00<008>FPiH038>F00<008^FRiH03H^F00<009>eTkD01i>e00D009>eTkFC]F</00:C]@03002KeI_E
00nKe@03002/=J`e00F/=@03Hbb/=J`e00J/=@03Hbb/=@0000;]D003003eSoF?00OeS`03Hbcooooo
00Woo`03003ooooo03koo`00>Ooo00<00?ooool05ooo00=S;?ooool06Ooo00=S;?ooool06?oo00=S
;?ooool03Ooo00ES;?ooooooof</023oo`@000;oo`03Hbcooooo00Ooo`@000^EG003002=?8dl00Z=
?0030024o8Cl00Z4o003001dW7BL00MdW0H000Z46@03001kn7_h00Ykn003001k^7^h00Yk^003001k
Ug^G00akU`030023Uh>G00^3U`030023UX>F00b3UP03002;UX^F00f;UP03002C]I>e00RC]@04002C
]I>eHb`2TkD00`00VmFKe@0?VmD00`00[3F/=@05[3D00f</[3F/=@06[3D00f</[3D00002kE000`00
mHoeS`07mHl00f</ooooo`09ool00`00ooooo`0nool003Woo`03003ooooo01Soo`03Hbcooooo01So
o`03Hbcooooo01Soo`03Hbcooooo00koo`03Hbcoof</01koo`<000Goo`03Hbcooooo00Woo`03002E
G9EL00^EG003002=?8dl00Z=?0030024o8Cl00^4o003001dW7BL00YdW003001/F`0000<000J46@03
001kn7_h00Ykn003001k^7^h00]k^003001kUg^G00]kU`030023Uh>G00^3U`030023UX>F00f3UP03
002;UX^F00f;UP03002C]I>e00NC]@04002C]I>eHb`2TkD00`00VmFKe@0?VmD00`00[3F/=@05[3D0
0f</[3F/=@07[3D01F</003]D>e@00002OF?00=S;?ooool02Ooo00<00?ooool0?_oo000iool00`00
ooooo`0Iool2Hb`Hool00f</ooooo`0Gool00f</ooooo`0@ool2Hb`Kool30008ool00f</ooooo`0:
ool00`00UEbEG00;UE`00`00SCb=?00:SC`00`00Q?b4o00;Q?`00`00M9adW009M9`00`00K5]/F`03
K5/50002Q1T00`00NoQkn009NoP00`00NkQk^00;NkP00`00NiMkU`0;NiL00`00PiN3U`0<PiL00`00
PiJ3UP0<PiH00`00RiJ;UP0=RiH00`00TkFC]@08TkD00`00TkES;002TkD00`00VmFKe@0@VmD00`00
[3F/=@04[3D00f</[3F/=@08[3D016</kE3]D0002OF?00=S;?ooool02_oo00<00?ooool0?Ooo000i
ool00`00ooooo`0Kool00f</ooooo`0Eool00f</ooooo`0Gool00f</ooooo`0Aool2Hb`Gool3000;
ool00f</ooooo`0:ool00`00UEbEG00<UE`00`00SCb=?00:SC`00`00Q?b4o00:Q?`00`00M9adW00:
M9`00`00K5]/F`07K5/4000;NoP00`00NkQk^00:NkP00`00NiMkU`0;NiL00`00PiN3U`0<PiL00`00
PiJ3UP0<PiH00`00RiJ;UP0=RiH00`00TkFC]@09TkD00`00HbbC]@02TkD00`00VmFKe@0?VmD00`00
[3F/=@04[3D00f</[3F/=@09[3D00f</kE000009mHl00f</ooooo`0:ool00`00ooooo`0mool003So
o`03003ooooo01goo`03Hbcooooo01Coo`03Hbcooooo01Ooo`03Hbcooooo017oo`03Hbcoof</01;o
o`@000koo`03Hbcooooo00_oo`03002EG9EL00bEG003002=?8dl00Z=?0030024o8Cl00Z4o003001d
W7BL00UdW003001/FfaK00Y/F`D000Mkn003001k^7^h00Yk^003001kUg^G00akU`030023Uh>G00^3
U`030023UX>F00b3UP03002;UX^F00f;UP03002C]I>e00ZC]@05002C]I>eTkD0000AVmD00`00[3F/
=@04[3D00f</[3F/=@09[3D00f</kE000009mHl00f</ooooo`0:ool00`00ooooo`0mool003Soo`03
003ooooo01koo`03Hbcooooo01?oo`03Hbcooooo01Ooo`03Hbcooooo017oo`04HbcoooooHb`>ool3
000Bool00f</ooooo`0<ool00`00UEbEG00;UE`00`00SCb=?00:SC`00`00Q?b4o00:Q?`00`00M9ad
W00:M9`00`00K5]/F`09K5/00`00Ho]Sn`02Ho/30004NoP00`00NkQk^00;NkP00`00NiMkU`0;NiL0
0`00PiN3U`0;PiL00`00PiJ3UP0=PiH00`00RiJ;UP0=RiH00`00TkFC]@0:TkD01000TkFC]@004I_E
00<00:`e[3D01:`e00=S;:`e[3D02J`e00<006</00002OF?00<006</ool02_oo00<00?ooool0?Ooo
000hool3000Oool2Hb`Cool00f</ooooo`0Gool00f</ooooo`0Aool01F</ooooooooHb`02_oo0`00
5Ooo00=S;?ooool03Ooo00<009ELUE`02iEL00<008dlSC`02Xdl00<008ClQ?`02XCl00<007BLM9`0
2WBL00<006aKK5/02FaK00<006?kHo/016?k100000=kn000NkP02g^h00<007^GNiL02g^G00<008>G
PiL038>G00<008>FPiH038>F00<008^FRiH03H^F00<009>eTkD02Y>e00@009>eTkD0016Ke@03002/
=J`e00F/=@03Hbb/=J`e00R/=@03003]D6</00WeS`03001S;?oo00[oo`03003ooooo03goo`00>?oo
00<00?ooool08Ooo00=S;?ooool04?oo00=S;?ooool05ooo00=S;?ooool04Ooo00=S;?ooool00_oo
00=S;?ooool01?oo0`006?oo00=S;?ooool03_oo00<009ELUE`02iEL00<008dlSC`02Xdl00<008Cl
Q?`02XCl00<007BLM9`02GBL00<006aKK5/02FaK00<006?kHo/026?k0`002g^h00<007^GNiL037^G
00<008>GPiL02h>G00<008>FPiH038>F00<008^FRiH03H^F00<009>eTkD02i>e00@009>eTkD0016K
e@03002/=J`e00B/=@03Hbb/=J`e00V/=@800003HbceSoF?00KeS`03001S;?oo00[oo`03003ooooo
03goo`00>?oo00<00?ooool08_oo00=S;?ooool03ooo00=S;?ooool05ooo00=S;?ooool04Ooo00=S
;?ooool00ooo00=S;?oo00000`006ooo00=S;?ooool03ooo00<009ELUE`02iEL00<008dlSC`02Xdl
00<008ClQ?`02HCl00<007BLM9`02WBL00<006aKK5/02FaK00<006?kHo/02F?k00=KfP0000002W^h
00<007^GNiL02g^G00<008>GPiL02h>G00<008>FPiH038>F00<008^FRiH03H^F00<009>eTkD039>e
00<009>e00004I_E00<00:`e[3D01:`e00=S;:`e[3D02J`e0P0000=S;?F?mHl01_F?00<006</ool0
2_oo00<00?ooool0?Ooo000hool00`00ooooo`0Sool00f</ooooo`0=ool00f</ooooo`0Hool00f</
ooooo`0Aool00f</ooooo`02ool3000Oool00f</ooooo`0?ool00`00UEbEG00;UE`00`00SCb=?00:
SC`00`00Q?b4o00:Q?`00`00M9adW009M9`00`00K5]/F`09K5/00`00Ho]Sn`09Ho/00`00FmYKfP03
0007NkP00`00NiMkU`0;NiL00`00PiN3U`0;PiL00`00PiJ3UP0=PiH00`00RiJ;UP0<RiH00`00TkFC
]@0=TkD2000AVmD00`00[3F/=@04[3D00f</[3F/=@09[3D200000oF?HbceS`06mHl00`00Hbcoo`0:
ool00`00ooooo`0mool003Ooo`03003ooooo02Goo`9S;0goo`03Hbcooooo01Soo`03Hbcooooo017o
o`03Hbcoo`00008000?oo`9S;1goo`03Hbcooooo013oo`03002EG9EL00^EG003002=?8dl00Z=?003
0024o8Cl00Z4o003001dW7BL00UdW003001/FfaK00U/F`03001Snf?k00USn`03001KfU_J009KfP80
00Ek^003001kUg^G00]kU`030023Uh>G00b3U`030023UX>F00b3UP03002;UX^F00f;UP03002C]I>e
00fC]@03002KeI_E00nKe@03002/=J`e00B/=@03Hbb/=J`e00V/=@8000;eS`03HbceSoF?00CeS`03
001S;?oo00_oo`03003ooooo03coo`00=ooo00<00?ooool09ooo00=S;?ooool02_oo00=S;?ooool0
6?oo00=S;?ooool04?oo0`002?oo00=S;?ooool06_oo00=S;?ooool04Ooo00<009ELUE`01IEL6P00
1HCl00<007BLM9`02WBL00<006aKK5/026aK00<006?kHo/02F?k00<005_JFmX015_J0`000W^h00<0
07^GNiL037^G00<008>GPiL02h>G00<008>FPiH038>F00<008^FRiH03H^F00<009>eTkD03I>e0P00
4I_E00<00:`e[3D00j`e00=S;:`e[3D02Z`e00D00?F?mHoeSf</00GeS`03001S;?oo00_oo`03003o
oooo03coo`00=ooo00<00?ooool0:?oo00=S;?ooool02Ooo00=S;?ooool05ooo00=S;?ooool03_oo
0`0000?oof</ool02Ooo00=S;?ooool06Ooo00=S;?ooool04_oo1`006_oo2@002GBL00<006aKK5/0
2FaK00<006?kHo/02F?k00<005_JFmX01U_J0P0000=k^000NiL037^G00<008>GPiL02h>G00<008>F
PiH03H>F00<008^FRiH038^F00<009>eTkD03Y>e00<009_EVmD03i_E00<00:`e[3D00j`e00=S;:`e
[3D02Z`e00D00?F?mHoeSf</00GeS`03001S;?oo00_oo`03003ooooo03coo`00=ooo00<00?ooool0
:Ooo0V</2Ooo00=S;?ooool05ooo00=S;?ooool02_oo10000ooo00=S;?ooool02_oo00=S;?ooool0
6?oo00=S;?ooool04_oo00<00?ooool09ooo10001WBL00<006aKK5/02FaK00<006?kHo/026?k00<0
05_JFmX025_J0`0037^G00<008>GPiL038>G00<008>FPiH038>F00<008^FRiH038^F00<009>eTkD0
3Y>e0P0049_E00<00:`e[3D00j`e00=S;:`e[3D02Z`e00<00?F?mHl00_F?00=S;?F?mHl00_F?00<0
06</ool02ooo00<00?ooool0??oo000gool00`00ooooo`0[ool00f</ooooo`06ool00f</ooooo`0G
ool00f</ooooo`07ool30007ool00f</ooooo`0;ool00f</ooooo`0Gool00f</ooooo`0Bool00`00
ooooo`0[ool500000gBL001/F`0:K5/00`00Ho]Sn`09Ho/00`00FmYKfP09FmX00e^j0000000;NiL0
0`00PiN3U`0;PiL00`00PiJ3UP0<PiH00`00RiJ;UP0=RiH00`00TkFC]@0=TkD00`00VmD0000?VmD0
0`00[3F/=@03[3D00f</[3F/=@09[3D20005mHl01f</mHoeSoF?003oof</00_oo`03003ooooo03co
o`00=ooo00<00?ooool0;?oo00=S;?ooool01Ooo00=S;?ooool05ooo00=S;?ooool01?oo0`002_oo
00=S;?ooool03?oo00=S;?ooool05_oo00=S;?ooool04_oo00<00?ooool0<?oo10002VaK00<006?k
Ho/026?k00<005_JFmX02E_J00<005^jFkX00`0027^G00<008>GPiL02h>G00<008>FPiH038>F00<0
08^FRiH03H^F00<009>eTkD03Y>e00<009_E00003i_E00<00:`e[3D00j`e00=S;:`e[3D01j`e00@0
0;cF_=H000GeS`06HbceSoF?003oof</2ooo00<00?ooool0??oo000fool00`00ooooo`0^ool00f</
ooooo`04ool00f</ooooo`0Gool016</oooooooo0`003Ooo00=S;?ooool03Ooo0V</5_oo00=S;?oo
ool04_oo00<00?ooool0=?oo10001VaK00<006?kHo/02F?k00<005_JFmX02E_J00<005^jFkX00U^j
0`001G^G00<008>GPiL02h>G00<008>FPiH03H>F00<008^FRiH038^F00<009>eTkD03Y>e00<009_E
00003i_E00<00:`e[3D00j`e00=S;:`e[3D01Z`e00D00;cF_=JleP0000GeS`06HbceSoF?003oof</
2ooo00<00?ooool0??oo000fool00`00ooooo`0_ool2Hb`3ool00f</ooooo`0Hool4000@ool00f</
ooooo`0?ool00f</ooooo`0Cool00f</ooooo`0Bool00`00ooooo`0hool30004K5/00`00Ho]Sn`09
Ho/00`00FmYKfP08FmX00`00FkYK^P05FkX20003NiL00`00PiN3U`0<PiL00`00PiJ3UP0<PiH00`00
RiJ;UP0<RiH00`00TkFC]@0>TkD01000VmFKe@003Y_E00<00:`e[3D00j`e00=S;:`e[3D01J`e00<0
0;cF_=H00[cF00<00?F?mHl01?F?00ES;?F?003oof</00coo`03003ooooo03_oo`00=_oo0`00<Ooo
00AS;?oooomS;1Ooo`<001Coo`03Hbcooooo013oo`03Hbcooooo01;oo`03Hbcooooo01;oo`03003o
oooo03_oo`D000]Sn`03001KfU_J00UKfP03001K^U^j00IK^P<00003NiL008>G00b3U`030023UX>F
00b3UP03002;UX^F00f;UP03002C]I>e00fC]@05002KeI_EVmD0000=VmD00`00[3F/=@03[3D00f</
[3F/=@05[3D00`00_=JleP03_=H00`00mHoeS`04mHl016</003oof</3?oo00<00?ooool0>ooo000f
ool00`00ooooo`0bool00f</oomS;00Dool3000Gool00f</ooooo`0Aool00f</ooooo`0Aool00f</
ooooo`0Bool00`00ooooo`0oool3000:Ho/00`00FmYKfP08FmX00`00FkYK^P09FkX3000<PiL00`00
PiJ3UP0<PiH00`00RiJ;UP0=RiH00`00TkFC]@0>TkD01@00VmFKeI_E000039_E00<00:`e[3D00j`e
00=S;:`e[3D01:`e00<00;cF_=H01;cF00<00?F?mHl01?F?00AS;000oomS;0coo`03003ooooo03_o
o`00=_oo00<00?ooool0<ooo0V</4Ooo0`006_oo00=S;?ooool04_oo00=S;?ooool03ooo00=S;?oo
ool04ooo00<00?ooool0@?oo00AS;?oo000000QSn`03001KfU_J00UKfP03001K^U^j00UK^P04001S
^@00000;PiL00`00PiJ3UP0<PiH00`00RiJ;UP0<RiH00`00TkFC]@0>TkD00`00VmFKe@02VmD00`00
VmFKe@0:VmD00`00[3F/=@02[3D00f</[3F/=@03[3D00`00_=JleP05_=H00`00mHoeS`05mHl00f</
001S;00<ool00`00ooooo`0kool003Koo`03003ooooo03Coo`9S;0coo`@001goo`03Hbcooooo01?o
o`03Hbcooooo00koo`03Hbcooooo01?oo`03003ooooo043oo`04Hbcoooooool30006Ho/00`00FmYK
fP08FmX00`00FkYK^P0:FkX00`00HkUS^@030008PiL00`00PiJ3UP0<PiH00`00RiJ;UP0<RiH00`00
TkFC]@0>TkD00`00VmFKe@03VmD00`00VmFKe@09VmD00`00[3F/=@02[3D00f</[3F/=@02[3D00`00
_=JleP07_=H00`00mHoeS`05mHl2Hb`<ool00`00ooooo`0kool003Koo`03003ooooo03Coo`03Hbco
of</00Soo`<0027oo`03Hbcooooo01Coo`03Hbcooooo00goo`03Hbcooooo01?oo`03003ooooo043o
o`03Hbcooooo00Coo`8000ASn`03001KfU_J00UKfP03001K^U^j00UK^P03001S^F>i00=S^@<000F3
U`030023UX>F00b3UP03002;UX^F00f;UP03002C]I>e00fC]@03002KeI_E00>Ke@03002KeI_E00VK
e@03002/=J`e00:/=@05Hbb/=J`e[3D0000:_=H00`00mHoeS`05mHl00`00HbaS;00;ool00`00oooo
o`0kool003Goo`03003ooooo03Goo`04HbcoooooHb`4ool3000Tool00f</ooooo`0Eool2Hb`=ool0
0f</ooooo`0Cool00`00ooooo`10ool00f</ooooo`06ool30002Ho/00`00FmYKfP08FmX00`00FkYK
^P0:FkX00`00HkUS^@05HkT20003PiL00`00PiJ3UP0<PiH00`00RiJ;UP0=RiH00`00TkFC]@0>TkD0
0`00VmFKe@03VmD00`00VmFKe@08VmD00`00[3F/=@02[3D016</[3F/=@003;cF00<00?F?mHl01?F?
00<006</Hb`02ooo00<00?ooool0>ooo000eool00`00ooooo`0eool01F</ooooooooHb`00`009ooo
00=S;?ooool05ooo00=S;?ooool02_oo00=S;?ooool04ooo00<00?ooool0@?oo00=S;?ooool02Ooo
0`002e_J00<005^jFkX02E^j00<006>iHkT01f>i0`0000>3U`00PiH03H>F00<008^FRiH038^F00<0
09>eTkD03Y>e00<009_EVmD019_E00<009_EVmD029_E00<00:`e[3D00Z`e00=S;000_=H03;cF00<0
0?F?mHl01?F?00<00?ooHb`02ooo00<00?ooool0>ooo000eool00`00ooooo`0eool00f</00000002
00000oooHbcoo`0Vool00f</ooooo`0Iool00f</ooooo`09ool00f</ooooo`0Cool00`00ooooo`10
ool00f</ooooo`0;ool30009FmX00`00FkYK^P0:FkX00`00HkUS^@09HkT3000=PiH00`00RiJ;UP0<
RiH00`00TkFC]@0>TkD00`00VmFKe@05VmD00`00VmFKe@07VmD00`00[3F/=@02[3D00f</_=JleP0<
_=H00`00mHoeS`04mHl01000oomS;6</2ooo00<00?ooool0>_oo000eool00`00ooooo`0cool30006
ool00f</ooooo`0Tool00f</ooooo`0Jool00f</ooooo`08ool00f</ooooo`0Cool00`00ooooo`10
ool00f</ooooo`0>ool20008FmX00`00FkYK^P09FkX00`00HkUS^@0:HkT00`00LmT000030009PiH0
0`00RiJ;UP0=RiH00`00TkFC]@0=TkD00`00VmFKe@06VmD00`00VmFKe@06VmD00`00[3F/=@02[3D0
0f</_=JleP0=_=H00`00mHoeS`03mHl01@00oomS;?ooHb`02_oo00<00?ooool0>_oo000eool00`00
ooooo`0`ool300000oooHbcoo`07ool00f</ooooo`0Sool00f</ooooo`0Kool00f</ooooo`07ool0
0f</ooooo`0Bool00`00ooooo`11ool00f</ooooo`0@ool20006FmX00`00FkYK^P09FkX00`00HkUS
^@0;HkT00`00LmUcf@02LmT40005PiH00`00RiJ;UP0=RiH00`00TkFC]@0>TkD00`00VmFKe@05VmD0
0`00VmFKe@06VmD01P00[3F/=J`e001S;0nleP03003eSoF?00?eS`05003oof</oomS;00:ool00`00
ooooo`0jool003Goo`03003ooooo02goo`<000Coo`03Hbcooooo00Ooo`9S;2?oo`03Hbcooooo01co
o`03Hbcooooo00Koo`03Hbcooooo01;oo`03003ooooo047oo`03Hbcooooo017oo`03Hb`0000000EK
fP03001K^U^j00UK^P03001S^F>i00YS^@03001cfG?I00Icf@@000:3UP03002;UX^F00b;UP03002C
]I>e00jC]@03002KeI_E00JKe@03002KeI_E00JKe@05002/=@00_=IS;00?_=H00`00mHoeS`03mHl0
0`00oomS;002ool00f</ooooo`07ool00`00ooooo`0jool003Coo`03003ooooo02[oo`@000Ooo`03
Hbcooooo00Woo`03Hbcooooo023oo`03Hbcooooo01goo`03Hbcooooo00Goo`03Hbcooooo01;oo`03
003ooooo047oo`03Hbcooooo00goo`AS;0?oo`8000=KfP03001K^U^j00UK^P03001S^F>i00]S^@03
001cfG?I00Ucf@D000b;UP03002C]I>e00jC]@03002KeI_E00NKe@03002KeI_E00FKe@8000:leP03
Hbble[cF00jleP03003eSoF?00;eS`03003oof</00?oo`03Hbcooooo00Koo`03003ooooo03[oo`00
=?oo0`009ooo0`002ooo00=S;?ooool02_oo00=S;?ooool07ooo00=S;?ooool07_oo0V</1Ooo00=S
;?ooool04_oo00<00?ooool0@Ooo00=S;?ooool02Ooo16</2Ooo0P000U_J00<005^jFkX02E^j00<0
06>iHkT02V>i00<007?ILmT02g?I00<007_hNoP01@0028^F00<009>eTkD03I>e00<009_EVmD029_E
00<009_EVmD00Y_E0`000kcF00=S;;cF_=H03[cF00<00?F?mHl00_F?00@00?oooomS;0?oo`03Hbco
oooo00Goo`03003ooooo03[oo`00=?oo00<00?ooool09?oo0`003_oo00=S;?ooool02ooo00=S;?oo
ool07_oo00=S;?ooool08?oo00=S;?ooool00_oo00=S;?ooool04_oo00<00?ooool0@Ooo00=S;?oo
ool01_oo0f</3ooo0`002e^j00<006>iHkT02V>i00<007?ILmT037?I00<007_hNoP017_h1P000X^F
00<009>eTkD03Y>e00<009_EVmD01Y_E1P000Y@g00D00;cF_=JleV</012leP03003eSoF?00;eS`04
003oooooHb`3ool00f</ooooo`05ool00`00ooooo`0jool003Coo`03003ooooo027oo`<0017oo`03
Hbcooooo00coo`9S;1koo`03Hbcooooo027oo`05HbcooooooomS;00Dool00`00ooooo`11ool00f</
ooooo`02ool4Hb`Dool2000;FkX00`00HkUS^@0:HkT00`00LmUcf@0;LmT00`00NoQkn00:NoP;0008
TkD00`00VmFKe@060002U3L00`00U3ND=`04U3L01000_=JleV</4KcF00D00?F?mHoeS`0000;oo`03
Hbcooooo00;oo`03Hbcooooo00Goo`03003ooooo03Woo`00=?oo00<00?ooool07_oo0`005?oo00=S
;?ooool03_oo00=S;?ooool06ooo00=S;?ooool08_oo00AS;?oooomS;1Coo`03003ooooo047oo`03
Hbcoof</009S;1[oo`8000UK^P03001S^F>i00YS^@03001cfG?I00acf@03001kn7_h00]kn003002<
68`H00J<60/000VD=`03002D=i@g00>D=`05002le[cF_=IS;00@_=H01@00mHoeSoF?00000_oo00=S
;?ooool00ooo00=S;?ooool01?oo00<00?ooool0>Ooo000dool00`00ooooo`0Jool4000Fool00f</
ooooo`0@ool00f</ooooo`0Jool00f</ooooo`0Sool00f</oomS;00Dool00`00ooooo`0oool4Hb`O
ool20007FkX00`00HkUS^@0;HkT00`00LmUcf@0;LmT00`00NoQkn00<NoP00`00S1R<600=S1P00`00
U3ND=`0:U3L00`00U3ND=`02U3L01@00_=Jle[cFHb`04;cF00D00?F?mHoeS`0000;oo`03Hbcooooo
00Coo`03Hbcooooo00?oo`03003ooooo03Woo`00<ooo00<00?ooool06?oo0`006_oo00=S;?ooool0
4Ooo0V</6_oo00=S;?ooool09?oo0V</5?oo00<00?ooool0>ooo16</0_oo00=S;?ooool08?oo0P00
1U^j00<006>iHkT02V>i00<007?ILmT02g?I00<007_hNoP037_h00<008`HS1P03X`H00<009@gU3L0
2Y@g00<009@gU3L00Y@g00@00;cF_=IS;16leP04003eSoF?0002ool00f</ooooo`04ool00f</oooo
o`03ool00`00ooooo`0iool003?oo`03003ooooo01Goo`<001goo`03Hbcooooo01?oo`03Hbcooooo
01Ooo`03Hbcooooo02Goo`9S;1?oo`03003ooooo03Soo`=S;0Koo`03Hbcooooo02;oo`8000AK^P03
001S^F>i00YS^@03001cfG?I00acf@03001kn7_h00akn003002<68`H00f<6003002D=i@g00^D=`05
002D=i@gU3L00002_=H00f</_=JleP0?_=H01000mHoeS`000_oo00=S;?ooool01Ooo00=S;?ooool0
0_oo00<00?ooool0>Ooo000cool00`00ooooo`0Bool3000Pool00f</ooooo`0Dool00f</ooooo`0E
ool00f</ooooo`0Vool00f</oomS;00Bool00`00ooooo`0dool4Hb`:ool00f</ooooo`0Sool20003
FkX00`00HkUS^@0:HkT00`00LmUcf@0;LmT00`00NoQkn00<NoP00`00S1R<600=S1P00`00U3ND=`0;
U3L01@00U3ND=i@g00000[cF00=S;;cF_=H04;cF00<00?F?00000_oo00=S;?ooool01_oo00ES;?oo
ooooo`0003_oo`00<ooo00<00?ooool03_oo10008ooo00=S;?ooool05Ooo00=S;?ooool05?oo00=S
;?ooool09_oo00AS;?oooomS;17oo`03003ooooo037oo`=S;0koo`03Hbcooooo02Goo`800003FkX0
06>i00]S^@03001cfG?I00]cf@03001kn7_h00akn003002<68`H00j<6003002D=i@g00^D=`07002D
=i@gU3L00;cFHb`04[cF00<00?F?00000ooo00=S;?ooool01_oo00AS;?ooool003_oo`00<ooo00<0
0?ooool02ooo0`009ooo00=S;?ooool05_oo0V</5?oo00=S;?ooool09Ooo00=S;?ooool00_oo00=S
;?ooool03_oo00<00?ooool0;Ooo16</4Ooo00=S;?ooool09ooo0`002f>i00<007?ILmT037?I00<0
07_hNoP037_h00<008`HS1P03H`H00<009@gU3L039@g00H009@gU3L00;cFHb`B_=H00`00mHl00003
ool00f</ooooo`06ool01F</oooooooo0000>_oo000cool00`00ooooo`08ool3000Zool00f</oooo
o`0Hool00f</ooooo`0Aool00f</ooooo`0Uool00f</ooooo`03ool00f</ooooo`0=ool00`00oooo
o`0Yool4Hb`Eool00f</ooooo`0Yool00`00HkUS^@0:HkT00`00LmUcf@0;LmT00`00NoQkn00<NoP0
0`00S1R<600=S1P00`00U3ND=`0=U3L01@00U3L00;cFHb`04kcF0P000ooo00=S;?ooool01ooo00AS
;?ooool003[oo`00<_oo00<00?ooool01_oo0`00;Ooo00=S;?ooool06Ooo00=S;?ooool04?oo00=S
;?ooool09Ooo00=S;?ooool01?oo00=S;?ooool03?oo00<00?ooool09_oo0f</6Ooo00=S;?ooool0
:_oo0P002V>i00<007?ILmT037?I00<007_hNoP037_h00<008`HS1P03H`H00<009@gU3L03I@g00@0
09@g001S;1>leP8000?oo`03Hbcooooo00Soo`03Hbcoo`0003[oo`00<_oo00<00?ooool00_oo1000
<?oo00=S;?ooool06_oo00=S;?ooool03ooo00=S;?ooool09Ooo00=S;?ooool01Ooo00=S;?ooool0
2ooo00<00?ooool08_oo16</7?oo00=S;?ooool0;?oo0P002F>i00<007?ILmT02g?I00<007_hNoP0
37_h00<008`HS1P03H`H00<009@gU3L03Y@g0P0000>leV</_=H04KcF0P000ooo00=S;?ooool02Ooo
00=S;000ool0>Ooo000bool00`00ool00002000cool00f</ooooo`0Lool2Hb`?ool00f</ooooo`0U
ool00f</ooooo`06ool2Hb`;ool00`00ooooo`0Oool3Hb`Pool00f</ooooo`0^ool20007HkT00`00
LmUcf@0;LmT00`00NoQkn00<NoP00`00S1R<600=S1P00`00U3ND=`0>U3L200000kcFHbbleP0B_=H0
1@00ooooooooHb`02ooo00=S;000ool0>Ooo000bool2000fool00f</ooooo`0Nool00f</ooooo`0<
ool00f</ooooo`0Uool00f</ooooo`08ool00f</ooooo`08ool00`00ooooo`0Kool4Hb`Sool00f</
ooooo`0`ool20005HkT00`00LmUcf@0<LmT00`00NoQkn00<NoP00`00S1R<600=S1P00`00U3ND=`0>
U3L200000f</_=JleP0A_=H01@00ooooooooHb`02ooo00=S;000ool0>Ooo0006ool50002ool20004
ool00`00ooooo`03ool4000Fool2000eool00f</ooooo`0Oool00f</ooooo`0;ool00f</ooooo`0U
ool00f</ooooo`09ool00f</ooooo`07ool00`00ooooo`0Gool4Hb`Wool00f</ooooo`0bool20004
HkT00`00LmUcf@0;LmT00`00NoQkn00<NoP00`00S1R<600=S1P00`00U3ND=`0?U3L00`00HbbleP0A
_=H00`00ooooo`02ool00f</ooooo`05ool4Hb`00ooo003oo`0iool000Soo`03003ooooo00?oo`03
003ooooo00;oo`03003ooooo00Ooo`03003ooooo01Goo`03003ooooo03;oo`03Hbcooooo023oo`9S
;0_oo`03Hbcooooo02Goo`03Hbcooooo00[oo`03Hbcooooo00Koo`03003ooooo01Coo`=S;2_oo`03
Hbcooooo03Coo`80009S^@03001cfG?I00acf@03001kn7_h00]kn003002<68`H00f<6003002D=i@g
00nD=`80016leP03003ooooo00Coo`03Hbcooooo00AS;0Goo`03003ooooo03Soo`002?oo00<00?oo
ool02Ooo00<00?ooool01ooo00<00?ooool05Ooo0P00<_oo00=S;?ooool08_oo00=S;?ooool02?oo
00=S;?ooool09Ooo00=S;?ooool02ooo00=S;?ooool01Ooo00<00?ooool04?oo16</;_oo00=S;?oo
ool0=_oo0P0000=S^@00LmT037?I00<007_hNoP037_h00<008`HS1P03H`H00<009@gU3L03i@g0P00
4;cF00<00?ooool00ooo16</2_oo00<00?ooool0=ooo0008ool00`00ooooo`09ool00`00ooooo`07
ool00`00ooooo`0Gool00`00ooooo`0_ool00f</ooooo`0Sool00f</ooooo`06ool00f</ooooo`0V
ool00f</ooooo`0<ool00f</ooooo`04ool00`00ooooo`0=ool3Hb`bool00f</ooooo`0hool3000<
LmT00`00NoQkn00<NoP00`00S1R<600=S1P00`00U3ND=`0?U3L2000?_=H00`00ooooo`04Hb`00ooo
Hbcoo`0;ool00`00ooooo`0gool000Soo`03003ooooo00Woo`03003ooooo00Ooo`03003ooooo01So
o`03003ooooo02koo`03Hbcooooo02Coo`03Hbcooooo00Goo`03Hbcooooo02Koo`03Hbcooooo00go
o`03Hbcooooo00?oo`03003ooooo00Woo`AS;3Goo`03Hbcooooo03_oo`8000]cf@03001kn7_h00ak
n003002<68`H00b<6003002D=i@g012D=`8000fleP04001S;6</Hb`5ool00f</ooooo`0:ool00`00
ooooo`0gool000Soo`03003ooooo00[oo`03003ooooo00Goo`8001_oo`8002koo`03Hbcooooo02Go
o`9S;0Goo`03Hbcooooo02Koo`03Hbcooooo00koo`9S;0?oo`03003ooooo00Goo`AS;3Woo`03Hbco
oooo03goo`<000Qcf@03001kn7_h00akn003002<68`H00f<6003002D=i@g00nD=`03002leP0000^l
eP03003ooooo00Ooo`03Hbcooooo00[oo`03003ooooo03Ooo`002?oo00<00?ooool02_oo00<00?oo
ool00ooo0`007_oo00<00?ooool0:_oo00=S;?ooool0:?oo00=S;?ooool00_oo00=S;?ooool09_oo
00=S;?ooool04?oo00AS;?ooool000Coo`=S;3goo`03Hbcooooo043oo`8000Icf@03001kn7_h00ak
n003002<68`H00f<6003002D=i@g00nD=`04002le[cF0009_=H00`00ooooo`08ool00f</ooooo`0:
ool00`00ooooo`0gool000Soo`03003ooooo00[oo`04003oooooool4000Pool2000Zool00f</oooo
o`0Yool01F</ooooooooHb`0:?oo00=S;?ooool04Ooo00=S;?oo000016</@?oo00=S;?ooool0@_oo
0P001G?I00<007_hNoP037_h00<008`HS1P038`H00<009@gU3L049@g00@00;cF_=H000NleP03003o
oooo00Woo`03Hbcooooo00[oo`03003ooooo03Ooo`002?oo00<00?ooool01Ooo00<00?ooool00ooo
00D00?ooooooo`0002Coo`03003ooooo02Ooo`03Hbcooooo02[oo`04HbcoooooHb`Xool00f</oooo
o`0Bool00f</003oo`13ool00f</ooooo`14ool30002LmT00`00NoQkn00<NoP00`00S1R<600=S1P0
0`00U3ND=`0?U3L01000_=JleP001kcF00<00?ooool02Ooo00=S;?ooool02_oo00<00?ooool0=ooo
0006ool00`00ool00007ool00`00ooooo`03ool01@00oooooooo00009Ooo0P009ooo00=S;?ooool0
:ooo0f</:?oo00=S;?ooool04ooo00<00?ooool0@_oo00=S;?ooool0Aooo0P0000=cf@00NoP03G_h
00<008`HS1P038`H00<009@gU3L03i@g00D00;cF_=JleP0000FleP03003ooooo00_oo`03Hbcooooo
00[oo`03003ooooo03Koo`001ooo0P002?oo1P001?oo10008ooo00<00?ooool09?oo00=S;?ooool0
;Ooo00=S;?ooool09_oo00=S;?ooool04ooo00<00?ooool0@_oo00=S;?ooool0BOoo0`003G_h00<0
08`HS1P038`H00<009@gU3L049@g00D00;cF_=JleP0000>leP03003ooooo00coo`03Hbcooooo00[o
o`03003ooooo03Koo`00@ooo00<00?ooool08ooo00=S;?ooool0;Ooo0V</9ooo00=S;?ooool04_oo
00<00?ooool0@ooo00=S;?ooool0B_oo00=S;?oo00000`002G_h00<008`HS1P03H`H00<009@gU3L0
3i@g00<00;cF_=H00[cF00<00;cF00003ooo00=S;?ooool02_oo00<00?ooool0=_oo0014ool2000S
ool00f</ooooo`0]ool016</oomS;6</9?oo00=S;?ooool04ooo00<00?ooool0@ooo00=S;?ooool0
B_oo00=S;?ooool00ooo0`001g_h00<008`HS1P038`H00<009@gU3L03i@g00<00;cF_=H00kcF00<0
0?ooool03_oo00=S;?ooool02_oo00<00?ooool0=_oo0016ool00`00ooooo`0Pool00f</ooooo`0]
ool01F</ooooooooHb`08ooo00=S;?ooool04ooo00<00?ooool0@ooo00=S;?ooool0B_oo00=S;?oo
ool01_oo0`0017_h00<008`HS1P038`H00<009@gU3L049@g00@00;cF_=JleP80013oo`03Hbcooooo
00[oo`03003ooooo03Koo`00Aooo0P007ooo00=S;?ooool0;_oo00=S;?ooool00_oo00=S;?ooool0
8?oo00=S;?ooool04ooo00<00?ooool0@ooo00=S;?ooool0B_oo00=S;?ooool02Ooo100000=kn000
S1P03H`H00<009@gU3L03i@g00L00;cF_=JleP00ool0000?ool00f</ooooo`0:ool00`00ooooo`0f
ool004Woo`03003ooooo01coo`03Hbcooooo02goo`03Hbcooooo00Coo`03Hbcooooo01ooo`03Hbco
oooo01?oo`03003ooooo04?oo`03Hbcooooo04[oo`03Hbcooooo00goo`@000b<6003002D=i@g00nD
=`04002le[cF0003ool00`00ooooo`0<ool00f</ooooo`0:ool00`00ooooo`0fool004[oo`8001co
o`03Hbcooooo02goo`03Hbcooooo00Goo`9S;1ooo`03Hbcooooo01?oo`03003ooooo04?oo`03Hbco
oooo04[oo`03Hbcooooo00coo`AS;003ool0000000<000N<6003002D=i@g012D=`8000Goo`03003o
oooo00_oo`03Hbcooooo00[oo`03003ooooo03Koo`00C?oo00<00?ooool06Ooo00=S;?ooool0;Ooo
00=S;?ooool01ooo00=S;?ooool07?oo00=S;?ooool04ooo00<00?ooool0@ooo00=S;?ooool0B_oo
00=S;?ooool02?oo16</2_oo1P000X`H00<009@gU3L01Y@g2P001ooo00<00?ooool02ooo00=S;?oo
ool02_oo00<00?ooool0=Ooo001=ool00`00ooooo`0Hool00f</ooooo`0]ool00f</ooooo`08ool0
0f</ooooo`0Kool00f</ooooo`0Cool00`00ooooo`13ool00f</ooooo`1;ool00f</ooooo`03ool4
Hb`Dool;000Aool00`00ooooo`0;ool00f</ooooo`0:ool00`00ooooo`0eool004koo`8001Soo`03
Hbcooooo02goo`03Hbcooooo00Woo`03Hbcooooo01[oo`03Hbcooooo01?oo`03003ooooo04?oo`03
Hbcooooo04_oo`03Hbcoof</00=S;3Goo`03003ooooo00[oo`03Hbcooooo00[oo`03003ooooo03Go
o`00D?oo00<00?ooool05Ooo00=S;?ooool0;Ooo00=S;?ooool02_oo0V</6_oo00=S;?ooool04ooo
00<00?ooool0@ooo00=S;?ooool0BOoo16</>_oo00<00?ooool02Ooo00=S;?ooool02_oo00<00?oo
ool0=Ooo001Aool2000Eool00f</ooooo`0]ool00f</ooooo`0<ool00f</ooooo`0Gool00f</oooo
o`0Cool00`00ooooo`13ool00f</ooooo`15ool4Hb`2ool00f</ooooo`0jool00`00ooooo`08ool0
0f</ooooo`0:ool00`00ooooo`0eool005?oo`03003ooooo01;oo`03Hbcooooo02goo`03Hbcooooo
00goo`03Hbcooooo01Koo`03Hbcooooo01?oo`03003ooooo04?oo`03Hbcooooo047oo`AS;0Koo`03
Hbcooooo03_oo`03003ooooo00Ooo`03Hbcooooo00[oo`03003ooooo03Goo`00E?oo0P004Ooo00=S
;?ooool0;_oo00=S;?ooool03_oo0V</5_oo00=S;?ooool04ooo00<00?ooool0@ooo00=S;?ooool0
?_oo0f</2_oo00=S;?ooool0??oo00<00?ooool01_oo00=S;?ooool02_oo00<00?ooool0=Ooo001F
ool00`00ooooo`0>ool00f</ooooo`0^ool00f</ooooo`0@ool00f</ooooo`0Cool00f</ooooo`0C
ool00`00ooooo`13ool00f</ooooo`0jool4Hb`=ool00f</ooooo`0lool00`00ooooo`06ool00f</
ooooo`0;ool00`00ooooo`0dool005Ooo`03003ooooo00goo`03Hbcooooo02koo`03Hbcooooo017o
o`03Hbcooooo01;oo`03Hbcooooo01?oo`03003ooooo04?oo`03Hbcooooo03Koo`AS;17oo`03Hbco
oooo03goo`03003ooooo00Koo`03Hbcooooo00[oo`03003ooooo03Coo`00F?oo0P003Ooo00=S;?oo
ool0;_oo00=S;?ooool04_oo00=S;?ooool04Ooo00=S;?ooool04ooo00<00?ooool0@ooo00=S;?oo
ool0<_oo16</5Ooo00=S;?ooool0?_oo00<00?ooool01Ooo00=S;?ooool02_oo00<00?ooool0=?oo
001Jool00`00ooooo`0:ool00f</ooooo`0]ool00f</ooooo`0Dool2Hb`Aool00f</ooooo`0Cool0
0`00ooooo`13ool00f</ooooo`0^ool4Hb`Iool00f</ooooo`0oool00`00ooooo`04ool00f</oooo
o`0:ool00`00ooooo`0dool005_oo`8000[oo`03Hbcooooo02goo`03Hbcooooo01Koo`03Hbcooooo
00koo`03Hbcooooo01?oo`03003ooooo04?oo`03Hbcooooo02[oo`AS;1goo`03Hbcooooo043oo`03
003ooooo00?oo`03Hbcooooo00[oo`03003ooooo03Coo`00GOoo00<00?ooool01ooo00=S;?ooool0
;Ooo00=S;?ooool05ooo00=S;?ooool03Ooo00=S;?ooool04ooo00<00?ooool0@ooo00=S;?ooool0
9_oo16</8Ooo00=S;?ooool0@?oo00<00?ooool00ooo00=S;?ooool02_oo00<00?ooool0=?oo001N
ool20007ool00f</ooooo`0]ool00f</ooooo`0Hool00f</ooooo`0<ool00f</ooooo`0Cool00`00
ooooo`13ool00f</ooooo`0Rool4Hb`Uool00f</ooooo`11ool00`00ooooo`02ool00f</ooooo`0:
ool00`00ooooo`0dool0063oo`03003ooooo00?oo`03Hbcooooo02koo`03Hbcooooo01Woo`9S;0_o
o`03Hbcooooo01Coo`03003ooooo04?oo`03Hbcooooo01koo`AS;2[oo`03Hbcooooo047oo`05003o
oooooomS;00<ool00`00ooooo`0dool0067oo`8000?oo`03Hbcooooo02koo`03Hbcooooo01_oo`03
Hbcooooo00Soo`03Hbcooooo01Coo`03003ooooo04?oo`03Hbcooooo01[oo`AS;2koo`03Hbcooooo
04;oo`04003oooooHb`=ool00`00ooooo`0cool006?oo`04003oooooHb``ool00f</ooooo`0Lool0
0f</ooooo`07ool00f</ooooo`0Dool00`00ooooo`13ool00f</ooooo`0Fool4Hb`bool00f</oooo
o`13ool01000ooooof</3?oo00<00?ooool0<ooo001Tool00`00oomS;00`ool00f</ooooo`0Mool2
Hb`7ool00f</ooooo`0Dool00`00ooooo`13ool00f</ooooo`0Cool3Hb`fool00f</ooooo`13ool0
1000ooooof</3?oo00<00?ooool0<ooo001Uool2000`ool00f</ooooo`0Oool00f</ooooo`04ool0
0f</ooooo`0Dool00`00ooooo`13ool00f</ooooo`0?ool4Hb`iool00f</ooooo`14ool00`00oomS
;00<ool00`00ooooo`0cool006Ooo`03003ooooo02goo`03Hbcooooo023oo`03Hbcooooo00?oo`03
Hbcooooo01Coo`03003ooooo04?oo`03Hbcooooo00_oo`AS;3goo`03Hbcooooo04Goo`03001S;?oo
00_oo`03003ooooo03?oo`00J?oo0P00;Ooo00=S;?ooool08Ooo00=S;?ooool00_oo00=S;?ooool0
5?oo00<00?ooool0@ooo00=S;?ooool01ooo16</@Ooo00=S;?ooool0A_oo00<00?ooool02_oo00<0
0?ooool0<ooo001Zool00`00ooooo`0Zool00f</ooooo`0Rool2Hb`2ool00f</ooooo`0Dool00`00
ooooo`13ool00f</ooooo`03ool4Hba5ool00f</ooooo`17ool00`00ooooo`09ool00`00ooooo`0c
ool006_oo`8002Woo`03Hbcooooo02Goo`03Hbcoof</01Koo`03003ooooo04?oo`03Hbcoof</00=S
;4Woo`03Hbcooooo04Ooo`03003ooooo00[oo`03003ooooo03;oo`00KOoo00<00?ooool09_oo00=S
;?ooool09_oo0V</5Ooo00<00?ooool0@_oo16</COoo00=S;?ooool0B?oo00<00?ooool02Ooo00<0
0?ooool0<_oo001^ool00`00ooooo`0Uool00f</ooooo`0Wool00f</ooooo`0Cool00`00ooooo`0n
ool4Hb`2ool00f</ooooo`1<ool00f</ooooo`19ool00`00ooooo`08ool00`00ooooo`0bool006oo
o`8002Goo`03Hbcooooo02Ooo`=S;1?oo`03003ooooo03[oo`AS;0Koo`03Hbcooooo04coo`03Hbco
oooo04[oo`03003ooooo00Ooo`03003ooooo03;oo`00LOoo00<00?ooool08_oo00=S;?ooool09ooo
00AS;?oooomS;1;oo`03003ooooo03Koo`AS;0[oo`03Hbcooooo04coo`03Hbcooooo04_oo`03003o
oooo00Koo`03003ooooo03;oo`00L_oo0P008_oo00=S;?ooool09ooo00ES;?ooooooof</017oo`03
003ooooo03;oo`AS;0koo`03Hbcooooo04coo`03Hbcooooo04_oo`03003ooooo00Koo`03003ooooo
03;oo`00M?oo00<00?ooool07ooo00=S;?ooool09ooo00=S;?ooool00_oo0V</3ooo00<00?ooool0
;_oo16</4_oo00=S;?ooool0COoo00=S;?ooool0Booo00<00?ooool01Ooo00<00?ooool0<_oo001e
ool2000Oool00f</ooooo`0Wool00f</ooooo`04ool00f</ooooo`0<ool00`00ooooo`0[ool3Hb`F
ool00f</ooooo`1=ool00f</ooooo`1<ool00`00ooooo`05ool00`00ooooo`0aool007Ooo`03003o
oooo01coo`03Hbcooooo02Ooo`03Hbcooooo00Goo`03Hbcooooo00_oo`03003ooooo02Ooo`AS;1Wo
o`03Hbcooooo04goo`03Hbcooooo04goo`03003ooooo00Coo`03003ooooo037oo`00N?oo00<00?oo
ool06ooo00=S;?ooool09ooo00=S;?ooool01_oo00=S;?ooool02_oo00<00?ooool08ooo16</7Ooo
00=S;?ooool0COoo00=S;?ooool0C_oo00<00?ooool00ooo00<00?ooool0<Ooo001iool2000Kool0
0f</ooooo`0Wool00f</ooooo`07ool2Hb`:ool00`00ooooo`0Oool4Hb`Qool00f</ooooo`1=ool0
0f</ooooo`1?ool00`00ooooo`02ool00`00ooooo`0aool007_oo`03003ooooo01Soo`03Hbcooooo
02Koo`03Hbcooooo00[oo`03Hbcooooo00Ooo`03003ooooo01_oo`AS;2Goo`03Hbcooooo04goo`03
Hbcooooo04ooo`03003ooooo00;oo`03003ooooo037oo`00O?oo0P005ooo00=S;?ooool09ooo00=S
;?ooool02ooo00=S;?ooool01_oo00<00?ooool05ooo16</:Ooo00=S;?ooool0COoo00=S;?ooool0
D?oo00D00?ooooooo`0003?oo`00O_oo00<00?ooool05?oo00=S;?ooool09ooo00=S;?ooool03?oo
00=S;?ooool01Ooo00<00?ooool04ooo16</;Ooo00=S;?ooool0COoo00=S;?ooool0DOoo00@00?oo
ool003?oo`00Oooo0P005?oo00=S;?ooool09ooo00=S;?ooool03Ooo0V</1Ooo00<00?ooool03ooo
16</<Ooo00=S;?ooool0COoo00=S;?ooool0D_oo00<00?oo0000<ooo0021ool00`00ooooo`0Aool0
0f</ooooo`0Wool00f</ooooo`0?ool00f</ooooo`02ool00`00ooooo`0;ool4Hb`eool00f</oooo
o`1=ool00f</ooooo`1Cool00`00ool0000bool008;oo`03003ooooo013oo`03Hbcooooo02Ooo`03
Hbcooooo013oo`05Hbcoooooool00009ool4Hb`iool00f</ooooo`1=ool00f</ooooo`1Cool00`00
ool0000bool008?oo`80013oo`03Hbcooooo02Ooo`03Hbcooooo017oo`9S;003ool00?oo00Coo`AS
;3goo`03Hbcooooo04goo`03Hbcooooo05Coo`8003;oo`00QOoo00<00?ooool03Ooo00=S;?ooool0
9ooo00=S;?ooool04ooo00=S;000ool016</@Ooo00=S;?ooool0COoo00=S;?ooool0Dooo0`00<_oo
0026ool2000=ool00f</ooooo`0Wool00f</ooooo`0Dool00`00Hbcoo`14ool00f</ooooo`1>ool0
0f</ooooo`1>ool4000eool008Soo`03003ooooo00[oo`03Hbcooooo02Ooo`03Hbcooooo01Coo`03
003ooooo04Coo`03Hbcooooo04koo`03Hbcooooo04[oo`@003Woo`00ROoo0P002_oo00=S;?ooool0
9ooo00=S;?ooool05?oo00<00?ooool0A?oo00=S;?ooool0C_oo00=S;?ooool0AOoo1@00?Ooo002;
ool00`00ooooo`07ool00f</ooooo`0Wool00f</ooooo`0Dool00`00ooooo`15ool00f</ooooo`1=
ool00f</ooooo`11ool40012ool008coo`8000Ooo`03Hbcooooo02Ooo`03Hbcooooo01Coo`03003o
oooo04Goo`03Hbcooooo04goo`03Hbcooooo03coo`D004Koo`00S_oo00<00?ooool00ooo00=S;?oo
ool0:?oo00=S;?ooool05?oo00<00?ooool0AOoo00=S;?ooool0COoo00=S;?ooool0>?oo1000Booo
002?ool00`00ooooo`02ool00f</ooooo`0Xool00f</ooooo`0Dool00`00ooooo`15ool00f</oooo
o`1=ool00f</ooooo`0dool4001?ool0093oo`8000;oo`03Hbcooooo02Soo`03Hbcooooo01Coo`03
003ooooo04Goo`03Hbcooooo04goo`03Hbcooooo02ooo`D005?oo`00T_oo00<00?ooHb`0:_oo00=S
;?ooool05?oo00<00?ooool0AOoo00=S;?ooool0COoo00=S;?ooool0:ooo1000F?oo002Cool2000Z
ool00f</ooooo`0Dool00`00ooooo`15ool00f</ooooo`1=ool00f</ooooo`0Vool5001Lool009Co
o`03Hb`00?oo02Ooo`03Hbcooooo01Goo`03003ooooo04Goo`03Hbcooooo04goo`03Hbcooooo02;o
o`@0067oo`00U_oo0P009_oo00=S;?ooool05?oo00<00?ooool0A_oo00=S;?ooool0COoo00=S;?oo
ool07_oo1000IOoo002Hool00`00ooooo`0Sool00f</ooooo`0Dool00`00ooooo`16ool00f</oooo
o`1=ool00f</ooooo`0Iool5001Yool009Woo`03003ooooo02;oo`03Hbcooooo01Coo`03003ooooo
04Koo`03Hbcooooo04koo`03Hbcooooo01Coo`@006koo`00V_oo0P008_oo00=S;?ooool05?oo00<0
0?ooool0A_oo00=S;?ooool0C_oo00=S;?ooool04?oo1000L_oo002Lool00`00ooooo`0Oool00f</
ooooo`0Dool00`00ooooo`16ool00f</ooooo`1>ool00f</ooooo`0;ool5001fool009goo`8001oo
o`03Hbcooooo01Coo`03003ooooo04Koo`03Hbcooooo04koo`03Hbcooooo00Ooo`@007_oo`00Wooo
00<00?ooool07?oo00=S;?ooool05?oo00<00?ooool0A_oo00=S;?ooool0C_oo00=S;?ooool00_oo
1@00Oooo002Pool2000Lool00f</ooooo`0Dool00`00ooooo`16ool00f</ooooo`1>ool00f</0000
00020024ool00:;oo`03003ooooo01Woo`03Hbcooooo01Coo`03003ooooo04Koo`03Hbcooooo04_o
o`@008Soo`00Xooo00<00?ooool06?oo00=S;?ooool05?oo00<00?ooool0A_oo00=S;?ooool0A_oo
1@00S?oo002Tool2000Hool00f</ooooo`0Dool00`00ooooo`16ool00f</ooooo`12ool4002Aool0
0:Koo`03003ooooo01Goo`03Hbcooooo01Coo`03003ooooo04Koo`03Hbcooooo03koo`@009Goo`00
Yooo0P005Ooo00=S;?ooool05?oo00<00?ooool0A_oo00=S;?ooool0>Ooo1@00VOoo002Yool00`00
ooooo`0Bool00f</ooooo`0Dool00`00ooooo`16ool00f</ooooo`0eool4002Nool00:[oo`8001;o
o`03Hbcooooo01Coo`03003ooooo04Koo`03Hbcooooo033oo`D00:;oo`00[?oo00<00?ooool03ooo
00=S;?ooool05?oo00<00?ooool0A_oo00=S;?ooool0;?oo1000Yooo002]ool2000?ool00f</oooo
o`0Dool00`00ooooo`16ool00f</ooooo`0Xool4002[ool00:ooo`03003ooooo00coo`03Hbcooooo
01Coo`03003ooooo04Koo`03Hbcooooo02?oo`D00:ooo`00/?oo00<00?ooool02_oo00=S;?ooool0
5Ooo00<00?ooool0A_oo00=S;?ooool07ooo1000]?oo002aool2000:ool00f</ooooo`0Eool00`00
ooooo`16ool00f</ooooo`0Jool5002hool00;?oo`03003ooooo00Ooo`03Hbcooooo01Goo`03003o
oooo04Koo`03Hbcooooo01Koo`@00;goo`00]?oo0P001ooo00=S;?ooool05Ooo00<00?ooool0A_oo
00=S;?ooool04_oo1000`Ooo002fool00`00ooooo`04ool00f</ooooo`0Eool00`00ooooo`16ool0
0f</ooooo`0=ool50035ool00;Ooo`8000Coo`03Hbcooooo01Goo`03003ooooo04Koo`03Hbcooooo
00Woo`@00<[oo`00^Ooo00D00?ooooooof</01Ooo`03003ooooo04Koo`03Hbcooooo00Goo`@00<ko
o`00^_oo00@00?oooomS;1Ooo`03003ooooo04Koo`03Hbcooooo00D00=;oo`00^ooo0P0000=S;?oo
ool05Ooo00<00?ooool0AOoo1000eooo002mool00`00ooooo`0Eool00`00ooooo`10ool5003Kool0
0;koo`8001Goo`03003ooooo03coo`@00>3oo`00`?oo00<00?ooool04Ooo00<00?ooool0>Ooo1000
i?oo0031ool2000Aool00`00ooooo`0dool5003Xool00<?oo`03003ooooo00koo`03003ooooo033o
o`@00>goo`00a?oo00<00?ooool03Ooo00<00?ooool0;?oo1000lOoo0035ool2000=ool00`00oooo
o`0Wool5003eool00<Ooo`03003ooooo00[oo`03003ooooo02?oo`@00?[oo`00b?oo0P002_oo00<0
0?ooool07_oo1@00o_oo003:ool00`00ooooo`07ool00`00ooooo`0Jool4003oool4ool00<_oo`80
00Ooo`03003ooooo01Koo`@00?ooo`Soo`00cOoo00<00?ooool01?oo00<00?ooool04Ooo1@00oooo
3?oo003>ool00`00ooooo`03ool00`00ooooo`0=ool4003ooolAool00<ooo`8000?oo`03003ooooo
00Soo`D00?oooaGoo`00dOoo00@00?ooool000Koo`@00?oooa[oo`00d_oo0`000_oo1000oooo7_oo
003Dool3003ooolRool00?ooooWoo`00oooonOoo003ooooiool00?ooooWoo`00oooonOoo003ooooi
ool00?ooo`goo`@00>Soo`00N?oo00<00?ooool0NOoo00<00?ooool05?oo0P00F_oo00<00?ooool0
S_oo001hool00`00ooooo`1iool00`00ooooo`0Cool2001Kool00`00ooooo`2>ool000?oo`T002;o
o`04003oooooool20002ool60002ool40004ool6000Aool50005ool30008ool00`00ooooo`02ool2
000Eool00`00ooooo`02ool00`00ooooo`0Jool60002ool20002ool40004ool4000Aool50005ool3
0008ool00`00ooooo`02ool2000Cool20002ool00`00ooooo`0Fool40004ool20004ool3000>ool5
0005ool30008ool00`00ooooo`0:ool2000Dool00`00ooooo`03ool30004ool300001?oo00000?oo
0`006?oo0`000ooo0P000_oo10001Ooo1@001?oo0`004_oo1@001Ooo0`002?oo00<00?ooool01?oo
0005ool00`00ooooo`03ool2000;ool6000@ool00`00ooooo`02ool01@00oooooooo00001?oo00<0
0?ooool00ooo00<00?ooool00_oo00<00?ooool00_oo00<00?ooool03?oo0P001Ooo00@00?oooooo
o`8000;oo`8000Goo`8000Coo`8001Coo`8000Coo`03003ooooo00Soo`H000goo`03003ooooo00;o
o`04003ooooo0006ool00`00ooooo`05ool00`00ooooo`0<ool20005ool01000oooooooo0P000_oo
0P001Ooo0P001?oo0P004_oo0P001?oo0P003Ooo1P001ooo00<00?ooool00_oo00D00?ooooooo`00
00?oo`03003ooooo00Woo`8000Goo`04003oooooool20002ool20005ool2000<ool2000Cool20004
ool20002ool200000ooo003oo`02ool20002ool00`00ooooo`0=ool60004ool01@00oooooooo0000
0ooo00<00?ooool01?oo00<00?ooool01?oo00<00?ooool00ooo00D00?ooooooo`0000ooo`8000Go
o`04003oooooool20002ool20005ool20006ool000Goo`03003ooooo00Coo`03003ooooo01ooo`03
003ooooo00Ooo`03003ooooo00Soo`03003ooooo00;oo`03003ooooo013oo`03003ooooo00Coo`05
003oooooool00004ool00`00ooooo`03ool2000Jool20004ool2000Mool00`00ooooo`0<ool00`00
ooooo`05ool00`00ooooo`0;ool00`00ooooo`04ool01@00oooooooo00001?oo00<00?ooool00ooo
0P006?oo00<00?ooool01?oo00<00?ooool06Ooo00<00?ooool01Ooo00@00?ooooooo`8000[oo`03
003ooooo00Coo`05003oooooool00004ool00`00ooooo`03ool2000Qool00`00ool00003ool00`00
ooooo`02ool00`00ool00004ool01000ooooo`006Ooo00@00?ooooooo`8000[oo`03003ooooo00?o
o`03003ooooo00?oo`04003oooooool2000>ool00`00ooooo`04ool01@00oooooooo00001?oo00<0
0?ooool00ooo0P001_oo0005ool00`00ooooo`0Qool80007ool00`00ooooo`08ool00`00ooooo`02
ool00`00ooooo`0?ool00`00ooooo`05ool00`00ooooo`02000:ool00`00ool0000Iool30002ool3
000Mool00`00ooooo`0<ool00`00ooooo`05ool00`00ooooo`0:ool00`00ooooo`05ool00`00oooo
o`02000:ool00`00ool0000Fool00`00ooooo`06ool00`00ooooo`0Hool00`00ooooo`04ool00`00
ooooo`03ool00`00ooooo`07ool00`00ooooo`05ool00`00ooooo`02000:ool00`00ool0000Pool0
0`00ool00003ool00`00ooooo`04ool00`00ooooo`02ool01000ooooo`006?oo00<00?ooool00ooo
00<00?ooool02?oo00<00?ooool00ooo00<00?ooool00_oo00<00?ooool00ooo00<00?ooool02ooo
00<00?ooool01Ooo00<00?ooool00P002_oo00<00?oo00001Ooo0005ool00`00ooooo`03ool00`00
ooooo`0:ool6000<ool01@00oooooooo00002_oo00<00?ooool01_oo0P001Ooo00<00?ooool03_oo
00<00?ooool01Ooo00<00?ooool01`001?oo0P0000?oo`00ool05ooo0P0000Goo`00ooooo`0000;o
o`03003ooooo00Koo`H000ooo`03003ooooo00_oo`03003ooooo00Goo`03003ooooo00[oo`03003o
oooo00Goo`03003ooooo00L000Coo`800003ool00?oo01Goo`03003ooooo00Koo`03003ooooo00[o
o`H000Ooo`8000Koo`03003ooooo00?oo`03003ooooo00Ooo`03003ooooo00Goo`03003ooooo00L0
00Coo`800003ool00?oo01koo`8000;oo`04003ooooo0006ool00`00ooooo`02ool01000ooooo`00
3ooo1P000ooo00<00?ooool00ooo00<00?ooool01ooo0P001Ooo00<00?ooool00_oo00<00?ooool0
0ooo00<00?ooool02ooo00<00?ooool01Ooo00<00?ooool01`001?oo0P0000?oo`00ool01?oo0005
ool6000Oool01@00oooooooo00002ooo00<00?ooool00ooo0`001ooo00<00?ooool03Ooo00<00?oo
ool01?oo0`000_oo00D00?ooooooo`0000Goo`05003oooooool0000Gool20002ool30002ool00`00
ooooo`0Lool00`00ooooo`09ool20006ool2000<ool00`00ooooo`04ool30002ool01@00oooooooo
00001Ooo00D00?ooooooo`0001Goo`03003ooooo00Koo`03003ooooo01Goo`<000Ooo`03003ooooo
00?oo`03003ooooo00Ooo`03003ooooo00Coo`<000;oo`05003oooooool00005ool01@00oooooooo
00007_oo00D00?ooooooo`0000;oo`03003ooooo00;oo`03003oo`0000Coo`04003ooooo000Hool0
0`00ooooo`03ool00`00ooooo`05ool30006ool00`00ooooo`02ool00`00ooooo`03ool00`00oooo
o`0;ool00`00ooooo`04ool30002ool01@00oooooooo00001Ooo00D00?ooooooo`0000Coo`001Ooo
00<00?ooool00_oo00<00?ooool07_oo00@00?ooool000coo`03003ooooo00?oo`03003ooooo00Oo
o`03003ooooo00coo`03003ooooo00Woo`D000Coo`8000?oo`03003ooooo01Goo`04003oooooool3
0002ool00`00ooooo`0Mool00`00ooooo`06ool30005ool3000=ool00`00ooooo`09ool50004ool2
0003ool00`00ooooo`0Cool00`00ooooo`06ool01000oooooooo10004Ooo00<00?ooool01_oo00<0
0?ooool00ooo00<00?ooool01ooo00<00?ooool02Ooo1@001?oo0P000ooo00<00?ooool010005_oo
10000_oo0P000_oo100000?oo`0000000_oo0`0000?oo`0000000P005_oo00<00?ooool00ooo00<0
0?ooool01_oo00<00?ooool01Ooo00<00?ooool00_oo00<00?ooool00ooo00<00?ooool02ooo00<0
0?ooool02Ooo1@001?oo0P000ooo00<00?ooool00_oo0005ool00`00ooooo`02ool00`00ooooo`0O
ool00`00ool0000<ool00`00ooooo`04ool00`00ooooo`06ool00`00ooooo`0<ool00`00ooooo`0B
ool00`00ooooo`03ool00`00ooooo`0Cool20003ool20004ool00`00ooooo`0Lool00`00ooooo`04
ool40004ool4000>ool00`00ooooo`0Bool00`00ooooo`03ool00`00ooooo`0Bool00`00ooooo`06
ool00`00ooooo`02ool01000ooooo`004Ooo00<00?ooool01Ooo00<00?ooool00ooo00<00?ooool0
1ooo00<00?ooool04_oo00<00?ooool00ooo00@00?ooool000;oo`03003ooooo02_oo`03003ooooo
01Koo`03003ooooo00?oo`03003ooooo00Ooo`03003ooooo00Coo`03003ooooo00;oo`03003ooooo
00?oo`03003ooooo00_oo`03003ooooo01;oo`03003ooooo00?oo`03003ooooo007oo`001Ooo00<0
0?ooool09?oo00<00?oo00001ooo00<00?ooool00_oo00@00?ooool000?oo`05003oooooool00004
ool00`00ooooo`0=ool00`00ooooo`04ool00`00ooooo`09ool20005ool00`00ooooo`0Cool20003
ool20004ool00`00ooooo`0Gool00`00ooooo`02ool00`00ooooo`05ool00`00ooooo`05ool00`00
ooooo`0?ool00`00ooooo`04ool00`00ooooo`09ool20005ool00`00ooooo`0Cool00`00ooooo`04
ool00`00ooooo`03ool00`00ooooo`0>ool01@00oooooooo00001ooo00<00?ooool00_oo00<00?oo
ool02Ooo00<00?ooool01?oo00<00?ooool02Ooo0P001Ooo00@00?ooool0037oo`03003ooooo01Go
o`03003ooooo00;oo`03003ooooo00Coo`05003oooooool00006ool00`00ooooo`02ool00`00oooo
o`02ool00`00ooooo`0=ool00`00ooooo`04ool00`00ooooo`09ool20005ool00`00ooooo`01ool0
00Goo`03003ooooo00?oo`03003ooooo01ooo`8000Ooo`04003oooooool20002ool01@00oooooooo
00000ooo00@00?ooooooo`8000ooo`8000Goo`03003ooooo00Woo`8000Goo`03003ooooo01;oo`80
00Coo`8000Coo`03003ooooo01Ooo`04003oooooool20007ool00`00ooooo`05ool00`00ooooo`0?
ool20005ool00`00ooooo`09ool20005ool00`00ooooo`0Cool20004ool20006ool00`00ooooo`0=
ool01@00oooooooo00002?oo00D00?ooooooo`0000_oo`8000Goo`03003ooooo00Woo`8000Goo`05
003oooooool0000`ool00`00ooooo`0Fool01@00oooooooo00001_oo00D00?ooooooo`0000Coo`03
003oo`0000Goo`05003oooooool0000?ool20005ool00`00ooooo`09ool20005ool00`00ooooo`01
ool000?oo`T002;oo`03003ooooo00Koo`@000Coo`@000Coo`@001;oo`H000Woo`D000?oo`@0017o
o`@00003ool0000000<000;oo`<001Woo`@000Woo`@000Coo`@000ooo`H000Woo`D000?oo`@001Go
o`@000Woo`03003ooooo00goo`@000Woo`<000koo`H000Woo`D000?oo`@000;oo`03003ooooo04Oo
o`<000Soo`@000Goo`8000Koo`<001;oo`H000Woo`D000?oo`@0007oo`00oooo4ooo00D00?oooooo
o`0004Koo`05003oooooool0002Fool00?oooa?oo`03003ooooo008004Koo`03003ooooo008009Ko
o`00oooo5?oo0`00B?oo0`00Uooo003ooooiool00?ooooWoo`00oooonOoo003ooooiool00?ooooWo
o`00oooonOoo003ooooiool00?ooooWoo`00oooonOoo003ooooiool00?ooooWoo`00\
\>"],
  ImageRangeCache->{{{0, 503}, {503, 0}} -> {-0.0594185, -0.0663828, \
0.00212799, 0.00212799}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics3D  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Export["\<RCsurf3b.gif\>", p1]\)], "Input"],

Cell[BoxData[
    \("RCsurf3b.gif"\)], "Output"]
}, Open  ]],

Cell["\<\
Now get the same surface in radians instead of degrees and do the \
integrals to get the average value of the RC over the surface.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[{
    \(\(RC2\  = \ 
        Table[\ Table[{data1[\([i + j, 4]\)], 
              data1[\([i + j, 5]\)]*\[Pi]\/180, 
              data1[\([i + j, 
                  6]\)]}, \[IndentingNewLine]\ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \
\ \ \ \ \ {i, 1, 13}], \ {j, 0, length - nphis, 
            nphis}];\)\), "\[IndentingNewLine]", 
    \(\(RC3\  = \ Flatten[RC2, 1];\)\), "\n", 
    \(\(RC4\  = \ Interpolation[RC3];\)\), "\n", 
    \(numerator1\  = \ 
      2*Integrate[
          RC4[x, y], {x, 0.85, 1}, {y, 0, \[Pi]}]\), "\[IndentingNewLine]", 
    \(denominator1\  = \ 
      2*Integrate[1, {x, 0.85, 1}, {y, 0, \[Pi]}]\), "\[IndentingNewLine]", 
    \(numerator1\/denominator1\)}], "Input"],

Cell[BoxData[
    \(0.8827437578923533`\)], "Output"],

Cell[BoxData[
    \(0.9424777960769379`\)], "Output"],

Cell[BoxData[
    \(0.9366202170138888`\)], "Output"]
}, Open  ]],

Cell["\<\
Here is a plot of of the phi dependence at a fixed value of \
cos(theta_pq).\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[{
    \(\(RC5\  = \ 
        Table[{N[phi]*180\/\[Pi], RC4[0.9999, phi]}, {phi, 
            0, \[Pi], \[Pi]\/20}];\)\), "\n", 
    \(ListPlot[
      RC5, \[IndentingNewLine]Frame \[Rule] 
        True, \[IndentingNewLine]FrameLabel \[Rule] {"\<\[Phi] (rad)\>", \
"\<RC\>", "\<\>", "\<\>"}, \[IndentingNewLine]PlotRange -> {{0, 180}, {0, 
            1.8}}, \[IndentingNewLine]PlotJoined \[Rule] 
        True, \[IndentingNewLine]PlotStyle -> {RGBColor[1, 0, 0], 
          Thickness[0.005]}]\)}], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Times
%%IncludeFont: Times
/Times findfont 16  scalefont  setfont
% Scaling calculations
-1.9082e-17 0.00555556 9.02598e-18 0.343352 [
[.13889 -0.0125 -11.5625 -14.5625 ]
[.13889 -0.0125 11.5625 0 ]
[.27778 -0.0125 -11.5625 -14.5625 ]
[.27778 -0.0125 11.5625 0 ]
[.41667 -0.0125 -11.5625 -14.5625 ]
[.41667 -0.0125 11.5625 0 ]
[.55556 -0.0125 -16.3438 -14.5625 ]
[.55556 -0.0125 16.3438 0 ]
[.69444 -0.0125 -16.3438 -14.5625 ]
[.69444 -0.0125 16.3438 0 ]
[.83333 -0.0125 -16.3438 -14.5625 ]
[.83333 -0.0125 16.3438 0 ]
[.97222 -0.0125 -16.3438 -14.5625 ]
[.97222 -0.0125 16.3438 0 ]
[ 0 0 -0.125 0 ]
[.5 -0.0125 -35.4688 -29.5625 ]
[.5 -0.0125 35.4688 -14.5625 ]
[-0.0125 .08584 -42.25 -7.28125 ]
[-0.0125 .08584 0 7.28125 ]
[-0.0125 .17168 -32.6875 -7.28125 ]
[-0.0125 .17168 0 7.28125 ]
[-0.0125 .25751 -42.25 -7.28125 ]
[-0.0125 .25751 0 7.28125 ]
[-0.0125 .34335 -13.5625 -7.28125 ]
[-0.0125 .34335 0 7.28125 ]
[-0.0125 .42919 -42.25 -7.28125 ]
[-0.0125 .42919 0 7.28125 ]
[-0.0125 .51503 -32.6875 -7.28125 ]
[-0.0125 .51503 0 7.28125 ]
[-0.0125 .60087 -42.25 -7.28125 ]
[-0.0125 .60087 0 7.28125 ]
[ 0 0 -0.125 0 ]
[-0.01875 .30902 -56.8125 -11.5625 ]
[-0.01875 .30902 -42.25 11.5625 ]
[ 0 .61803 .125 0 ]
[ 1 0 .125 0 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.13889 0 m
.13889 .00625 L
s
gsave
.13889 -0.0125 -72.5625 -18.5625 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 22.5625 translate 1 -1 scale
63.000 15.062 moveto
%%IncludeResource: font Times
%%IncludeFont: Times
/Times findfont 16.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(25) show
1.000 setlinewidth
grestore
.27778 0 m
.27778 .00625 L
s
gsave
.27778 -0.0125 -72.5625 -18.5625 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 22.5625 translate 1 -1 scale
63.000 15.062 moveto
%%IncludeResource: font Times
%%IncludeFont: Times
/Times findfont 16.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(50) show
1.000 setlinewidth
grestore
.41667 0 m
.41667 .00625 L
s
gsave
.41667 -0.0125 -72.5625 -18.5625 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 22.5625 translate 1 -1 scale
63.000 15.062 moveto
%%IncludeResource: font Times
%%IncludeFont: Times
/Times findfont 16.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(75) show
1.000 setlinewidth
grestore
.55556 0 m
.55556 .00625 L
s
gsave
.55556 -0.0125 -77.3438 -18.5625 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 22.5625 translate 1 -1 scale
63.000 15.062 moveto
%%IncludeResource: font Times
%%IncludeFont: Times
/Times findfont 16.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(100) show
1.000 setlinewidth
grestore
.69444 0 m
.69444 .00625 L
s
gsave
.69444 -0.0125 -77.3438 -18.5625 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 22.5625 translate 1 -1 scale
63.000 15.062 moveto
%%IncludeResource: font Times
%%IncludeFont: Times
/Times findfont 16.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(125) show
1.000 setlinewidth
grestore
.83333 0 m
.83333 .00625 L
s
gsave
.83333 -0.0125 -77.3438 -18.5625 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 22.5625 translate 1 -1 scale
63.000 15.062 moveto
%%IncludeResource: font Times
%%IncludeFont: Times
/Times findfont 16.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(150) show
1.000 setlinewidth
grestore
.97222 0 m
.97222 .00625 L
s
gsave
.97222 -0.0125 -77.3438 -18.5625 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 22.5625 translate 1 -1 scale
63.000 15.062 moveto
%%IncludeResource: font Times
%%IncludeFont: Times
/Times findfont 16.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(175) show
1.000 setlinewidth
grestore
.125 Mabswid
.02778 0 m
.02778 .00375 L
s
.05556 0 m
.05556 .00375 L
s
.08333 0 m
.08333 .00375 L
s
.11111 0 m
.11111 .00375 L
s
.16667 0 m
.16667 .00375 L
s
.19444 0 m
.19444 .00375 L
s
.22222 0 m
.22222 .00375 L
s
.25 0 m
.25 .00375 L
s
.30556 0 m
.30556 .00375 L
s
.33333 0 m
.33333 .00375 L
s
.36111 0 m
.36111 .00375 L
s
.38889 0 m
.38889 .00375 L
s
.44444 0 m
.44444 .00375 L
s
.47222 0 m
.47222 .00375 L
s
.5 0 m
.5 .00375 L
s
.52778 0 m
.52778 .00375 L
s
.58333 0 m
.58333 .00375 L
s
.61111 0 m
.61111 .00375 L
s
.63889 0 m
.63889 .00375 L
s
.66667 0 m
.66667 .00375 L
s
.72222 0 m
.72222 .00375 L
s
.75 0 m
.75 .00375 L
s
.77778 0 m
.77778 .00375 L
s
.80556 0 m
.80556 .00375 L
s
.86111 0 m
.86111 .00375 L
s
.88889 0 m
.88889 .00375 L
s
.91667 0 m
.91667 .00375 L
s
.94444 0 m
.94444 .00375 L
s
.25 Mabswid
0 0 m
1 0 L
s
gsave
.5 -0.0125 -96.4688 -33.5625 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding ISOLatin1Encoding def
        currentdict
    end

    newfontname exch definefont pop
} def
63.000 15.062 moveto
%%IncludeResource: font Times
%%IncludeFont: Times
%%BeginResource: font Times-MISO
%%BeginFont: Times-MISO
/Times /Times-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Times-MISO
%%IncludeFont: Times-MISO
/Times-MISO findfont 16.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 15.062 moveto
%%IncludeResource: font Math1Mono
%%IncludeFont: Math1Mono
/Math1Mono findfont 16.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(f) show
%%IncludeResource: font Math2Mono
%%IncludeFont: Math2Mono
/Math2Mono findfont 16.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
82.125 15.062 moveto
(H) show
91.688 15.062 moveto
%%IncludeResource: font Times-MISO
%%IncludeFont: Times-MISO
/Times-MISO findfont 16.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(rad) show
%%IncludeResource: font Math2Mono
%%IncludeFont: Math2Mono
/Math2Mono findfont 16.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
120.375 15.062 moveto
(L) show
129.938 15.062 moveto
%%IncludeResource: font Times-MISO
%%IncludeFont: Times-MISO
/Times-MISO findfont 16.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
0 .08584 m
.00625 .08584 L
s
gsave
-0.0125 .08584 -103.25 -11.2812 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 22.5625 translate 1 -1 scale
63.000 15.062 moveto
%%IncludeResource: font Times
%%IncludeFont: Times
/Times findfont 16.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(0.25) show
1.000 setlinewidth
grestore
0 .17168 m
.00625 .17168 L
s
gsave
-0.0125 .17168 -93.6875 -11.2812 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 22.5625 translate 1 -1 scale
63.000 15.062 moveto
%%IncludeResource: font Times
%%IncludeFont: Times
/Times findfont 16.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(0.5) show
1.000 setlinewidth
grestore
0 .25751 m
.00625 .25751 L
s
gsave
-0.0125 .25751 -103.25 -11.2812 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 22.5625 translate 1 -1 scale
63.000 15.062 moveto
%%IncludeResource: font Times
%%IncludeFont: Times
/Times findfont 16.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(0.75) show
1.000 setlinewidth
grestore
0 .34335 m
.00625 .34335 L
s
gsave
-0.0125 .34335 -74.5625 -11.2812 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 22.5625 translate 1 -1 scale
63.000 15.062 moveto
%%IncludeResource: font Times
%%IncludeFont: Times
/Times findfont 16.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(1) show
1.000 setlinewidth
grestore
0 .42919 m
.00625 .42919 L
s
gsave
-0.0125 .42919 -103.25 -11.2812 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 22.5625 translate 1 -1 scale
63.000 15.062 moveto
%%IncludeResource: font Times
%%IncludeFont: Times
/Times findfont 16.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(1.25) show
1.000 setlinewidth
grestore
0 .51503 m
.00625 .51503 L
s
gsave
-0.0125 .51503 -93.6875 -11.2812 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 22.5625 translate 1 -1 scale
63.000 15.062 moveto
%%IncludeResource: font Times
%%IncludeFont: Times
/Times findfont 16.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(1.5) show
1.000 setlinewidth
grestore
0 .60087 m
.00625 .60087 L
s
gsave
-0.0125 .60087 -103.25 -11.2812 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 22.5625 translate 1 -1 scale
63.000 15.062 moveto
%%IncludeResource: font Times
%%IncludeFont: Times
/Times findfont 16.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(1.75) show
1.000 setlinewidth
grestore
.125 Mabswid
0 .01717 m
.00375 .01717 L
s
0 .03434 m
.00375 .03434 L
s
0 .0515 m
.00375 .0515 L
s
0 .06867 m
.00375 .06867 L
s
0 .10301 m
.00375 .10301 L
s
0 .12017 m
.00375 .12017 L
s
0 .13734 m
.00375 .13734 L
s
0 .15451 m
.00375 .15451 L
s
0 .18884 m
.00375 .18884 L
s
0 .20601 m
.00375 .20601 L
s
0 .22318 m
.00375 .22318 L
s
0 .24035 m
.00375 .24035 L
s
0 .27468 m
.00375 .27468 L
s
0 .29185 m
.00375 .29185 L
s
0 .30902 m
.00375 .30902 L
s
0 .32618 m
.00375 .32618 L
s
0 .36052 m
.00375 .36052 L
s
0 .37769 m
.00375 .37769 L
s
0 .39486 m
.00375 .39486 L
s
0 .41202 m
.00375 .41202 L
s
0 .44636 m
.00375 .44636 L
s
0 .46353 m
.00375 .46353 L
s
0 .48069 m
.00375 .48069 L
s
0 .49786 m
.00375 .49786 L
s
0 .5322 m
.00375 .5322 L
s
0 .54936 m
.00375 .54936 L
s
0 .56653 m
.00375 .56653 L
s
0 .5837 m
.00375 .5837 L
s
0 .61803 m
.00375 .61803 L
s
.25 Mabswid
0 0 m
0 .61803 L
s
gsave
-0.01875 .30902 -122.094 -11.2812 Mabsadd m
1 1 Mabs scale
currentpoint translate
72.5625 11.2812 translate
90 rotate
-72.5625 -11.2812 translate
0 22.5625 translate 1 -1 scale
63.000 15.062 moveto
%%IncludeResource: font Times
%%IncludeFont: Times
/Times findfont 16.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 15.062 moveto
%%IncludeResource: font Times
%%IncludeFont: Times
/Times findfont 16.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(RC) show
82.125 15.062 moveto
%%IncludeResource: font Times
%%IncludeFont: Times
/Times findfont 16.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
0 .61178 m
0 .61803 L
s
.13889 .61178 m
.13889 .61803 L
s
.27778 .61178 m
.27778 .61803 L
s
.41667 .61178 m
.41667 .61803 L
s
.55556 .61178 m
.55556 .61803 L
s
.69444 .61178 m
.69444 .61803 L
s
.83333 .61178 m
.83333 .61803 L
s
.97222 .61178 m
.97222 .61803 L
s
.125 Mabswid
.02778 .61428 m
.02778 .61803 L
s
.05556 .61428 m
.05556 .61803 L
s
.08333 .61428 m
.08333 .61803 L
s
.11111 .61428 m
.11111 .61803 L
s
.16667 .61428 m
.16667 .61803 L
s
.19444 .61428 m
.19444 .61803 L
s
.22222 .61428 m
.22222 .61803 L
s
.25 .61428 m
.25 .61803 L
s
.30556 .61428 m
.30556 .61803 L
s
.33333 .61428 m
.33333 .61803 L
s
.36111 .61428 m
.36111 .61803 L
s
.38889 .61428 m
.38889 .61803 L
s
.44444 .61428 m
.44444 .61803 L
s
.47222 .61428 m
.47222 .61803 L
s
.5 .61428 m
.5 .61803 L
s
.52778 .61428 m
.52778 .61803 L
s
.58333 .61428 m
.58333 .61803 L
s
.61111 .61428 m
.61111 .61803 L
s
.63889 .61428 m
.63889 .61803 L
s
.66667 .61428 m
.66667 .61803 L
s
.72222 .61428 m
.72222 .61803 L
s
.75 .61428 m
.75 .61803 L
s
.77778 .61428 m
.77778 .61803 L
s
.80556 .61428 m
.80556 .61803 L
s
.86111 .61428 m
.86111 .61803 L
s
.88889 .61428 m
.88889 .61803 L
s
.91667 .61428 m
.91667 .61803 L
s
.94444 .61428 m
.94444 .61803 L
s
.25 Mabswid
0 .61803 m
1 .61803 L
s
.99375 0 m
1 0 L
s
.99375 .08584 m
1 .08584 L
s
.99375 .17168 m
1 .17168 L
s
.99375 .25751 m
1 .25751 L
s
.99375 .34335 m
1 .34335 L
s
.99375 .42919 m
1 .42919 L
s
.99375 .51503 m
1 .51503 L
s
.99375 .60087 m
1 .60087 L
s
.125 Mabswid
.99625 .01717 m
1 .01717 L
s
.99625 .03434 m
1 .03434 L
s
.99625 .0515 m
1 .0515 L
s
.99625 .06867 m
1 .06867 L
s
.99625 .10301 m
1 .10301 L
s
.99625 .12017 m
1 .12017 L
s
.99625 .13734 m
1 .13734 L
s
.99625 .15451 m
1 .15451 L
s
.99625 .18884 m
1 .18884 L
s
.99625 .20601 m
1 .20601 L
s
.99625 .22318 m
1 .22318 L
s
.99625 .24035 m
1 .24035 L
s
.99625 .27468 m
1 .27468 L
s
.99625 .29185 m
1 .29185 L
s
.99625 .30902 m
1 .30902 L
s
.99625 .32618 m
1 .32618 L
s
.99625 .36052 m
1 .36052 L
s
.99625 .37769 m
1 .37769 L
s
.99625 .39486 m
1 .39486 L
s
.99625 .41202 m
1 .41202 L
s
.99625 .44636 m
1 .44636 L
s
.99625 .46353 m
1 .46353 L
s
.99625 .48069 m
1 .48069 L
s
.99625 .49786 m
1 .49786 L
s
.99625 .5322 m
1 .5322 L
s
.99625 .54936 m
1 .54936 L
s
.99625 .56653 m
1 .56653 L
s
.99625 .5837 m
1 .5837 L
s
.25 Mabswid
1 0 m
1 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
1 0 0 r
.005 w
0 .26072 m
.05 .26072 L
.1 .26073 L
.15 .26076 L
.2 .26079 L
.25 .26083 L
.3 .26088 L
.35 .26095 L
.4 .26103 L
.45 .26113 L
.5 .26125 L
.55 .26141 L
.6 .2616 L
.65 .26184 L
.7 .26215 L
.75 .26255 L
.8 .26305 L
.85 .26366 L
.9 .26432 L
.95 .26484 L
1 .26509 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{519.688, 321.188},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`4000870001@B000`400?l00000o`00003oo`3ooooo
0?ooo`T0oooo003o0?ooool0oooo2@3oool00?l0ooooo`3oool90?ooo`00o`3ooooo0?ooo`T0oooo
003o0?ooool0oooo2@3oool00?l0ooooo`3oool90?ooo`00o`3ooolJ0?ooo`030000003oool0oooo
0200oooo00<000000?ooo`3oool0b03oool00?l0oooo103oool00`000000oooo0?ooo`0B0?ooo`03
0000003oool0oooo0280oooo00<000000?ooo`3oool0a`3oool00?l0oooo103oool00`000000oooo
0?ooo`0B0?ooo`030000003oool0oooo0280oooo00<000000?ooo`3oool0a`3oool00?l0oooo103o
ool400000100oooo00<000000?ooo`3oool00`3oool3000000D0oooo0P0000000`3oool000000000
00020?ooo`80000000<0oooo0000003oool03@3oool00`000000oooo0?ooo`360?ooo`00o`3oool3
0?ooo`030000003oool000000080oooo00<000000?ooo`3oool03@3oool00`000000oooo0?ooo`04
0?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool00P0000000`3oool0000000000002
0?ooo`<000003@3oool00`000000oooo0?ooo`360?ooo`00o`3oool20?ooo`040000003oool0oooo
000000<0oooo00<000000?ooo`3oool0303oool00`000000oooo0?ooo`040?ooo`030000003oool0
oooo00<0oooo00L000000?ooo`3oool0oooo0000003oool0000000@0oooo00<000000?ooo`3oool0
303oool00`000000oooo0?ooo`360?ooo`00o`3oool20?ooo`040000003oool0oooo000000<0oooo
00<000000?ooo`3oool0303oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo00<0oooo
0P0000020?ooo`030000003oool0000000@0oooo00<000000?ooo`3oool0303oool00`000000oooo
0?ooo`360?ooo`00o`3oool20?ooo`050000003oool0oooo0?ooo`0000000P3oool00`000000oooo
0?ooo`0<0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool01@3oool3000000030?oo
o`000000oooo00<0oooo00<000000?ooo`3oool0303oool00`000000oooo0?ooo`360?ooo`00o`3o
ool30?ooo`060000003oool0oooo0000003oool000003`3oool00`000000oooo0?ooo`040?ooo`03
0000003oool0oooo00<0oooo00L000000?ooo`3oool0oooo0000003oool0000000@0oooo00<00000
0?ooo`3oool0303oool00`000000oooo0?ooo`360?ooo`00o`3oool40?ooo`@00000403oool00`00
0000oooo0?ooo`030?ooo`<0000000D0oooo0000003oool0oooo000000030?ooo`040000003oool0
oooo00000080oooo0P00000>0?ooo`030000003oool0oooo0<H0oooo003o0?ooo`H0oooo00<00000
0?ooo`3oool0403oool00`000000oooo0?ooo`030?ooo`040000003oool00000000000<0oooo0`00
00040?ooo`80000000<0oooo0000003oool0303oool00`000000oooo0?ooo`370?ooo`00o`3oool7
0?ooo`030000003oool0oooo00l0oooo00<000000?ooo`3oool0503oool00`000000oooo0?ooo`0;
0?ooo`030000003oool0oooo0<L0oooo003o0?ooo`L0oooo00<000000?ooo`3oool0403oool00`00
0000oooo0?ooo`0B0?ooo`800000303oool00`000000oooo0?ooo`380?ooo`00o`3oool`0?ooo`03
0000003oool0oooo0=D0oooo003o0?ooool0oooo2@3oool00?l0ooooo`3oool90?ooo`00o`3ooooo
0?ooo`T0oooo001f0?ooo`H000000`3oool3000003<0oooo0`0000060?ooo`<00000=03oool00`00
0000oooo0?ooo`040?ooo`<00000;`3oool3000000D0oooo0`0000050?ooo`<00000;03oool30000
00<0oooo1P0000030?ooo`<00000;03oool3000000@0oooo0`0000060?ooo`<00000:`3oool30000
00D0oooo00<000000?ooo`3oool0103oool3000000`0oooo001g0?ooo`030000003oool0oooo0080
oooo00<000000?ooo`0000000`3oool200000300oooo00@000000?ooo`3oool0oooo0P0000030?oo
o`050000003oool0oooo0?ooo`000000<`3oool00`000000oooo0?ooo`030?ooo`040000003oool0
oooo0?ooo`800000;P3oool00`000000oooo0?ooo`030?ooo`050000003oool0oooo0?ooo`000000
0`3oool01@000000oooo0?ooo`3oool0000002`0oooo00<000000?ooo`3oool00`3oool00`000000
oooo0?ooo`020?ooo`030000003oool0000000<0oooo0P00000[0?ooo`030000003oool0oooo0080
oooo00@000000?ooo`3oool0oooo0P0000030?ooo`050000003oool0oooo0?ooo`000000:`3oool0
0`000000oooo0?ooo`040?ooo`030000003oool0oooo00<0oooo00@000000?ooo`3oool0oooo0P00
000:0?ooo`00N03oool00`000000oooo0?ooo`090?ooo`030000003oool0oooo03<0oooo00@00000
0?ooo`3oool000000`3oool00`000000oooo0?ooo`0b0?ooo`030000003oool0oooo00P0oooo00<0
00000?ooo`3oool0:`3oool00`000000oooo0?ooo`030?ooo`050000003oool0oooo0?ooo`000000
0`3oool01@000000oooo0?ooo`3oool0000002`0oooo00<000000?ooo`3oool0103oool00`000000
oooo0?ooo`090?ooo`030000003oool0oooo02P0oooo00<000000?ooo`3oool0203oool010000000
oooo0?ooo`0000030?ooo`030000003oool0oooo02T0oooo00<000000?ooo`3oool01@3oool00`00
0000oooo0?ooo`080?ooo`030000003oool0oooo00L0oooo001i0?ooo`030000003oool0oooo00P0
oooo00<000000?ooo`3oool0<`3oool00`000000oooo000000050?ooo`030000003oool0oooo0340
oooo00<000000?ooo`3oool0203oool00`000000oooo0?ooo`0[0?ooo`030000003oool0oooo0080
oooo00<000000?ooo`3oool00`3oool00`000000oooo000000050?ooo`030000003oool0oooo02T0
oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`080?ooo`030000003oool0oooo02P0
oooo00<000000?ooo`3oool0203oool00`000000oooo000000050?ooo`030000003oool0oooo02P0
oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`080?ooo`030000003oool0oooo00L0
oooo001j0?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool0<`3oool00`000000oooo
000000050?ooo`030000003oool0oooo0340oooo00<000000?ooo`3oool0203oool00`000000oooo
0?ooo`0[0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool00`3oool00`000000oooo
000000050?ooo`030000003oool0oooo02T0oooo00<000000?ooo`3oool01P3oool00`000000oooo
0?ooo`070?ooo`030000003oool0oooo02P0oooo00<000000?ooo`3oool0203oool00`000000oooo
000000050?ooo`030000003oool0oooo02P0oooo00<000000?ooo`3oool01@3oool00`000000oooo
0?ooo`080?ooo`030000003oool0oooo00L0oooo001k0?ooo`030000003oool0oooo00D0oooo00<0
00000?ooo`3oool0<`3oool010000000oooo0?ooo`0000050?ooo`030000003oool0oooo0380oooo
00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`0/0?ooo`030000003oool0oooo0080oooo
00<000000?ooo`3oool00`3oool00`000000oooo000000050?ooo`030000003oool0oooo02T0oooo
00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo02T0oooo
00<000000?ooo`3oool01`3oool010000000oooo0?ooo`0000050?ooo`030000003oool0oooo02P0
oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo00P0
oooo001l0?ooo`030000003oool0oooo00@00000<P3oool4000000<0oooo00<000000?ooo`3oool0
0`3oool00`000000oooo0?ooo`0b0?ooo`030000003oool0oooo0080oooo1000000_0?ooo`030000
003oool0oooo0080oooo00<000000?ooo`3oool00`3oool00`000000oooo000000050?ooo`030000
003oool0oooo02T0oooo00<000000?ooo`3oool0203oool00`000000oooo0?ooo`04000002`0oooo
00<000000?ooo`3oool00`3oool4000000<0oooo00<000000?ooo`3oool00`3oool00`000000oooo
0?ooo`0X0?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool00P3oool4000000/0oooo
001l0?ooo`040000003oool0oooo000003D0oooo00<000000?ooo`3oool0103oool00`000000oooo
0?ooo`030?ooo`030000003oool0oooo03<0oooo00D000000?ooo`3oool0oooo0000000b0?ooo`03
0000003oool0oooo0080oooo00<000000?ooo`3oool00`3oool00`000000oooo000000050?ooo`03
0000003oool0oooo02T0oooo00<000000?ooo`3oool0203oool010000000oooo0?ooo`00000_0?oo
o`030000003oool0oooo00<0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`030?oo
o`030000003oool0oooo02P0oooo00<000000?ooo`3oool01`3oool01@000000oooo0?ooo`3oool0
000000h0oooo001f0?ooo`030000003oool0oooo00<0oooo00D000000?ooo`3oool0oooo0000000e
0?ooo`030000003oool0oooo00@0oooo00D000000?ooo`3oool0oooo0000000a0?ooo`030000003o
ool0oooo0080oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`0]0?ooo`030000003o
ool0000000D0oooo00D000000?ooo`3oool0oooo000000030?ooo`050000003oool0oooo0?ooo`00
0000:P3oool00`000000oooo000000040?ooo`030000003oool0oooo00<0oooo00D000000?ooo`3o
ool0oooo0000000/0?ooo`030000003oool0000000H0oooo00<000000?ooo`3oool0103oool01@00
0000oooo0?ooo`3oool0000002T0oooo00<000000?ooo`000000103oool00`000000oooo0?ooo`02
0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool02`3oool007L0oooo00D000000?oo
o`3oool0oooo000000040?ooo`030000003oool0oooo03<0oooo00<000000?ooo`3oool0103oool0
1@000000oooo0?ooo`3oool000000340oooo00<000000?ooo`3oool00`3oool01@000000oooo0?oo
o`3oool000000300oooo0P0000050?ooo`050000003oool0oooo0?ooo`0000000`3oool01@000000
oooo0?ooo`3oool0000002/0oooo0P0000050?ooo`050000003oool0oooo0?ooo`000000103oool0
0`000000oooo0?ooo`0[0?ooo`8000001P3oool00`000000oooo0?ooo`040?ooo`050000003oool0
oooo0?ooo`000000:P3oool2000000@0oooo00<000000?ooo`3oool00`3oool01@000000oooo0?oo
o`3oool0000000d0oooo001h0?ooo`<000001@3oool500000340oooo1@0000030?ooo`<00000<P3o
ool7000000<0oooo1@00000]0?ooo`030000003oool0oooo00@0oooo0`0000050?ooo`<00000;@3o
ool00`000000oooo0?ooo`040?ooo`<000001@3oool5000002X0oooo00<000000?ooo`3oool0103o
ool5000000<0oooo0`00000/0?ooo`030000003oool0oooo0080oooo1`0000030?ooo`D000002@3o
ool00?l0ooooo`3oool90?ooo`00o`3ooooo0?ooo`T0oooo003o0?ooool0oooo2@3oool00?l0oooo
o`3oool90?ooo`00o`3ooooo0?ooo`T0oooo00110?ooool00000``0000040?ooo`00@@3oool00`00
0000oooo0?ooo`090?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool02@3oool00`00
0000oooo0?ooo`0:0?ooo`030000003oool0oooo00T0oooo00<000000?ooo`3oool02P3oool00`00
0000oooo0?ooo`090?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool02@3oool00`00
0000oooo0?ooo`0:0?ooo`030000003oool0oooo00T0oooo00<000000?ooo`3oool02P3oool00`00
0000oooo0?ooo`090?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool02@3oool00`00
0000oooo0?ooo`0:0?ooo`030000003oool0oooo00T0oooo00<000000?ooo`3oool02P3oool00`00
0000oooo0?ooo`090?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool02@3oool00`00
0000oooo0?ooo`090?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool02@3oool00`00
0000oooo0?ooo`0:0?ooo`030000003oool0oooo00T0oooo00<000000?ooo`3oool02P3oool00`00
0000oooo0?ooo`090?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool02@3oool00`00
0000oooo0?ooo`0:0?ooo`030000003oool0oooo00T0oooo00<000000?ooo`3oool02P3oool00`00
0000oooo0?ooo`090?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool02@3oool00`00
0000oooo0?ooo`020?ooo`00@@3oool00`000000oooo0?ooo`090?ooo`030000003oool0oooo00X0
oooo00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo00T0
oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`090?ooo`030000003oool0oooo00X0
oooo00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo00T0
oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`090?ooo`030000003oool0oooo00X0
oooo00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo00T0
oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`090?ooo`030000003oool0oooo00X0
oooo00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`090?ooo`030000003oool0oooo00X0
oooo00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo00T0
oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`090?ooo`030000003oool0oooo00X0
oooo00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo00T0
oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`090?ooo`030000003oool0oooo00X0
oooo00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`020?ooo`00@@3oool00`000000oooo
0?ooo`0k0?ooo`030000003oool0oooo03`0oooo00<000000?ooo`3oool0>`3oool00`000000oooo
0?ooo`0l0?ooo`030000003oool0oooo03/0oooo00<000000?ooo`3oool0>`3oool00`000000oooo
0?ooo`0l0?ooo`030000003oool0oooo00T0oooo00<000000?ooo`3oool00P3oool00440oooo00<0
00000?ooo`3oool0o`3ooono0?ooo`030000003oool0oooo0080oooo00110?ooo`030000003oool0
oooo0?l0oooo_`3oool00`000000oooo0?ooo`020?ooo`00@@3oool00`000000oooo0?ooo`3o0?oo
okl0oooo00<000000?ooo`3oool00P3oool00440oooo00<000000?ooo`3oool0o`3ooono0?ooo`03
0000003oool0oooo0080oooo00110?ooo`800000o`3ooono0?ooo`800000103oool00440oooo00<0
00000?ooo`3oool0o`3ooono0?ooo`030000003oool0oooo0080oooo00110?ooo`030000003oool0
oooo0?l0oooo_`3oool00`000000oooo0?ooo`020?ooo`00@@3oool00`000000oooo0?ooo`3o0?oo
okl0oooo00<000000?ooo`3oool00P3oool00440oooo00<000000?ooo`3oool0o`3ooono0?ooo`03
0000003oool0oooo0080oooo00110?ooo`030000003oool0oooo0?l0oooo_`3oool00`000000oooo
0?ooo`020?ooo`00@@3oool00`000000oooo0?ooo`3o0?oookl0oooo00<000000?ooo`3oool00P3o
ool00440oooo00<000000?ooo`3oool0o`3ooono0?ooo`030000003oool0oooo0080oooo00110?oo
o`800000o`3ooono0?ooo`800000103oool00440oooo00<000000?ooo`3oool0o`3ooono0?ooo`03
0000003oool0oooo0080oooo00110?ooo`030000003oool0oooo0?l0oooo_`3oool00`000000oooo
0?ooo`020?ooo`00@@3oool00`000000oooo0?ooo`3o0?oookl0oooo00<000000?ooo`3oool00P3o
ool00440oooo00<000000?ooo`3oool0o`3ooono0?ooo`030000003oool0oooo0080oooo00110?oo
o`030000003oool0oooo0?l0oooo_`3oool00`000000oooo0?ooo`020?ooo`00@@3oool00`000000
oooo0?ooo`3o0?oookl0oooo00<000000?ooo`3oool00P3oool00440oooo0P00003o0?oookl0oooo
0P0000040?ooo`00@@3oool00`000000oooo0?ooo`3o0?oookl0oooo00<000000?ooo`3oool00P3o
ool00440oooo00<000000?ooo`3oool0o`3ooono0?ooo`030000003oool0oooo0080oooo00110?oo
o`030000003oool0oooo0?l0oooo_`3oool00`000000oooo0?ooo`020?ooo`00@@3oool00`000000
oooo0?ooo`3o0?oookl0oooo00<000000?ooo`3oool00P3oool00440oooo00<000000?ooo`3oool0
o`3ooono0?ooo`030000003oool0oooo0080oooo00110?ooo`030000003oool0oooo0?l0oooo_`3o
ool00`000000oooo0?ooo`020?ooo`00@@3oool00`000000oooo0?ooo`3o0?oookl0oooo00<00000
0?ooo`3oool00P3oool00440oooo0P00003o0?oookl0oooo0P0000040?ooo`00@@3oool00`000000
oooo0?ooo`3o0?oookl0oooo00<000000?ooo`3oool00P3oool00440oooo00<000000?ooo`3oool0
o`3ooono0?ooo`030000003oool0oooo0080oooo00110?ooo`030000003oool0oooo0?l0oooo_`3o
ool00`000000oooo0?ooo`020?ooo`00@@3oool00`000000oooo0?ooo`3o0?oookl0oooo00<00000
0?ooo`3oool00P3oool001D0oooo0`0000040?ooo`030000003oool0oooo00H000000`3oool30000
01H0oooo00<000000?ooo`3oool0o`3ooono0?ooo`030000003oool0oooo0080oooo000D0?ooo`05
0000003oool0oooo0?ooo`0000000`3oool01@000000oooo0?ooo`3oool0000000@0oooo00<00000
0?ooo`0000000`3oool2000001@0oooo00<000000?ooo`3oool0o`3ooono0?ooo`030000003oool0
oooo0080oooo000D0?ooo`050000003oool0oooo0?ooo`000000203oool00`000000oooo0?ooo`09
0?ooo`030000003oool0oooo0140oooo00<000000?ooo`3oool0o`3ooono0?ooo`030000003oool0
oooo0080oooo000C0?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool01P3oool00`00
0000oooo0?ooo`080?ooo`030000003oool0oooo0140oooo1000003o0?oook`0oooo0`0000040?oo
o`004`3oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3o
ool01`3oool00`000000oooo0?ooo`0A0?ooo`030000003oool0oooo0?l0oooo_`3oool00`000000
oooo0?ooo`020?ooo`004`3oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo00P0oooo
00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`0B0?ooo`030000003oool0oooo0?l0oooo
_`3oool00`000000oooo0?ooo`020?ooo`004`3oool00`000000oooo0?ooo`030?ooo`030000003o
ool0oooo00T0oooo00<000000?ooo`3oool01000000E0?ooo`030000003oool0oooo0?l0oooo_`3o
ool00`000000oooo0?ooo`020?ooo`004`3oool00`000000oooo0?ooo`030?ooo`030000003oool0
oooo00T0oooo00@000000?ooo`3oool00000603oool00`000000oooo0?ooo`3o0?oookl0oooo00<0
00000?ooo`3oool00P3oool001@0oooo00D000000?ooo`3oool0oooo000000060?ooo`030000003o
ool0oooo00<0oooo00D000000?ooo`3oool0oooo0000000G0?ooo`030000003oool0oooo0?l0oooo
_`3oool00`000000oooo0?ooo`020?ooo`00503oool01@000000oooo0?ooo`3oool0000000L0oooo
00D000000?ooo`3oool0oooo000000040?ooo`030000003oool0oooo01D0oooo00<000000?ooo`3o
ool0o`3ooono0?ooo`030000003oool0oooo0080oooo000E0?ooo`<000002@3oool3000000D0oooo
1@00000C0?ooo`800000o`3ooono0?ooo`800000103oool00440oooo00<000000?ooo`3oool0o`3o
oono0?ooo`030000003oool0oooo0080oooo00110?ooo`030000003oool0oooo0?l0oooo_`3oool0
0`000000oooo0?ooo`020?ooo`00@@3oool00`000000oooo0?ooo`3o0?oookl0oooo00<000000?oo
o`3oool00P3oool00440oooo00<000000?ooo`3oool0o`3ooono0?ooo`030000003oool0oooo0080
oooo00110?ooo`030000003oool0oooo0?l0oooo_`3oool00`000000oooo0?ooo`020?ooo`00@@3o
ool00`000000oooo0?ooo`3o0?oookl0oooo00<000000?ooo`3oool00P3oool00440oooo00<00000
0?ooo`3oool0o`3ooono0?ooo`030000003oool0oooo0080oooo00110?ooo`800000o`3ooono0?oo
o`800000103oool00440oooo00<000000?ooo`3oool0o`3ooono0?ooo`030000003oool0oooo0080
oooo00110?ooo`030000003oool0oooo0?l0oooo_`3oool00`000000oooo0?ooo`020?ooo`00@@3o
ool00`000000oooo0?ooo`3o0?oookl0oooo00<000000?ooo`3oool00P3oool00440oooo00<00000
0?ooo`3oool0o`3ooono0?ooo`030000003oool0oooo0080oooo00110?ooo`030000003oool0oooo
0?l0oooo_`3oool00`000000oooo0?ooo`020?ooo`00@@3oool00`000000oooo0?ooo`3o0?oookl0
oooo00<000000?ooo`3oool00P3oool00440oooo00<000000?ooo`3oool0o`3ooono0?ooo`030000
003oool0oooo0080oooo00110?ooo`800000o`3ooono0?ooo`800000103oool00440oooo00<00000
0?ooo`3oool0o`3ooono0?ooo`030000003oool0oooo0080oooo00110?ooo`030000003oool0oooo
0?l0oooo_`3oool00`000000oooo0?ooo`020?ooo`00@@3oool00`000000oooo0?ooo`3o0?oookl0
oooo00<000000?ooo`3oool00P3oool00440oooo00<000000?ooo`3oool0o`3ooono0?ooo`030000
003oool0oooo0080oooo00110?ooo`030000003oool0oooo0?l0oooo_`3oool00`000000oooo0?oo
o`020?ooo`00@@3oool00`000000oooo0?ooo`3o0?oookl0oooo00<000000?ooo`3oool00P3oool0
0440oooo00<000000?ooo`3oool0o`3ooono0?ooo`030000003oool0oooo0080oooo00110?ooo`80
0000o`3ooono0?ooo`800000103oool00440oooo00<000000?ooo`3oool0o`3ooono0?ooo`030000
003oool0oooo0080oooo00110?ooo`030000003oool0oooo0?l0oooo_`3oool00`000000oooo0?oo
o`020?ooo`00@@3oool00`000000oooo0?ooo`3o0?oookl0oooo00<000000?ooo`3oool00P3oool0
01h0oooo0`0000040?ooo`040000003oool0oooo0?ooo`<000005@3oool00`000000oooo0?ooo`3o
0?oookl0oooo00<000000?ooo`3oool00P3oool001d0oooo00D000000?ooo`3oool0oooo00000003
0?ooo`040000003oool0oooo000000<0oooo0P00000C0?ooo`030000003oool0oooo0?l0oooo_`3o
ool00`000000oooo0?ooo`020?ooo`007@3oool01@000000oooo0?ooo`3oool0000000`0oooo00<0
00000?ooo`3oool0403oool00`000000oooo0?ooo`3o0?oookl0oooo00<000000?ooo`3oool00P3o
ool001`0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`090?ooo`030000003oool0
oooo0100oooo1000003o0?oook`0oooo0`0000040?ooo`00703oool00`000000oooo0?ooo`030?oo
o`030000003oool0oooo00T0oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`3o0?oo
okl0oooo00<000000?ooo`3oool00P3oool001`0oooo00<000000?ooo`3oool00`3oool00`000000
oooo0?ooo`080?ooo`030000003oool0oooo0140oooo00<000000?ooo`3oool0o`3ooono0?ooo`03
0000003oool0oooo0080oooo000L0?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool0
103oool4000001@0oooo00<000000?ooo`3oool0o`3ooono0?ooo`030000003oool0oooo0080oooo
000L0?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool0103oool00`000000oooo0?oo
o`0E0?ooo`030000003oool0oooo0?l0oooo_`3oool00`000000oooo0?ooo`020?ooo`007@3oool0
1@000000oooo0?ooo`3oool0000000P0oooo00<000000?ooo`3oool0503oool00`000000oooo0?oo
o`3o0?oookl0oooo00<000000?ooo`3oool00P3oool001d0oooo00D000000?ooo`3oool0oooo0000
00080?ooo`030000003oool0oooo01@0oooo00<000000?ooo`3oool0o`3ooono0?ooo`030000003o
ool0oooo0080oooo000N0?ooo`<000002@3oool500000180oooo00<000000?ooo`3oool0o`3ooono
0?ooo`030000003oool0oooo0080oooo00110?ooo`800000o`3ooono0?ooo`800000103oool00440
oooo00<000000?ooo`3oool0o`3ooono0?ooo`030000003oool0oooo0080oooo00110?ooo`030000
003oool0oooo0?l0oooo_`3oool00`000000oooo0?ooo`020?ooo`00@@3oool00`000000oooo0?oo
o`3o0?oookl0oooo00<000000?ooo`3oool00P3oool00440oooo00<000000?ooo`3oool0o`3ooono
0?ooo`030000003oool0oooo0080oooo00110?ooo`030000003oool0oooo0?l0oooo_`3oool00`00
0000oooo0?ooo`020?ooo`00@@3oool00`000000oooo0?ooo`3o0?oookl0oooo00<000000?ooo`3o
ool00P3oool00440oooo00<000000?ooo`3oool0o`3ooono0?ooo`030000003oool0oooo0080oooo
00110?ooo`800000o`3ooono0?ooo`800000103oool00440oooo00<000000?ooo`3oool0o`3ooono
0?ooo`030000003oool0oooo0080oooo00110?ooo`030000003oool0oooo0?l0oooo_`3oool00`00
0000oooo0?ooo`020?ooo`00@@3oool00`000000oooo0?ooo`3o0?oookl0oooo00<000000?ooo`3o
ool00P3oool00440oooo00<000000?ooo`3oool0o`3ooono0?ooo`030000003oool0oooo0080oooo
00110?ooo`030000003oool0oooo0?l0oooo_`3oool00`000000oooo0?ooo`020?ooo`00@@3oool0
0`000000oooo0?ooo`3o0?oookl0oooo00<000000?ooo`3oool00P3oool00440oooo0P00003o0?oo
okl0oooo0P0000040?ooo`00@@3oool00`000000oooo0?ooo`3o0?oookl0oooo00<000000?ooo`3o
ool00P3oool00440oooo00<000000?ooo`3oool0o`3ooono0?ooo`030000003oool0oooo0080oooo
00110?ooo`030000003oool0oooo0?l0oooo_`3oool00`000000oooo0?ooo`020?ooo`00@@3oool0
0`000000oooo0?ooo`3o0?oookl0oooo00<000000?ooo`3oool00P3oool00440oooo00<000000?oo
o`3oool0o`3ooono0?ooo`030000003oool0oooo0080oooo00110?ooo`030000003oool0oooo0?l0
oooo_`3oool00`000000oooo0?ooo`020?ooo`00@@3oool00`000000oooo0?ooo`3o0?oookl0oooo
00<000000?ooo`3oool00P3oool00440oooo0P00003o0?oookl0oooo0P0000040?ooo`00@@3oool0
0`000000oooo0?ooo`3o0?oookl0oooo00<000000?ooo`3oool00P3oool00440oooo00<000000?oo
o`3oool0o`3ooono0?ooo`030000003oool0oooo0080oooo00110?ooo`030000003oool0oooo0?l0
oooo_`3oool00`000000oooo0?ooo`020?ooo`00@@3oool00`000000oooo0?ooo`3o0?oookl0oooo
00<000000?ooo`3oool00P3oool001D0oooo0`0000040?ooo`030000003oool0oooo0080oooo00<0
00000?ooo`3oool0103oool3000001H0oooo00<000000?ooo`3oool0o`3ooono0?ooo`030000003o
ool0oooo0080oooo000D0?ooo`050000003oool0oooo0?ooo`0000000`3oool00`000000oooo0?oo
o`020?ooo`030000003oool0oooo00<0oooo00@000000?ooo`3oool0oooo0P00000D0?ooo`030000
003oool0oooo0?l0oooo_`3oool00`000000oooo0?ooo`020?ooo`00503oool01@000000oooo0?oo
o`3oool0000000T0oooo00<000000?ooo`3oool0203oool00`000000oooo0?ooo`0A0?ooo`030000
003oool0oooo0?l0oooo_`3oool00`000000oooo0?ooo`020?ooo`004`3oool00`000000oooo0?oo
o`030?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool0203oool00`000000oooo0?oo
o`0A0?ooo`@00000o`3ooonl0?ooo`<00000103oool001<0oooo00<000000?ooo`3oool00`3oool0
0`000000oooo0?ooo`060?ooo`030000003oool0oooo00P0oooo00<000000?ooo`3oool0403ooooS
0?l00=l0oooo00<000000?ooo`3oool00P3oool001<0oooo00<000000?ooo`3oool00`3oool00`00
0000oooo0?ooo`070?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool04@3ooooo0?l0
0840o`00@P3oool00`000000oooo0?ooo`020?ooo`004`3oool00`000000oooo0?ooo`030?ooo`03
0000003oool0oooo00L0oooo00<000000?ooo`3oool00P3oool4000001D0oooo00<000000?ooo`3o
ool0gP3ooooQ0?l000@0oooo000C0?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool0
203oool01@000000oooo0?ooo`3oool0000001P0oooo00<000000?ooo`3oool0o`3oooml0?oood@0
o`00103oool001@0oooo00D000000?ooo`3oool0oooo000000060?ooo`030000003oool0oooo0080
oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`0E0?ooo`030000003oool0oooo0?l0
oooo_`3oool00`000000oooo0?ooo`020?ooo`00503oool01@000000oooo0?ooo`3oool0000000H0
oooo00<000000?ooo`3oool00`3oool01@000000oooo0?ooo`3oool0000001L0oooo00<000000?oo
o`3oool0o`3ooono0?ooo`030000003oool0oooo0080oooo000E0?ooo`<000001`3oool7000000<0
oooo1@00000C0?ooo`030000003oool0oooo0?l0oooo_`3oool00`000000oooo0?ooo`020?ooo`00
@@3oool200000?l0oooo_`3oool2000000@0oooo00110?ooo`030000003oool0oooo0?l0oooo_`3o
ool00`000000oooo0?ooo`020?ooo`00@@3oool00`000000oooo0?ooo`3o0?oookl0oooo00<00000
0?ooo`3oool00P3oool00440oooo00<000000?ooo`3oool0o`3ooono0?ooo`030000003oool0oooo
0080oooo00110?ooo`030000003oool0oooo0?l0oooo_`3oool00`000000oooo0?ooo`020?ooo`00
@@3oool00`000000oooo0?ooo`3o0?oookl0oooo00<000000?ooo`3oool00P3oool000/0oooo0`00
00040?ooo`<00000;03oool00`000000oooo0?ooo`3o0?oookl0oooo00<000000?ooo`3oool00P3o
ool000`0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`0/0?ooo`800000o`3ooono
0?ooo`800000103oool000`0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`0]0?oo
o`030000003oool0oooo0?l0oooo_`3oool00`000000oooo0?ooo`020?ooo`00303oool01@000000
oooo0?ooo`3oool000000300oooo00<000000?ooo`3oool0o`3ooono0?ooo`030000003oool0oooo
0080oooo000<0?ooo`040000003oool0oooo00000340oooo00<000000?ooo`3oool0o`3ooono0?oo
o`030000003oool0oooo0080oooo000<0?ooo`D00000<03oool00`000000oooo0?ooo`3o0?oookl0
oooo00<000000?ooo`3oool00P3oool000`0oooo00<000000?ooo`3oool00P3oool00`000000oooo
0?ooo`0]0?ooo`030000003oool0oooo0?l0oooo_`3oool00`000000oooo0?ooo`020?ooo`00303o
ool00`000000oooo0?ooo`030?ooo`030000003oool0oooo02`0oooo00<000000?ooo`3oool0o`3o
oono0?ooo`030000003oool0oooo0080oooo000<0?ooo`030000003oool0oooo00<0oooo00<00000
0?ooo`3oool0;03oool00`000000oooo0?ooo`3o0?oookl0oooo00<000000?ooo`3oool00P3oool0
00`0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`0]0?ooo`800000o`3ooono0?oo
o`800000103oool000/0oooo1P00000`0?ooo`030000003oool0oooo0?l0oooo_`3oool00`000000
oooo0?ooo`020?ooo`003P3oool4000002l0oooo00<000000?ooo`3oool0o`3ooono0?ooo`030000
003oool0oooo0080oooo000<0?ooo`800000103oool2000002d0oooo00<000000?ooo`3oool0o`3o
oono0?ooo`030000003oool0oooo0080oooo000<0?ooo`030000003oool0oooo00D0oooo00<00000
0?ooo`3oool0:P3oool00`000000oooo0?ooo`3o0?oookl0oooo00<000000?ooo`3oool00P3oool0
00/0oooo00<000000?ooo`3oool0<`3oool00`000000oooo0?ooo`3o0?oookl0oooo00<000000?oo
o`3oool00P3oool000/0oooo00<000000?ooo`3oool0<`3oool00`000000oooo0?ooo`3o0?oookl0
oooo00<000000?ooo`3oool00P3oool000/0oooo00<000000?ooo`3oool0<`3oool00`000000oooo
0?ooo`3o0?oookl0oooo00<000000?ooo`3oool00P3oool000/0oooo00<000000?ooo`3oool0<`3o
ool200000?l0oooo_`3oool2000000@0oooo000;0?ooo`030000003oool0oooo00H0oooo00<00000
0?ooo`3oool0:P3oool00`000000oooo0?ooo`3o0?oookl0oooo00<000000?ooo`3oool00P3oool0
00`0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`0Z0?ooo`030000003oool0oooo
0?l0oooo_`3oool00`000000oooo0?ooo`020?ooo`00303oool2000000D0oooo0P00000/0?ooo`03
0000003oool0oooo0?l0oooo_`3oool00`000000oooo0?ooo`020?ooo`003P3oool5000000030?oo
o`000000oooo02/0oooo00<000000?ooo`3oool0o`3ooono0?ooo`030000003oool0oooo0080oooo
000b0?ooo`<00000303oool00`000000oooo0?ooo`3o0?oookl0oooo00<000000?ooo`3oool00P3o
ool003<0oooo00<000000?ooo`3oool02`3oool00`000000oooo0?ooo`3o0?oookl0oooo00<00000
0?ooo`3oool00P3oool003<0oooo00<000000?ooo`3oool02`3oool400000?l0oooo_03oool30000
00@0oooo000c0?ooo`030000003oool0oooo00/0oooo00<000000?ooo`3oool0o`3ooono0?ooo`03
0000003oool0oooo0080oooo000c0?ooo`030000003oool0oooo00/0oooo00<000000?ooo`3oool0
o`3ooono0?ooo`030000003oool0oooo0080oooo000c0?ooo`030000003oool0oooo00/0oooo00<0
00000?ooo`3oool0o`3ooono0?ooo`030000003oool0oooo0080oooo000c0?ooo`030000003oool0
oooo00/0oooo00<000000?ooo`3oool0o`3ooono0?ooo`030000003oool0oooo0080oooo000c0?oo
o`030000003oool0oooo00/0oooo00<000000?ooo`3oool0o`3ooono0?ooo`030000003oool0oooo
0080oooo000a0?ooo`030000003oool0000000d0oooo00<000000?ooo`3oool0o`3ooono0?ooo`03
0000003oool0oooo0080oooo000b0?ooo`8000003@3oool00`000000oooo0?ooo`3o0?oookl0oooo
00<000000?ooo`3oool00P3oool003<0oooo00<000000?ooo`3oool02`3oool200000?l0oooo_`3o
ool2000000@0oooo00110?ooo`030000003oool0oooo0?l0oooo_`3oool00`000000oooo0?ooo`02
0?ooo`00@@3oool00`000000oooo0?ooo`3o0?oookl0oooo00<000000?ooo`3oool00P3oool00440
oooo00<000000?ooo`3oool0o`3ooono0?ooo`030000003oool0oooo0080oooo00110?ooo`030000
003oool0oooo0?l0oooo_`3oool00`000000oooo0?ooo`020?ooo`00@@3oool00`000000oooo0?oo
o`3o0?oookl0oooo00<000000?ooo`3oool00P3oool00440oooo00<000000?ooo`3oool0o`3ooono
0?ooo`030000003oool0oooo0080oooo00110?ooo`030000003oool0oooo0?l0oooo_`3oool00`00
0000oooo0?ooo`020?ooo`00@@3oool200000?l0oooo_`3oool2000000@0oooo00110?ooo`030000
003oool0oooo0?l0oooo_`3oool00`000000oooo0?ooo`020?ooo`00@@3oool00`000000oooo0?oo
o`3o0?oookl0oooo00<000000?ooo`3oool00P3oool00440oooo00<000000?ooo`3oool0o`3ooono
0?ooo`030000003oool0oooo0080oooo00110?ooo`030000003oool0oooo0?l0oooo_`3oool00`00
0000oooo0?ooo`020?ooo`00@@3oool00`000000oooo0?ooo`3o0?oookl0oooo00<000000?ooo`3o
ool00P3oool00440oooo00<000000?ooo`3oool0o`3ooono0?ooo`030000003oool0oooo0080oooo
00110?ooo`030000003oool0oooo0?l0oooo_`3oool00`000000oooo0?ooo`020?ooo`00@@3oool2
00000?l0oooo_`3oool2000000@0oooo00110?ooo`030000003oool0oooo0?l0oooo_`3oool00`00
0000oooo0?ooo`020?ooo`00@@3oool00`000000oooo0?ooo`3o0?oookl0oooo00<000000?ooo`3o
ool00P3oool00440oooo00<000000?ooo`3oool0o`3ooono0?ooo`030000003oool0oooo0080oooo
00110?ooo`030000003oool0oooo0?l0oooo_`3oool00`000000oooo0?ooo`020?ooo`00@@3oool0
0`000000oooo0?ooo`3o0?oookl0oooo00<000000?ooo`3oool00P3oool00440oooo00<000000?oo
o`3oool0o`3ooono0?ooo`030000003oool0oooo0080oooo00110?ooo`800000o`3ooono0?ooo`80
0000103oool00440oooo00<000000?ooo`3oool0o`3ooono0?ooo`030000003oool0oooo0080oooo
00110?ooo`030000003oool0oooo0?l0oooo_`3oool00`000000oooo0?ooo`020?ooo`00@@3oool0
0`000000oooo0?ooo`3o0?oookl0oooo00<000000?ooo`3oool00P3oool00440oooo00<000000?oo
o`3oool0o`3ooono0?ooo`030000003oool0oooo0080oooo000E0?ooo`<00000103oool00`000000
oooo0?ooo`06000000<0oooo0`00000F0?ooo`030000003oool0oooo0?l0oooo_`3oool00`000000
oooo0?ooo`020?ooo`005P3oool00`000000oooo0?ooo`030?ooo`050000003oool0oooo0?ooo`00
0000103oool00`000000oooo000000030?ooo`800000503oool00`000000oooo0?ooo`3o0?oookl0
oooo00<000000?ooo`3oool00P3oool001H0oooo00<000000?ooo`3oool0203oool00`000000oooo
0?ooo`090?ooo`030000003oool0oooo0140oooo00<000000?ooo`3oool0o`3ooono0?ooo`030000
003oool0oooo0080oooo000F0?ooo`030000003oool0oooo00T0oooo00<000000?ooo`3oool0203o
ool00`000000oooo0?ooo`0A0?ooo`@00000o`3ooonl0?ooo`<00000103oool001H0oooo00<00000
0?ooo`3oool02P3oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo0140oooo00<00000
0?ooo`3oool0o`3ooono0?ooo`030000003oool0oooo0080oooo000F0?ooo`030000003oool0oooo
00/0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`0B0?ooo`030000003oool0oooo
0?l0oooo_`3oool00`000000oooo0?ooo`020?ooo`005P3oool00`000000oooo0?ooo`0<0?ooo`03
0000003oool0oooo00@000005@3oool00`000000oooo0?ooo`3o0?oookl0oooo00<000000?ooo`3o
ool00P3oool001H0oooo00<000000?ooo`3oool0303oool010000000oooo0?ooo`00000H0?ooo`03
0000003oool0oooo0?l0oooo_`3oool00`000000oooo0?ooo`020?ooo`00503oool00`000000oooo
000000080?ooo`030000003oool0oooo00<0oooo00D000000?ooo`3oool0oooo0000000G0?ooo`03
0000003oool0oooo0?l0oooo_`3oool00`000000oooo0?ooo`020?ooo`005@3oool2000000T0oooo
00D000000?ooo`3oool0oooo000000040?ooo`030000003oool0oooo01D0oooo00<000000?ooo`3o
ool0o`3ooono0?ooo`030000003oool0oooo0080oooo000F0?ooo`030000003oool0oooo00P0oooo
0`0000050?ooo`D000004`3oool00`000000oooo0?ooo`3o0?oookl0oooo00<000000?ooo`3oool0
0P3oool00440oooo0P00003o0?oookl0oooo0P0000040?ooo`00@@3oool00`000000oooo0?ooo`3o
0?oookl0oooo00<000000?ooo`3oool00P3oool00440oooo00<000000?ooo`3oool0o`3ooono0?oo
o`030000003oool0oooo0080oooo00110?ooo`030000003oool0oooo0?l0oooo_`3oool00`000000
oooo0?ooo`020?ooo`00@@3oool00`000000oooo0?ooo`3o0?oookl0oooo00<000000?ooo`3oool0
0P3oool00440oooo00<000000?ooo`3oool0o`3ooono0?ooo`030000003oool0oooo0080oooo0011
0?ooo`030000003oool0oooo0?l0oooo_`3oool00`000000oooo0?ooo`020?ooo`00@@3oool20000
0?l0oooo_`3oool2000000@0oooo00110?ooo`030000003oool0oooo0?l0oooo_`3oool00`000000
oooo0?ooo`020?ooo`00@@3oool00`000000oooo0?ooo`3o0?oookl0oooo00<000000?ooo`3oool0
0P3oool00440oooo00<000000?ooo`3oool0o`3ooono0?ooo`030000003oool0oooo0080oooo0011
0?ooo`030000003oool0oooo0?l0oooo_`3oool00`000000oooo0?ooo`020?ooo`00@@3oool00`00
0000oooo0?ooo`3o0?oookl0oooo00<000000?ooo`3oool00P3oool00440oooo00<000000?ooo`3o
ool0o`3ooono0?ooo`030000003oool0oooo0080oooo00110?ooo`030000003oool0oooo0?l0oooo
_`3oool00`000000oooo0?ooo`020?ooo`00@@3oool200000?l0oooo_`3oool2000000@0oooo0011
0?ooo`030000003oool0oooo0?l0oooo_`3oool00`000000oooo0?ooo`020?ooo`00@@3oool00`00
0000oooo0?ooo`3o0?oookl0oooo00<000000?ooo`3oool00P3oool00440oooo00<000000?ooo`3o
ool0o`3ooono0?ooo`030000003oool0oooo0080oooo00110?ooo`030000003oool0oooo0?l0oooo
_`3oool00`000000oooo0?ooo`020?ooo`00@@3oool00`000000oooo0?ooo`3o0?oookl0oooo00<0
00000?ooo`3oool00P3oool00440oooo00<000000?ooo`3oool0o`3ooono0?ooo`030000003oool0
oooo0080oooo00110?ooo`030000003oool0oooo0?l0oooo_`3oool00`000000oooo0?ooo`020?oo
o`00@@3oool200000?l0oooo_`3oool2000000@0oooo00110?ooo`030000003oool0oooo0?l0oooo
_`3oool00`000000oooo0?ooo`020?ooo`00@@3oool00`000000oooo0?ooo`3o0?oookl0oooo00<0
00000?ooo`3oool00P3oool00440oooo00<000000?ooo`3oool0o`3ooono0?ooo`030000003oool0
oooo0080oooo00110?ooo`030000003oool0oooo0?l0oooo_`3oool00`000000oooo0?ooo`020?oo
o`007P3oool3000000@0oooo00@000000?ooo`3oool0oooo0`00000E0?ooo`030000003oool0oooo
0?l0oooo_`3oool00`000000oooo0?ooo`020?ooo`007`3oool00`000000oooo0?ooo`030?ooo`04
0000003oool0oooo000000<0oooo0P00000C0?ooo`030000003oool0oooo0?l0oooo_`3oool00`00
0000oooo0?ooo`020?ooo`007`3oool00`000000oooo0?ooo`0<0?ooo`030000003oool0oooo0100
oooo00<000000?ooo`3oool0o`3ooono0?ooo`030000003oool0oooo0080oooo000O0?ooo`030000
003oool0oooo00`0oooo00<000000?ooo`3oool0403oool400000?l0oooo_03oool3000000@0oooo
000O0?ooo`030000003oool0oooo00`0oooo00<000000?ooo`3oool0403oool00`000000oooo0?oo
o`3o0?oookl0oooo00<000000?ooo`3oool00P3oool001l0oooo00<000000?ooo`3oool02`3oool0
0`000000oooo0?ooo`0A0?ooo`030000003oool0oooo0?l0oooo_`3oool00`000000oooo0?ooo`02
0?ooo`007`3oool00`000000oooo0?ooo`070?ooo`@00000503oool00`000000oooo0?ooo`3o0?oo
okl0oooo00<000000?ooo`3oool00P3oool001l0oooo00<000000?ooo`3oool01`3oool00`000000
oooo0?ooo`0E0?ooo`030000003oool0oooo0?l0oooo_`3oool00`000000oooo0?ooo`020?ooo`00
7@3oool00`000000oooo0000000:0?ooo`030000003oool0oooo01@0oooo00<000000?ooo`3oool0
o`3ooono0?ooo`030000003oool0oooo0080oooo000N0?ooo`8000002P3oool00`000000oooo0?oo
o`0D0?ooo`030000003oool0oooo0?l0oooo_`3oool00`000000oooo0?ooo`020?ooo`007`3oool0
0`000000oooo0?ooo`080?ooo`D000004P3oool200000?l0oooo_`3oool2000000@0oooo00110?oo
o`030000003oool0oooo0?l0oooo_`3oool00`000000oooo0?ooo`020?ooo`00@@3oool00`000000
oooo0?ooo`3o0?oookl0oooo00<000000?ooo`3oool00P3oool00440oooo00<000000?ooo`3oool0
o`3ooono0?ooo`030000003oool0oooo0080oooo00110?ooo`030000003oool0oooo0?l0oooo_`3o
ool00`000000oooo0?ooo`020?ooo`00@@3oool00`000000oooo0?ooo`3o0?oookl0oooo00<00000
0?ooo`3oool00P3oool00440oooo00<000000?ooo`3oool0o`3ooono0?ooo`030000003oool0oooo
0080oooo00110?ooo`030000003oool0oooo0?l0oooo_`3oool00`000000oooo0?ooo`020?ooo`00
@@3oool200000?l0oooo_`3oool2000000@0oooo00110?ooo`030000003oool0oooo0?l0oooo_`3o
ool00`000000oooo0?ooo`020?ooo`00@@3oool00`000000oooo0?ooo`3o0?oookl0oooo00<00000
0?ooo`3oool00P3oool00440oooo00<000000?ooo`3oool0o`3ooono0?ooo`030000003oool0oooo
0080oooo00110?ooo`030000003oool0oooo0?l0oooo_`3oool00`000000oooo0?ooo`020?ooo`00
@@3oool00`000000oooo0?ooo`3o0?oookl0oooo00<000000?ooo`3oool00P3oool00440oooo00<0
00000?ooo`3oool0o`3ooono0?ooo`030000003oool0oooo0080oooo00110?ooo`030000003oool0
oooo0?l0oooo_`3oool00`000000oooo0?ooo`020?ooo`00@@3oool200000?l0oooo_`3oool20000
00@0oooo00110?ooo`030000003oool0oooo0?l0oooo_`3oool00`000000oooo0?ooo`020?ooo`00
@@3oool00`000000oooo0?ooo`3o0?oookl0oooo00<000000?ooo`3oool00P3oool00440oooo00<0
00000?ooo`3oool0o`3ooono0?ooo`030000003oool0oooo0080oooo00110?ooo`030000003oool0
oooo0?l0oooo_`3oool00`000000oooo0?ooo`020?ooo`00@@3oool00`000000oooo0?ooo`3o0?oo
okl0oooo00<000000?ooo`3oool00P3oool00440oooo00<000000?ooo`3oool0o`3ooono0?ooo`03
0000003oool0oooo0080oooo00110?ooo`800000o`3ooono0?ooo`800000103oool00440oooo00<0
00000?ooo`3oool0o`3ooono0?ooo`030000003oool0oooo0080oooo00110?ooo`030000003oool0
oooo0?l0oooo_`3oool00`000000oooo0?ooo`020?ooo`00@@3oool00`000000oooo0?ooo`3o0?oo
okl0oooo00<000000?ooo`3oool00P3oool00440oooo00<000000?ooo`3oool0o`3ooono0?ooo`03
0000003oool0oooo0080oooo000E0?ooo`<00000103oool00`000000oooo0?ooo`020?ooo`030000
003oool0oooo00@0oooo0`00000F0?ooo`030000003oool0oooo0?l0oooo_`3oool00`000000oooo
0?ooo`020?ooo`005P3oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo0080oooo00<0
00000?ooo`3oool00`3oool010000000oooo0?ooo`3oool2000001@0oooo00<000000?ooo`3oool0
o`3ooono0?ooo`030000003oool0oooo0080oooo000F0?ooo`030000003oool0oooo00T0oooo00<0
00000?ooo`3oool0203oool00`000000oooo0?ooo`0A0?ooo`030000003oool0oooo0?l0oooo_`3o
ool00`000000oooo0?ooo`020?ooo`005P3oool00`000000oooo0?ooo`090?ooo`030000003oool0
oooo00P0oooo00<000000?ooo`3oool04@3oool400000?l0oooo_03oool3000000@0oooo000F0?oo
o`030000003oool0oooo00T0oooo00<000000?ooo`3oool0203oool00`000000oooo0?ooo`0A0?oo
o`030000003oool0oooo0?l0oooo_`3oool00`000000oooo0?ooo`020?ooo`005P3oool00`000000
oooo0?ooo`0:0?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool04P3oool00`000000
oooo0?ooo`3o0?oookl0oooo00<000000?ooo`3oool00P3oool001H0oooo00<000000?ooo`3oool0
2P3oool00`000000oooo0?ooo`020?ooo`@000005@3oool00`000000oooo0?ooo`3o0?oookl0oooo
00<000000?ooo`3oool00P3oool001H0oooo00<000000?ooo`3oool02`3oool01@000000oooo0?oo
o`3oool0000001P0oooo00<000000?ooo`3oool0o`3ooono0?ooo`030000003oool0oooo0080oooo
000D0?ooo`030000003oool0000000P0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?oo
o`020?ooo`030000003oool0oooo01D0oooo00<000000?ooo`3oool0>`3oool00`000000oooo0?oo
o`0l0?ooo`030000003oool0oooo03/0oooo00<000000?ooo`3oool0?03oool00`000000oooo0?oo
o`0k0?ooo`030000003oool0oooo03/0oooo00<000000?ooo`3oool0?03oool00`000000oooo0?oo
o`090?ooo`030000003oool0oooo0080oooo000E0?ooo`800000203oool00`000000oooo0?ooo`03
0?ooo`050000003oool0oooo0?ooo`0000005`3oool00`000000oooo0?ooo`090?ooo`030000003o
ool0oooo00X0oooo00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`0:0?ooo`030000003o
ool0oooo00T0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`090?ooo`030000003o
ool0oooo00X0oooo00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`0:0?ooo`030000003o
ool0oooo00T0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`090?ooo`030000003o
ool0oooo00X0oooo00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`0:0?ooo`030000003o
ool0oooo00T0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`090?ooo`030000003o
ool0oooo00X0oooo00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`090?ooo`030000003o
ool0oooo00X0oooo00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`0:0?ooo`030000003o
ool0oooo00T0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`090?ooo`030000003o
ool0oooo00X0oooo00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`0:0?ooo`030000003o
ool0oooo00T0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`090?ooo`030000003o
ool0oooo00X0oooo00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`020?ooo`005P3oool0
0`000000oooo0?ooo`060?ooo`L000000`3oool5000001<0oooo00<000000?ooo`3oool02@3oool0
0`000000oooo0?ooo`0:0?ooo`030000003oool0oooo00T0oooo00<000000?ooo`3oool02P3oool0
0`000000oooo0?ooo`090?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool02@3oool0
0`000000oooo0?ooo`0:0?ooo`030000003oool0oooo00T0oooo00<000000?ooo`3oool02P3oool0
0`000000oooo0?ooo`090?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool02@3oool0
0`000000oooo0?ooo`0:0?ooo`030000003oool0oooo00T0oooo00<000000?ooo`3oool02P3oool0
0`000000oooo0?ooo`090?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool02@3oool0
0`000000oooo0?ooo`0:0?ooo`030000003oool0oooo00T0oooo00<000000?ooo`3oool02@3oool0
0`000000oooo0?ooo`0:0?ooo`030000003oool0oooo00T0oooo00<000000?ooo`3oool02P3oool0
0`000000oooo0?ooo`090?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool02@3oool0
0`000000oooo0?ooo`0:0?ooo`030000003oool0oooo00T0oooo00<000000?ooo`3oool02P3oool0
0`000000oooo0?ooo`090?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool02@3oool0
0`000000oooo0?ooo`0:0?ooo`030000003oool0oooo00T0oooo00<000000?ooo`3oool00P3oool0
0440ooooo`000033000000@0oooo003o0?ooool0oooo2@3oool00?l0ooooo`3oool90?ooo`00o`3o
oooo0?ooo`T0oooo0000\
\>"],
  ImageRangeCache->{{{91.5625, 506.312}, {590.438, 334.5}} -> {-71.9836, \
2.45933, 0.400592, 0.00648173}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]]
},
FrontEndVersion->"5.0 for X",
ScreenRectangle->{{0, 1600}, {0, 1200}},
WindowSize->{805, 901},
WindowMargins->{{262, Automatic}, {Automatic, 36}},
PrintingPageRange->{Automatic, Automatic},
PrintingOptions->{"PaperSize"->{612, 792},
"PaperOrientation"->"Portrait",
"PostScriptOutputFile":>FrontEnd`FileName[{$RootDirectory, "home", \
"gilfoyle", "research", "eod", "exclurad", "wvo", "plots"}, \
"RCsurface.nb.ps", CharacterEncoding -> "iso8859-1"],
"Magnification"->1}
]

(*******************************************************************
Cached data follows.  If you edit this Notebook file directly, not
using Mathematica, you must remove the line containing CacheID at
the top of  the file.  The cache data will then be recreated when
you save this file from within Mathematica.
*******************************************************************)

(*CellTagsOutline
CellTagsIndex->{}
*)

(*CellTagsIndex
CellTagsIndex->{}
*)

(*NotebookFileOutline
Notebook[{
Cell[1754, 51, 147, 3, 32, "Text"],

Cell[CellGroupData[{
Cell[1926, 58, 170, 3, 59, "Input"],
Cell[2099, 63, 47, 1, 27, "Output"]
}, Open  ]],
Cell[2161, 67, 150, 3, 32, "Text"],

Cell[CellGroupData[{
Cell[2336, 74, 301, 6, 75, "Input"],
Cell[2640, 82, 37, 1, 27, "Output"]
}, Open  ]],
Cell[2692, 86, 59, 0, 32, "Text"],
Cell[2754, 88, 342, 6, 59, "Input"],

Cell[CellGroupData[{
Cell[3121, 98, 827, 14, 164, "Input"],
Cell[3951, 114, 202338, 4823, 512, 63426, 3105, "GraphicsData", "PostScript", \
"Graphics"],
Cell[206292, 4939, 132, 3, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[206461, 4947, 63, 1, 27, "Input"],
Cell[206527, 4950, 48, 1, 27, "Output"]
}, Open  ]],
Cell[206590, 4954, 154, 3, 32, "Text"],

Cell[CellGroupData[{
Cell[206769, 4961, 703, 15, 160, "Input"],
Cell[207475, 4978, 53, 1, 27, "Output"],
Cell[207531, 4981, 53, 1, 27, "Output"],
Cell[207587, 4984, 53, 1, 27, "Output"]
}, Open  ]],
Cell[207655, 4988, 100, 3, 32, "Text"],

Cell[CellGroupData[{
Cell[207780, 4995, 519, 10, 142, "Input"],
Cell[208302, 5007, 50615, 1308, 330, 13481, 845, "GraphicsData", \
"PostScript", "Graphics"],
Cell[258920, 6317, 130, 3, 27, "Output"]
}, Open  ]]
}
]
*)



(*******************************************************************
End of Mathematica Notebook file.
*******************************************************************)

